<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link        https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @var      Integration_With_Salesforce_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'INTEGRATION_WITH_SALESFORCE_VERSION' ) ) {
			$this->version = INTEGRATION_WITH_SALESFORCE_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'integration-with-salesforce';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_ajax_callback();
		$this->initialize_cpt();
		$this->define_woo_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Integration_With_Salesforce_Loader. Orchestrates the hooks of the plugin.
	 * - Integration_With_Salesforce_I18n. Defines internationalization functionality.
	 * - Integration_With_Salesforce_Admin. Defines all hooks for the admin area.
	 * - Integration_With_Salesforce_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 */
	private function load_dependencies() {

		/**
		 * The file reponsible for defining all gobal functions used in plugin.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'woo-includes/woo-functions.php';

		/**
		 * The classes responsible for Framework.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'woo-crm-fw/woo/class-integration-with-salesforce-framework-base.php';
		require_once plugin_dir_path( __DIR__ ) . 'woo-crm-fw/woo/class-integration-with-salesforce-request-module.php';
		require_once plugin_dir_path( __DIR__ ) . 'woo-crm-fw/api/class-integration-with-salesforce-api-base.php';
		require_once plugin_dir_path( __DIR__ ) . 'woo-crm-fw/woo/class-integration-with-salesforce-feed-module.php';
		require_once plugin_dir_path( __DIR__ ) . 'woo-crm-fw/woo/class-integration-with-salesforce-framework.php';
		require_once plugin_dir_path( __DIR__ ) . 'woo-crm-fw/api/class-integration-with-salesforce-api.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'includes/class-integration-with-salesforce-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'includes/class-integration-with-salesforce-i18n.php';

		/**
		 * The class responsible for defining all ajax events.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'includes/class-integration-with-salesforce-ajax-callbacks.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'admin/class-integration-with-salesforce-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public area.
		 */
		require_once plugin_dir_path( __DIR__ ) . 'public/class-integration-with-salesforce-public.php';
		$this->loader = new Integration_With_Salesforce_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Integration_With_Salesforce_I18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 */
	private function set_locale() {

		$plugin_i18n = new Integration_With_Salesforce_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Integration_With_Salesforce_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles', 99 );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts', 99 );

		// Add plugins menu page.
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_admin_menu_page', 99 );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'remove_default_submenu', 99 );

		// All admin actions and filters after License Validation goes here.
		$this->loader->add_filter( 'mwb_add_plugins_menus_array', $plugin_admin, 'crm_admin_submenu_page', 15 );
		$this->loader->add_filter( 'mwb_add_crm_plugins_menus_array', $plugin_admin, 'crm_admin_submenu_page', 15 );
		$this->loader->add_action( 'parent_file', $plugin_admin, 'prefix_highlight_taxonomy_parent_menu' );
		$this->loader->add_filter( 'submenu_file', $plugin_admin, 'crm_bfw_set_submenu_file_to_handle_menu_for_wp_pages', 10, 2 );

		// Initiate all functionality of crm.
		$this->loader->add_action( 'woocommerce_init', $plugin_admin, 'woo_crm_init' );

		$this->loader->add_action( 'admin_init', $plugin_admin, 'do_crm_auth' );

		// Specify components.
		$this->loader->add_filter( 'mwb_crm_' . self::get_current_crm_name( 'slug' ) . '_connect_tabs', $plugin_admin, 'return_tabs' );

		// Includes all the templates.
		$this->loader->add_action( 'mwb_crm_' . self::get_current_crm_name( 'slug' ) . '_connect_render_tab', $plugin_admin, 'render_tabs' );
		$this->loader->add_action( 'mwb_crm_' . self::get_current_crm_name( 'slug' ) . '_connect_render_tabcontent', $plugin_admin, 'render_tab_contents' );

		// Clear log callback.
		$this->loader->add_action( 'mwb_woo_salesforce_clear_log', $plugin_admin, 'clear_sync_log' );

		// Fall back for account name.
		$this->loader->add_filter( 'mwb_woo_salesforce_Account_request_data', $plugin_admin, 'filter_account_request_data', 10, 3 );

		// Add cron schedule for 5 mins.
		$this->loader->add_filter( 'cron_schedules', $plugin_admin, 'add_background_sync_schedule' );

		// Schedule background sync.
		$this->loader->add_filter( 'admin_init', $plugin_admin, 'schedule_background_sync_event' );
		// Background Sync callback.
		$this->loader->add_action( 'mwb_salesforce_background_sync', $plugin_admin, 'perform_background_sync' );

		$this->loader->add_filter( 'mwb_woo_salesforce_Lead_request_data', $plugin_admin, 'filter_lead_request_data', 10, 3 );
		// add membership callback.
		$this->loader->add_filter( 'woo_salesforce_order_callback', $plugin_admin, 'woo_salesforce_order_callback_fn' );

		$this->loader->add_action( 'mwb_woo_salesforce_product_fetch', $plugin_admin, 'perform_sf_product_fetch' );

		// Create dummy hook to manage param count.
		$this->loader->add_filter( 'edit_user_profile_update', $plugin_admin, 'add_wps_sf_edit_user_profile_update_hook', 10, 1 );
		$this->loader->add_filter( 'personal_options_update', $plugin_admin, 'add_wps_sf_edit_user_profile_update_hook', 10, 1 );

		$this->loader->add_filter( 'wps_salesforce_user_request', $plugin_admin, 'wps_salesforce_get_filtered_user_request', 10, 3 );

		if ( 'yes' == get_option( 'mwb-salesforce-enable-product-deletion-woo-to-salesforce', 'no' ) ) {
			$this->loader->add_action( 'wp_trash_post', $plugin_admin, 'action_wp_trash_post', 10, 1 );

			$this->loader->add_action( 'untrash_post', $plugin_admin, 'action_wp_untrash_post', 10, 1 );

			$this->loader->add_action( 'before_delete_post', $plugin_admin, 'custom_function_on_product_delete' );
		}
	}

	/**
	 * Register all of the hooks related to the ajax requests.
	 *
	 * @since  1.0.0
	 */
	private function define_ajax_callback() {

		$ajax_module = new Integration_With_Salesforce_Ajax_Callbacks();

		$this->loader->add_action(
			'wp_ajax_mwb_woo_salesforce_ajax',
			$ajax_module,
			'mwb_woo_salesforce_ajax'
		);

		$this->loader->add_action(
			'wp_ajax_get_datatable_data',
			$ajax_module,
			'get_datatable_data_cb'
		);
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Integration_With_Salesforce_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}


	/**
	 * Initialize custom post type class.
	 */
	private function initialize_cpt() {

		$feed_module = self::get_integration_module( 'feed' );
		// Register custom posttypes.
		$this->loader->add_action( 'init', $feed_module, 'register_plugin_cpt' );
		// Add meta box for feeds.
		$this->loader->add_action( 'add_meta_boxes', $feed_module, 'add_feed_meta_boxes', 10, 2 );
		// Save meta box data.
		$this->loader->add_action( 'save_post', $feed_module, 'save_metabox_data' );

		$this->loader->add_action( 'edit_user_profile', $feed_module, 'add_feed_data_to_user_page', 10, 2 );
		$this->loader->add_action( 'show_user_profile', $feed_module, 'add_feed_data_to_user_page', 10, 2 );
	}

	/**
	 * Register all of the hooks related to the woocommerce objects add/updates.
	 *
	 * @throws Exception Exception.
	 */
	private function define_woo_hooks() {

		$request_module = self::get_integration_module( 'request' );
		// Get all Feeds and then extend the hooks.
		$salesforce = self::get_integration_module( 'framework' );

		// Add all callbacks.
		$hookable_callbacks = $salesforce->get_hook_requests_for_feed();

		if ( empty( $hookable_callbacks ) ) {
			return;
		}
		/**
		 * Remove all order status change sync feeds. Run them once.
		 * 'woocommerce_order_status_pending'
		 * 'woocommerce_order_status_failed'
		 * 'woocommerce_order_fully_refunded'
		 * 'woocommerce_order_partially_refunded'
		 * 'woocommerce_order_status_cancelled'
		 * 'woocommerce_order_status_completed'
		 * 'woocommerce_order_status_on-hold'
		 * 'woocommerce_order_status_processing'
		 */
		$feeds      = $request_module->get_feed_id_by_request();
		$hook_delay = 100;

		if ( ! empty( $feeds ) && is_array( $feeds ) ) {

			$hooks = array(
				'woocommerce_order_status_pending',
				'woocommerce_order_status_failed',
				'woocommerce_order_fully_refunded',
				'woocommerce_order_partially_refunded',
				'woocommerce_order_status_cancelled',
				'woocommerce_order_status_completed',
				'woocommerce_order_status_on-hold',
				'woocommerce_order_status_processing',
			);

			foreach ( $hooks as $key => $hook ) {
				$this->loader->add_action(
					$hook,
					$request_module,
					'shop_order_status_changed',
					$hook_delay,
					$salesforce->get_hook_param_count( $hook )
				);
			}

			foreach ( $feeds as $key => $feeds_id ) {
				unset( $hookable_callbacks[ $feeds_id ] );
			}
		}

		// Same events and callbacks entires will be handled in the feeds.
		$hookable_callbacks = array_unique( $hookable_callbacks, SORT_REGULAR );

		// Now add single event callbacks.
		if ( ! empty( $hookable_callbacks ) && is_array( $hookable_callbacks ) ) {
			foreach ( $hookable_callbacks as $feed_id => $hookable_callback ) {
				try {
					if ( is_array( $hookable_callback['hook'] ) ) {
						foreach ( $hookable_callback['hook'] as $key => $single_hook ) {
							$this->loader->add_action(
								$single_hook,
								$request_module,
								$hookable_callback['callback'],
								$hook_delay,
								$salesforce->get_hook_param_count(
									$single_hook
								)
							);

							$hook_delay++;
						}
					} else {
						$this->loader->add_action(
							$hookable_callback['hook'],
							$request_module,
							$hookable_callback['callback'],
							$hook_delay,
							$salesforce->get_hook_param_count(
								$hookable_callback['hook']
							)
						);
					}

					$hook_delay++;

				} catch ( \Throwable $th ) {
					throw new Exception( esc_html( $th->getMessage() ), 1 );
				}
			}
		}

		$this->loader->add_action(
			'mwb_salesforce_trigger_associated_feeds',
			$request_module,
			'may_be_trigger_associated_feeds',
			10,
			3
		);
	}

	/**
	 * Get instance of feed module class.
	 *
	 * @since    1.0.0
	 * @param string $module Module name.
	 */
	public static function get_integration_module( $module = false ) {

		if ( ! $module ) {
			return false;
		}

		$crm_name = self::get_current_crm_name();
		switch ( $module ) {
			case 'feed':
				$class_name = 'Integration_With_' . $crm_name . '_Feed_Module';
				break;
			case 'request':
				$class_name = 'Integration_With_' . $crm_name . '_Request_Module';
				break;
			case 'api':
				$class_name = 'Integration_With_' . $crm_name . '_Api';
				break;
			case 'framework':
				$class_name = 'Integration_With_' . $crm_name . '_Framework';
				break;

			case 'framework-base':
				$class_name = 'Integration_With_' . $crm_name . '_Framework_Base';
				break;
		}
		$class_instance = $class_name::get_instance();
		return $class_instance;
	}

	/**
	 * Current CRM name.
	 *
	 * @param string $get Key.
	 */
	public static function get_current_crm_name( $get = '' ) {
		$slug = 'salesforce';
		if ( 'slug' == $get ) {
			return esc_html( ( $slug ) );
		} else {
			return esc_html( ucwords( $slug ) );
		}
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 */
	private function define_public_hooks() {

		$plugin_public = new Integration_With_Salesforce_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'profile_update', $plugin_public, 'add_wps_sf_edit_user_profile_update_hook', 10, 1 );
	}
}
