<?php
/**
 * Base Api Class
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */

/**
 * Base Api Class.
 *
 * This class defines all code necessary api communication.
 *
 * @since      1.0.0
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Api_Base {

	/**
	 * API base url.
	 *
	 * @since    1.0.0
	 * @var      string    $base_url    API base url.
	 */
	public $base_url;

	/**
	 * Get Request.
	 *
	 * @param string $endpoint Api endpoint of mautic.
	 * @param array  $data Data to be used in request.
	 * @param array  $headers header to be used in request.
	 */
	public function get( $endpoint, $data = array(), $headers = array() ) {
		return $this->request( 'GET', $endpoint, $data, $headers );
	}

	/**
	 * Post Request.
	 *
	 * @param string $endpoint Api endpoint of mautic.
	 * @param array  $data Data to be used in request.
	 * @param array  $headers header to be used in request.
	 */
	public function post( $endpoint, $data = array(), $headers = array() ) {
		return $this->request( 'POST', $endpoint, $data, $headers );
	}

	/**
	 * Patch Request.
	 *
	 * @param string $endpoint Api endpoint of mautic.
	 * @param array  $data Data to be used in request.
	 * @param array  $headers header to be used in request.
	 */
	public function patch( $endpoint, $data = array(), $headers = array() ) {
		return $this->request( 'PATCH', $endpoint, $data, $headers );
	}

	/**
	 * Delete Request.
	 *
	 * @param string $endpoint Api endpoint of mautic.
	 * @param array  $data Data to be used in request.
	 * @param array  $headers header to be used in request.
	 */
	public function delete( $endpoint, $data = array(), $headers = array() ) {
		return $this->request( 'DELETE', $endpoint, $data, $headers );
	}

	/**
	 * Send api request
	 *
	 * @param string $method   HTTP method.
	 * @param string $endpoint Api endpoint.
	 * @param array  $data     Request data.
	 * @param array  $headers header to be used in request.
	 * @throws Exception Exception.
	 */
	private function request( $method, $endpoint, $data = array(), $headers = array() ) {
		$method   = strtoupper( trim( $method ) );
		$url      = $this->base_url . $endpoint;
		$headers  = array_merge( $headers, $this->get_headers() );
		$response = array();
		$args     = array(
			'method'    => $method,
			'headers'   => $headers,
			'timeout'   => 20,
			/**
			 * Filters the value of Salesforce sslverify.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $request  Woo product request.
			 */
			'sslverify' => apply_filters( 'mwb_woo_salesforce_use_sslverify', true ),
		);
		if ( ! empty( $data ) ) {
			if ( in_array( $method, array( 'GET', 'DELETE' ), true ) ) {
				$url = esc_url_raw( add_query_arg( $data, $url ) );
			} else {
				$args['body'] = $data;
			}
		}
		/**
		 * Filters the value of Salesforce http request args.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed $args  Woo request args.
		 * @param mixed $url product ids.
		 */
		$args     = apply_filters( 'mwb_woo_salesforce_http_request_args', $args, $url );
		$response = wp_remote_request( $url, $args );
		$response_data = array();

		// Add better exception handling.
		try {
			$response_data = $this->parse_response( $response );
		} catch ( Exception $e ) {
			// Got an error in api response may be due to unappropriate connection. Check logs for more information.
			return $response_data;
		}

		$this->log_request( $method, $url, $data, $response_data );

		return $response_data;
	}

	/**
	 * Log request in sync log.
	 *
	 * @param  string $method   Request Method.
	 * @param  string $url      Request Url.
	 * @param  array  $request  Request data.
	 * @param  array  $response Response data.
	 */
	private function log_request( $method, $url, $request, $response ) {

		$log_dir = WC_LOG_DIR . 'mwb-woo-salesforce-' . gmdate( 'Y-m-d' ) . '.log';

		if ( ! is_dir( $log_dir ) ) {
			@fopen( WC_LOG_DIR . 'mwb-woo-salesforce-' . gmdate( 'Y-m-d' ) . '.log', 'a' ); //phpcs:ignore
		}

		if ( ! is_admin() ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
		}

		global $wp_filesystem;  // Define global object of WordPress filesystem.
		WP_Filesystem();        // Intialise new file system object.

		$file_data = '';
		$file_data = $wp_filesystem->get_contents( $log_dir, '' );

		$log  = 'Url : ' . $url . PHP_EOL;
		$log .= 'Method : ' . $method . PHP_EOL;
		$log .= 'Time: ' . current_time( 'F j, Y  g:i a' ) . PHP_EOL;
		$log .= 'Request : ' . wp_json_encode( $request ) . PHP_EOL;
		$log .= 'Response : ' . wp_json_encode( $response ) . PHP_EOL;
		$log .= '------------------------------------' . PHP_EOL;

		$file_data .= $log;
		$wp_filesystem->put_contents( $log_dir, $file_data );
	}

	/**
	 * Parse Api response.
	 *
	 * @param array $response Raw response.
	 * @throws Exception Exception.
	 * @return array filtered reponse.
	 */
	private function parse_response( $response ) {

		if ( $response instanceof WP_Error ) {
			throw new Exception( 'Error', 0 );
		}
		$code    = (int) wp_remote_retrieve_response_code( $response );
		$message = wp_remote_retrieve_response_message( $response );
		$body    = wp_remote_retrieve_body( $response );
		$data    = json_decode( $body, ARRAY_A );
		return compact( 'code', 'message', 'data' );
	}

	/**
	 * Get headers.
	 *
	 * @return array headers.
	 */
	public function get_headers() {
		return array();
	}
}
