<?php
/**
 * Base Api Class
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/woo-crm-fw
 */

/**
 * Base Api Class.
 *
 * This class defines all code necessary api communication.
 *
 * @since      1.0.0
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/woo-crm-fw
 */
class Integration_With_Salesforce_Api extends Integration_With_Salesforce_Api_Base {

	/**
	 * CRM Prefix.
	 *
	 * @since    1.0.0
	 * @var      string    $crm_prefix    Prefix of current crm.
	 */
	private static $crm_prefix;

	/**
	 * Base authentication url.
	 *
	 * @since    1.0.0
	 * @var      string    $base_auth_url    Base authentication url.
	 */
	private static $base_auth_url = 'https://login.salesforce.com/services/oauth2/';

	/**
	 * Sandbox authentication url.
	 *
	 * @since    1.0.0
	 * @var      string    $sandbox_base_auth_url    Sandbox authentication url.
	 */
	private static $sandbox_base_auth_url = 'https://test.salesforce.com/services/oauth2/';

	/**
	 * Salesforce API Consumer key.
	 *
	 * @since    1.0.0
	 * @var      string    $consumer_key    Salesforce API Consumer key.
	 */
	private static $consumer_key;

	/**
	 * Salesforce API Consumer secret.
	 *
	 * @since    1.0.0
	 * @var      string    $consumer_secret    Salesforce API Consumer secret.
	 */
	private static $consumer_secret;

	/**
	 * Salesforce Account Enviornment.
	 *
	 * @since    1.0.0
	 * @var      string    $enviornment    Salesforce Account Enviornment.
	 */
	private static $enviornment;

	/**
	 * Salesforce access token.
	 *
	 * @since    1.0.0
	 * @var      string    $access_token    Salesforce access token.
	 */
	private static $access_token;

	/**
	 * Salesforce access token.
	 *
	 * @since    1.0.0
	 * @var      string    $refresh_token    Salesforce access token.
	 */
	private static $refresh_token;

	/**
	 * Salesforce Instance Url.
	 *
	 * @since    1.0.0
	 * @var      string    $instance_url    Salesforce Instance Url.
	 */
	private static $instance_url;

	/**
	 * Last access token issued timestamp.
	 *
	 * @since    1.0.0
	 * @var      string    $issued_at    Last access token issued timestamp.
	 */
	private static $issued_at;

	/**
	 * Class instance.
	 *
	 * @since    1.0.0
	 * @var      Integration_With_Salesforce_Api    $_instance    Class instance.
	 */
	protected static $_instance = null;

	/**
	 * Salesforce API Version.
	 *
	 * @since    1.0.0
	 * @var      string    $api_version    Salesforce API Version.
	 */
	private static $api_version = 'v51.0';

	/**
	 * Main Integration_With_Salesforce_Api Instance.
	 *
	 * Ensures only one instance of Integration_With_Salesforce_Api is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return Integration_With_Salesforce_Api - Main instance.
	 */
	public static function get_instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		self::initialize();
		return self::$_instance;
	}

	/**
	 * Initialize properties.
	 *
	 * @since 1.0.0
	 *
	 * @param  array $token_data Saved token data.
	 */
	private static function initialize( $token_data = array() ) {

		self::$crm_prefix = Integration_With_Salesforce::get_current_crm_name( 'slug' );

		self::$consumer_key    = get_option( 'mwb-' . self::$crm_prefix . '-consumer-key', '' );
		self::$consumer_secret = get_option( 'mwb-' . self::$crm_prefix . '-consumer-secret', '' );
		self::$enviornment     = get_option( 'mwb-' . self::$crm_prefix . '-enviornment', '' );

		if ( empty( $token_data ) ) {
			$token_data = get_option( 'mwb-woo-' . self::$crm_prefix . '-token-data', array() );
		}

		if ( ! empty( $token_data ) ) {
			self::$access_token  = ! empty( $token_data['access_token'] ) ? $token_data['access_token'] : '';
			self::$refresh_token = ! empty( $token_data['refresh_token'] ) ? $token_data['refresh_token'] : '';
			self::$instance_url  = ! empty( $token_data['instance_url'] ) ? $token_data['instance_url'] : '';
			self::$issued_at     = ! empty( $token_data['issued_at'] ) ? $token_data['issued_at'] : '';
		}
	}

	/**
	 * Get api domain.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $use_custom_app If custom app is used for authentication.
	 * @return string Site redirecrt Uri.
	 */
	public function get_redirect_uri( $use_custom_app = 'no' ) {

		if ( 'yes' == $use_custom_app ) {
			return admin_url();
		}

		return 'https://auth.makewebbetter.com/integration/salesforce-auth/';
	}

	/**
	 * Get instance url.
	 *
	 * @since 1.0.0
	 *
	 * @return string Instance url.
	 */
	public function get_instance_url() {
		return ! empty( self::$instance_url ) ? self::$instance_url : false;
	}

	/**
	 * Get access token.
	 *
	 * @since 1.0.0
	 *
	 * @return string Access token.
	 */
	public function get_access_token() {
		return ! empty( self::$access_token ) ? self::$access_token : false;
	}

	/**
	 * Get refresh token.
	 *
	 * @since 1.0.0
	 *
	 * @return string Refresh token.
	 */
	public function get_refresh_token() {
		return ! empty( self::$refresh_token ) ? self::$refresh_token : false;
	}

	/**
	 * Get active access token issued time.
	 *
	 * @since 1.0.0
	 *
	 * @return string expiry seconds.
	 */
	public function get_access_token_issue_time() {
		return ! empty( self::$issued_at ) ? ( self::$issued_at / 1000 ) : false;
	}

	/**
	 * Get refreshed token data from api.
	 *
	 * @since 1.0.0
	 *
	 * @return bool.
	 */
	public function renew_access_token() {

		$refresh_token = $this->get_refresh_token();

		if ( ! empty( $refresh_token ) ) {
			$response = $this->process_access_token( false, $refresh_token );
		}

		return ! empty( $response['code'] ) && 200 == $response['code'] ? true : false;
	}

	/**
	 * Save New token data into db.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $code          Unique code to generate token.
	 */
	public function save_access_token( $code ) {
		$this->process_access_token( $code );
	}

	/**
	 * Get Base Authorization url.
	 *
	 * @since 1.0.0
	 *
	 * @return string Base Authorization url.
	 */
	public function base_auth_url() {
		$url = ( 'production' == self::$enviornment ) ? self::$base_auth_url : self::$sandbox_base_auth_url;
		return $url;
	}

	/**
	 * Get Authorization url.
	 *
	 * @since 1.0.0
	 *
	 * @return string Authorization url.
	 */
	public function get_auth_code_url() {

		$use_custom_app = get_option( 'mwb-salesforce-use-custom-app', 'no' );

		$query_args = array(
			'response_type' => 'code',
			'state'         => urlencode( $this->get_oauth_state( $use_custom_app ) ),
			'client_id'     => urlencode( self::$consumer_key ),
			'redirect_uri'  => urlencode( $this->get_redirect_uri( $use_custom_app ) ),
			'scope'         => urlencode( 'api refresh_token' ),
		);

		$salesforce_login_url = esc_url_raw( add_query_arg( $query_args, $this->base_auth_url() . 'authorize' ) );

		return $salesforce_login_url;
	}

	/**
	 * Get oauth state with current instance redirect url.
	 *
	 * @param  string $use_custom_app If custom app is used for authentication.
	 * @return string State.
	 */
	public function get_oauth_state( $use_custom_app = 'no' ) {

		$nonce = wp_create_nonce( 'mwb_' . self::$crm_prefix . '_state' );

		if ( 'yes' == $use_custom_app ) {
			return $nonce;
		}

		$admin_redirect_url = admin_url();
		$args               = array(
			'mwb_nonce'  => $nonce,
			'mwb_source' => 'salesforce',
		);
		$admin_redirect_url = esc_url_raw( add_query_arg( $args, $admin_redirect_url ) );
		return $admin_redirect_url;
	}

	/**
	 * Get refresh token data from api.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $code          Unique code to generate token.
	 * @param  string $refresh_token Unique code to renew token.
	 */
	public function process_access_token( $code = '', $refresh_token = '' ) {

		$endpoint = '/token';

		$this->base_url = $this->base_auth_url();

		$use_custom_app = get_option( 'mwb-salesforce-use-custom-app', 'no' );

		$params = array(
			'grant_type'    => 'authorization_code',
			'client_id'     => self::$consumer_key,
			'client_secret' => self::$consumer_secret,
			'redirect_uri'  => $this->get_redirect_uri( $use_custom_app ),
			'code'          => $code,
		);

		if ( empty( $code ) ) {
			$params['refresh_token'] = $refresh_token;
			$params['grant_type']    = 'refresh_token';
			unset( $params['code'] );
		}

		$response = $this->post( $endpoint, $params, $this->get_token_auth_header() );

		if ( isset( $response['code'] ) && 200 == $response['code'] ) {

			// Save token.
			$token_data = ! empty( $response['data'] ) ? $response['data'] : array();
			$token_data = $this->merge_refresh_token( $token_data );
			update_option( 'mwb-woo-' . self::$crm_prefix . '-token-data', $token_data );
			update_option( 'mwb-woo-' . self::$crm_prefix . '-authorised', '1' );
			self::initialize( $token_data );
		} else {
			// On failure add to log.
			delete_option( 'mwb-woo-' . self::$crm_prefix . '-token-data' );
			delete_option( 'mwb-woo-' . self::$crm_prefix . '-authorised' );
		}

		return $response;
	}


	/**
	 * Merge refresh token with new access token data.
	 *
	 * @since 1.0.0
	 *
	 * @param  array $new_token_data Latest token data.
	 * @return  array                 Token data.
	 */
	public function merge_refresh_token( $new_token_data ) {

		$old_token_data = get_option( 'mwb-woo-' . self::$crm_prefix . '-token-data', array() );

		if ( empty( $old_token_data ) ) {
			return $new_token_data;
		}

		foreach ( $old_token_data as $key => $value ) {
			if ( isset( $new_token_data[ $key ] ) ) {
				$old_token_data[ $key ] = $new_token_data[ $key ];
			}
		}
		return $old_token_data;
	}

	/**
	 * Get authorization headers for getting token.
	 *
	 * @since 1.0.0
	 *
	 * @return array headers.
	 */
	public function get_token_auth_header() {
		return array();
	}

	/**
	 * Get Request headers.
	 *
	 * @since 1.0.0
	 * @return array headers.
	 */
	public function get_auth_header() {

		$headers = array(
			'Authorization' => 'Bearer ' . self::$access_token,
			'content-type'  => 'application/json',
		);

		return $headers;
	}

	/**
	 * Prepare bulk request data for create or update records.
	 *
	 * @since 1.0.0
	 *
	 * @param  string $object     Object Name.
	 * @param  array  $record_data All records data.
	 * @return array Bulk request data.
	 */
	public function prepare_bulk_request_data( $object, $record_data ) {

		$request_data     = array();
		$record_to_update = array();
		$record_to_create = array();
		$created_indexes  = array();
		$updated_indexes  = array();
		foreach ( $record_data as $key => $data ) {

			$data['attributes'] = array( 'type' => $object );
			if ( ! empty( $data['id'] ) ) {
				// Add a method here to unset unique indexes.
				// Unique indexes can not be sent in update request.
				if ( 'PricebookEntry' === $object ) {
					unset( $data['Pricebook2Id'] );
					unset( $data['Product2Id'] );
				}
				if ( 'Product2' === $object ) {
					if ( isset( $data['StockKeepingUnit'] ) ) {
						unset( $data['StockKeepingUnit'] );
					}
				}
				$record_to_update[] = $data;
				$updated_indexes[]  = $key;
			} else {
				unset( $data['id'] );
				$record_to_create[] = $data;
				$created_indexes[]  = $key;
			}
			array_push( $request_data, $data );
		}

		return array(
			'record_to_create' => array(
				'records' => $record_to_create,
			),
			'record_to_update' => array(
				'records' => $record_to_update,
			),
			'created_indexes'  => $created_indexes,
			'updated_indexes'  => $updated_indexes,
		);
	}

	/**
	 * Bulk record sync.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $record_data All records data.
	 * @param array  $indexes     Indexes of records.
	 * @param string $method      Sync method.
	 * @param int    $feed_id         feed id.
	 * @return array Sync response according to indexes.
	 */
	public function bulk_sync_record( $record_data, $indexes, $method, $feed_id ) {

		$request_module = Integration_With_Salesforce_Request_Module::get_instance();
		$filter_exist   = $request_module->maybe_check_filter( $feed_id );

		$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );
		$crm_object  = $feed_module->get_feed_data( $feed_id, 'crm_object' );

		$response = array();
		if ( ! empty( $filter_exist ) ) {
			$syncing_data = array();
			$record_index = array();
			if ( ! empty( $record_data['records'] ) ) {
				foreach ( $record_data['records'] as $key => $data ) {
					$filter_result = $request_module->mwb_salesforce_validate_filter( $filter_exist, $data );
					if ( true === $filter_result ) { // If filter results true, then send data to CRM.
						array_push( $syncing_data, $data );
						array_push( $record_index, $indexes[ $key ] );

					} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
						array_push( $syncing_data, $data );
						array_push( $record_index, $indexes[ $key ] );
					}
				}
				if ( ! empty( $syncing_data ) ) {
					$response = $this->mwb_salesforce_sync_data( array( 'records' => $syncing_data ), $record_index, $method, $crm_object );
				}
			}
		} else {
			$response = $this->mwb_salesforce_sync_data( $record_data, $indexes, $method, $crm_object );
		}

		return $response;
	}

	/**
	 * Sync the data to salesforce.
	 *
	 * @param array  $record_data All records data.
	 * @param array  $indexes     Indexes of records.
	 * @param string $method      Sync method.
	 * @param string $crm_object  Crm object.
	 * @return array Sync response according to indexes.
	 */
	public function mwb_salesforce_sync_data( $record_data, $indexes, $method, $crm_object ) {

		$object_fields = $this->get_object_fields( $crm_object, true );
		$api_name      = array_column( $object_fields, 'name' );

		foreach ( $record_data['records'] as $index => $mapping ) {
			foreach ( $mapping as $key => $data ) {
				if ( ( 'id' == $key ) || ( 'attributes' == $key ) || ( 'post_id' == $key ) ) {
					continue;
				}
				if ( ! in_array( $key, $api_name ) ) {
					unset( $record_data['records'][ $index ][ $key ] );
				}
			}
		}
		
		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/composite/sobjects';
		$params         = wp_json_encode( $record_data );
		$headers        = $this->get_auth_header();
		$response       = $this->$method( $endpoint, $params, $headers );

		if ( $this->if_access_token_expired( $response ) ) {
			$this->base_url = self::$instance_url;
			$endpoint       = '/services/data/' . self::$api_version . '/composite/sobjects';
			$params         = wp_json_encode( $record_data );
			$headers        = $this->get_auth_header();
			$response       = $this->$method( $endpoint, $params, $headers );
		}

		$indexed_response = array();

		if ( $this->is_success( $response ) ) {
			foreach ( $response['data'] as $line_key => $line_response ) {
				$line_response['message'] = ( 'post' == $method ) ? 'Created' : 'Updated'; //phpcs:ignore
				$indexed_response[ $indexes[ $line_key ] ] = $line_response;
			}
		}
		if ( empty( $indexed_response ) ) {
			$indexed_response = $response;
		}

		return $indexed_response;
	}

	/**
	 * Create or udpate bulk record.
	 *
	 * @since 1.0.0
	 *
	 * @param string $object        Object name.
	 * @param array  $record_data   All records data.
	 * @param string $primary_field Primary field key.
	 * @param int    $feed_id         feed id.
	 * @return array Sync response.
	 */
	public function bulk_create_or_update_record( $object, $record_data, $primary_field, $feed_id ) {

		$request_data = $this->prepare_bulk_request_data( $object, $record_data );
		$create_response = array(); // 1,3,5
		$update_response = array(); // 2,4,6
		$merged_response = array();
		$record_response = array();
		if ( ! empty( $request_data['record_to_create'] ) && ! empty( $request_data['record_to_create']['records'] ) ) {
			$create_response = $this->bulk_sync_record(
				$request_data['record_to_create'],
				$request_data['created_indexes'],
				'post',
				$feed_id
			);
		}

		if ( ! empty( $request_data['record_to_update'] ) && ! empty( $request_data['record_to_update']['records'] ) ) {
			$update_response = $this->bulk_sync_record(
				$request_data['record_to_update'],
				$request_data['updated_indexes'],
				'patch',
				$feed_id
			);
		}

		$merged_response = array_replace( $create_response, $update_response );

		if ( isset( $merged_response['code'] ) && 400 == $merged_response['code'] && 'EXCEEDED_ID_LIMIT' == $merged_response['data'][0]['errorCode'] ) {
			if ( 'Product2' == $object ) {
				$merged_response = array();
				if ( ! empty( $request_data['record_to_create'] ) && ! empty( $request_data['record_to_create']['records'] ) ) {
					$req_prod = array_chunk( $request_data['record_to_create']['records'], 200 );
					$req_prod_indexes = array_chunk( $request_data['created_indexes'], 200 );
					foreach ( $req_prod as $keyy => $valuee ) {
						$req['records'] = $valuee;
						$create_response = $this->bulk_sync_record(
							$req,
							$req_prod_indexes[$keyy],
							'post',
							$feed_id
						);
						$merged_response = array_merge( $merged_response, $create_response );
					}
				}
				if ( ! empty( $request_data['record_to_update'] ) && ! empty( $request_data['record_to_update']['records'] ) ) {
					$req_prod1 = array_chunk( $request_data['record_to_update']['records'], 200 );
					$req_prod_indexes1 = array_chunk( $request_data['updated_indexes'], 200 );
					foreach ( $req_prod1 as $keyy1 => $valuee1 ) {
						$req1['records'] = $valuee1;
						$update_response = $this->bulk_sync_record(
							$req1,
							$req_prod_indexes1[$keyy1],
							'patch',
							$feed_id
						);
						$merged_response = array_merge( $merged_response, $update_response );
					}
				}
			} else if ( 'PricebookEntry' == $object ) {
				$merged_response = array();
				if ( ! empty( $request_data['record_to_create'] ) && ! empty( $request_data['record_to_create']['records'] ) ) {
					$req_prod = array_chunk( $request_data['record_to_create']['records'], 200 );
					$req_prod_indexes = array_chunk( $request_data['created_indexes'], 200 );
					foreach ( $req_prod as $keyy => $valuee ) {
						$req['records'] = $valuee;
						$create_response = $this->bulk_sync_record(
							$req,
							$req_prod_indexes[$keyy],
							'post',
							$feed_id
						);
						$merged_response = array_merge( $merged_response, $create_response );
					}
				}
				if ( ! empty( $request_data['record_to_update'] ) && ! empty( $request_data['record_to_update']['records'] ) ) {
					$req_prod1 = array_chunk( $request_data['record_to_update']['records'], 200 );
					$req_prod_indexes1 = array_chunk( $request_data['updated_indexes'], 200 );
					foreach ( $req_prod1 as $keyy1 => $valuee1 ) {
						$req1['records'] = $valuee1;
						$update_response = $this->bulk_sync_record(
							$req1,
							$req_prod_indexes1[$keyy1],
							'patch',
							$feed_id
						);
						$merged_response = array_merge( $merged_response, $update_response );
					}
				}
			}
		}

		ksort( $merged_response );

		$duplicates = array();
		$existing_pricebook_entries = array();
		$deleted_record_keys = array();
		$deleted_record_data = array();

		if ( ! empty( $merged_response ) ) {
			foreach ( $merged_response as $line_key => $line_response ) {
				if ( $line_response['success'] ) {
					$record_response[ $line_key ] = $line_response;
				} elseif ( ! empty( $line_response['errors'] ) ) {
					if ( ! empty( $line_response['errors'][0] ) && ! empty( $line_response['errors'][0]['statusCode'] ) ) {
						if ( 'DUPLICATES_DETECTED' === $line_response['errors'][0]['statusCode'] || 'DUPLICATE_VALUE' === $line_response['errors'][0]['statusCode'] ) {
							array_push( $duplicates, $line_key );
						} else if ( 'FIELD_INTEGRITY_EXCEPTION' === $line_response['errors'][0]['statusCode'] && 'PricebookEntry' == $object ) {
							array_push( $existing_pricebook_entries, $line_key );
						} else if ( 'ENTITY_IS_DELETED' === $line_response['errors'][0]['statusCode'] ) {

							array_push( $deleted_record_keys, $line_key );

						} else {
							$record_response[ $line_key ] = $line_response;
						}
					}
				}
			}
		}

		// Manage deleted records.
		if ( ! empty( $deleted_record_keys ) ) {

			$deleted_records_response = $this->manage_deleted_records( $object, $record_data, $primary_field, $record_response, $feed_id, $deleted_record_keys );
			$record_response          = array_replace( $record_response, $deleted_records_response );
		}

		if ( ! empty( $duplicates ) ) {
			$duplicate_query = $this->prepare_duplicate_query( $record_data, $primary_field, $duplicates, $object, $primary_field );
			$query_response  = $this->query_records( $duplicate_query );
			$query_response  = $this->merge_query_response( $query_response, $record_data, $primary_field, $record_response );
			ksort( $query_response );
			$record_response = $query_response;
		}

		// Manage already existing pricebook entry records.

		if ( ! empty( $existing_pricebook_entries ) ) {
			$pb_query          = $this->prepare_duplicate_query( $record_data, array( 'Pricebook2Id', 'Product2Id' ) , $existing_pricebook_entries, $object, 'Product2Id' );
			$pb_query_response = $this->query_records( $pb_query );
			$query_response    = $this->merge_query_response( $pb_query_response, $record_data, array( 'Pricebook2Id', 'Product2Id' ), $record_response );
			ksort( $query_response );
			$record_response = $query_response;
		}

		return $record_response;
	}


	/**
	 * Merge query response with main response.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $query_response  Query request response.
	 * @param array  $record_data     All records data.
	 * @param string $primary_field   Primary field key.
	 * @param array  $record_response Main response.
	 * @return array Merged record response.
	 */
	public function merge_query_response( $query_response, $record_data, $primary_field, $record_response ) {

		if ( ! is_array( $primary_field ) ) {
			$primary_field = array( $primary_field );
		}

		foreach ( $record_data as $key => $data ) {
			$seach_data = array_intersect_key( $data , array_flip( $primary_field ) );
			$primary_match = array();
			foreach ($query_response as $qkey => $qvalue) {
				
				if ( isset( $qvalue['attributes'] ) ) {
					// Unset any nested values.
					unset( $qvalue['attributes'] );
				}

				$primary_match = array_intersect( $qvalue, $seach_data );

				if ( $primary_match == $seach_data ) {
					$record_response[ $key ] = array(
						'id'      => $query_response[ $qkey ]['Id'],
						'success' => true,
						'errors'  => array(),
						'message' => 'Updated',
					);
				}
			}
		}

		return $record_response;
	}

	/**
	 * Query records.
	 *
	 * @since 1.0.0
	 *
	 * @param string $query  Query string.
	 * @return array Query Response.
	 */
	public function query_records( $query, $prod_two_way = false ) {

		$this->base_url = self::$instance_url;
		$records        = array();
		$endpoint       = '/services/data/' . self::$api_version . '/query';
		$params         = array(
			'q' => $query,
		);
		$headers        = $this->get_auth_header();
		$response       = $this->get( $endpoint, $params, $headers );
		if ( $this->is_success( $response ) ) {
			if ( ! empty( $response['data'] ) && ! empty( $response['data']['records'] ) ) {
				$records = $response['data']['records'];
			}
			if ( true == $prod_two_way ) {
				$records = $response;
			}
		}

		return $records;
	}

	/**
	 * Deleted a new record.
	 *
	 * @param  string $module      Module to be deleted.
	 * @param  array  $record_data Record data.
	 * @return array               Response data.
	 */
	public function delete_record( $record_id, $module, $record_data ) {
		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/sobjects/' . $module . '/' . $record_id;
		$request_data    = array();
		$headers         = $this->get_auth_header();
		return $this->delete( $endpoint, $request_data, $headers );
	}

	/**
	 * Deleted a new record.
	 *
	 * @param  string $module      Module to be deleted.
	 * @param  array  $record_data Record data.
	 * @return array               Response data.
	 */
	public function get_record( $record_id, $module, $record_data ) {
		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/sobjects/' . $module . '/' . $record_id;
		$request_data    = array();
		$headers         = $this->get_auth_header();
		return $this->get( $endpoint, $request_data, $headers );
	}

	/**
	 * Prepare duplicate query based on record data.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $record_data    All records data.
	 * @param array  $primary_field  Array of fields to select.
	 * @param array  $duplicates     Duplicates array.
	 * @param string $object         Object name.
	 * @param string $search_field   Search field key.
	 * @return string Duplicate query string.
	 */
	public function prepare_duplicate_query( $record_data, $primary_field, $duplicates, $object, $search_field ) {

		if ( ! is_array( $primary_field ) ) {
			$primary_field = array( $primary_field );
		}

		$duplicate_records = array_filter(
			$record_data,
			function ( $key ) use ( $duplicates ) {
				return in_array( $key, $duplicates );
			},
			ARRAY_FILTER_USE_KEY
		);

		$duplicate_data   = array_column( $duplicate_records, $search_field );
		$duplicate_string = '';
		if ( ! empty( $duplicate_data ) ) {
			array_walk(
				$duplicate_data,
				function ( &$item ) {
					$item = "'" . $item . "'";
				}
			);
			$duplicate_string = '( ' . implode( ' , ', $duplicate_data ) . ' )';
		}
		$duplicate_query = sprintf(
			'SELECT Id, %s FROM %s WHERE %s IN %s',
			implode( ',' , $primary_field),
			$object,
			$search_field,
			$duplicate_string
		);

		return $duplicate_query;
	}

	/**
	 * Delete bulk records.
	 *
	 * @since 1.0.0
	 *
	 * @param array $record_ids Array of record ids.
	 * @return array Array of response data.
	 */
	public function delete_bulk_records( $record_ids ) {
		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/composite/sobjects';
		$params         = array( 'ids' => implode( ',', $record_ids ) );
		$headers        = $this->get_auth_header();
		$response       = $this->delete( $endpoint, $params, $headers );
		if ( $this->if_access_token_expired( $response ) ) {
			$this->base_url = self::$instance_url;
			$endpoint       = '/services/data/' . self::$api_version . '/composite/sobjects';
			$params         = array( 'ids' => implode( ',', $record_ids ) );
			$headers        = $this->get_auth_header();
			$response       = $this->delete( $endpoint, $params, $headers );
		}
		return $response;
	}

	/**
	 * Create single record on Salesforce
	 *
	 * @param  string  $object      CRM object name.
	 * @param  array   $record_data Request data.
	 * @param  boolean $is_bulk     Is a bulk request.
	 * @param  array   $log_data    Data to create log.
	 *
	 * @since 1.0.0
	 *
	 * @return array Response data.
	 */
	public function create_or_update_record( $object, $record_data, $is_bulk = false, $log_data = array() ) {

		$response_data = array(
			'succes' => false,
			'msg'    => __( 'Something went wrong', 'integration-with-salesforce' ),
		);

		// Check for the existing record based on selected primary field.
		$primary_field = 'Name';
		$record_id     = false;

		$search_response = $this->check_for_existing_record( $object, $record_data, $primary_field );
		if ( $this->if_access_token_expired( $search_response ) ) {
			$search_response = $this->check_for_existing_record( $object, $record_data, $primary_field );
		}

		// Get record id from search query result.
		$record_id = $this->may_be_get_record_id_from_search( $search_response, $record_data, $primary_field );

		if ( ! $record_id ) {
			// No result found create a new record and return.
			$response = $this->create_record( $object, $record_data, $is_bulk, $log_data );
			if ( $this->if_access_token_expired( $response ) ) {
				$response = $this->create_record( $object, $record_data, $is_bulk, $log_data );
			}
			if ( $this->is_success( $response ) ) {
				$response_data['success'] = true;
				$response_data['msg']     = 'Record Created';
			}
			return $response_data;
		}

		// Update an existing record based on record_id.
		$response = $this->update_record( $record_id, $object, $record_data, $is_bulk, $log_data );
		if ( $this->if_access_token_expired( $response ) ) {
			$response = $this->update_record( $record_id, $object, $record_data, $is_bulk, $log_data );
		}
		if ( $this->is_success( $response ) ) {
			$response_data['success'] = true;
			$response_data['msg']     = 'Record Updated';
		}
		return $response_data;
	}

	/**
	 * Check for exsiting record in search query response.
	 *
	 * @param array  $response      Search query response.
	 * @param array  $record_data   Request data of searched record.
	 * @param string $primary_field Primary field name.
	 *
	 * @return string|bool          Id of existing record or false.
	 */
	private function may_be_get_record_id_from_search( $response, $record_data, $primary_field ) {
		$record_id     = false;
		$found_records = array();
		if ( isset( $response['code'] ) && 200 == $response['code'] && 'OK' == $response['message'] ) {
			if ( ! empty( $response['data'] ) && ! empty( $response['data']['searchRecords'] ) ) {
				$found_records = $response['data']['searchRecords'];
			}
		}

		if ( count( $found_records ) > 0 ) {
			foreach ( $found_records as $key => $record ) {
				if ( $record[ $primary_field ] === $record_data[ $primary_field ] ) {
					$record_id = $record['Id'];
					break;
				}
			}
		}
		return $record_id;
	}

	/**
	 * Check for existing record using parameterizedSearch.
	 *
	 * @param string $object        Target object name.
	 * @param array  $record_data   Record data.
	 * @param string $primary_field Primary field.
	 *
	 * @return array                Response data array.
	 */
	private function check_for_existing_record( $object, $record_data, $primary_field ) {
		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/parameterizedSearch';
		$params         = array(
			'q'        => $record_data[ $primary_field ],
			'fields'   => array( 'Id', $primary_field ),
			'sobjects' => array(
				array( 'name' => $object ),
			),
		);

		$params   = wp_json_encode( $params );
		$headers  = $this->get_auth_header();
		$response = $this->post( $endpoint, $params, $headers );
		return $response;
	}

	/**
	 * Check if resposne has success code.
	 *
	 * @param  array $response  Response data.
	 *
	 * @since 1.0.0
	 *
	 * @return boolean true|false.
	 */
	private function is_success( $response ) {
		if ( ! empty( $response['code'] ) ) {
			return in_array( $response['code'], array( 200, 201, 204, 202 ) );
		}
		return true;
	}

	/**
	 * Create a new record.
	 *
	 * @param  string  $object     Object name.
	 * @param  array   $record_data Record data.
	 * @param  boolean $is_bulk    Is a bulk request.
	 * @param  array   $log_data   Data to create log.
	 * @return array               Response data.
	 */
	private function create_record( $object, $record_data, $is_bulk, $log_data ) {

		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/sobjects/' . $object;
		$params         = wp_json_encode( $record_data );
		$headers        = $this->get_auth_header();
		$response       = $this->post( $endpoint, $params, $headers );
		return $response;
	}

	/**
	 * Update an existing record.
	 *
	 * @param  string  $record_id   Record id to be updated.
	 * @param  string  $object      Object name.
	 * @param  array   $record_data Record data.
	 * @param  boolean $is_bulk     Is a bulk request.
	 * @param  array   $log_data    Data to create log.
	 * @return array                Response data.
	 */
	private function update_record( $record_id, $object, $record_data, $is_bulk, $log_data ) {

		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/sobjects/' . $object . '/' . $record_id;
		$params         = wp_json_encode( $record_data );
		$headers        = $this->get_auth_header();
		$response       = $this->patch( $endpoint, $params, $headers );
		return $response;
	}

	/**
	 * Get available pricebook in crm.
	 *
	 * @param  boolean $force Fetch from api.
	 * @return array          Response data.
	 */
	public function get_pricebook_data( $force = false ) {
		$pricebook_data = get_transient( 'mwb_woo_salesforce_pricebook_data', '' );
		if ( ! $force && ! empty( $pricebook_data ) ) {
			return $pricebook_data;
		}
		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/queryAll';
		$query_args     = array(
			'q' => 'SELECT Id, Name, IsStandard FROM Pricebook2 WHERE IsDeleted = false ',
		);
		$headers        = $this->get_auth_header();
		$response       = $this->get( $endpoint, $query_args, $headers );

		if ( $this->if_access_token_expired( $response ) ) {
			$this->base_url = self::$instance_url;
			$headers        = $this->get_auth_header();
			$response       = $this->get( $endpoint, $query_args, $headers );
		}
		if ( isset( $response['code'] ) && 200 == $response['code'] && 'OK' == $response['message'] ) {
			if ( ! empty( $response['data'] ) && ! empty( $response['data']['records'] ) ) {
				$pricebook_data = $response['data']['records'];
				set_transient( 'mwb_woo_salesforce_pricebook_data', $pricebook_data );
			}
		}

		return $pricebook_data;
	}

	/**
	 * Check if response has expired access token message.
	 *
	 * @param  array $response Api response.
	 * @return bool            Access token status.
	 */
	private function if_access_token_expired( $response ) {
		if ( isset( $response['code'] ) && 401 == $response['code'] && 'Unauthorized' == $response['message'] ) {
			return $this->renew_access_token();
		}
		return false;
	}

	/**
	 * Get available object in crm.
	 *
	 * @param  boolean $force            Fetch from api.
	 * @param  boolean $validate_request Request for validating api.
	 * @return array          Response data.
	 */
	public function get_crm_objects( $force = false, $validate_request = false ) {

		$data = get_transient( 'mwb_woo_salesforce_objects_data' );
		if ( ! $force && false !== ( $data ) ) {
			return $data;
		}

		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/sobjects';
		$headers        = $this->get_auth_header();
		$response       = $this->get( $endpoint, array(), $headers );

		$objects = array();
		if ( ! empty( $response ) ) {
			if ( isset( $response['code'] ) && 401 == $response['code'] && 'Unauthorized' == $response['message'] ) {
				if ( $this->renew_access_token() ) {
					$this->base_url = self::$instance_url;
					$headers        = $this->get_auth_header();
					$response       = $this->get( $endpoint, array(), $headers );
				}
			}
			if ( isset( $response['code'] ) && 200 == $response['code'] && 'OK' == $response['message'] ) {
				if ( ! empty( $response['data'] ) && ! empty( $response['data']['sobjects'] ) ) {
					$objects = $response['data']['sobjects'];
					set_transient( 'mwb_woo_salesforce_objects_data', $objects );
				}
			}

			if ( $validate_request ) {
				$objects = $response;
			}
		}
		return $objects;
	}

	/**
	 * Get fields assosiated with an object.
	 *
	 * @param  string  $object Name of object.
	 * @param  boolean $force  Fetch from api.
	 * @return array           Response data.
	 */
	public function get_object_fields( $object, $force = false ) {

		$data = get_transient( 'mwb_woo_salesforce_' . $object . '_fields' );
		if ( ! $force && false !== ( $data ) ) {
			return $data;
		}

		$this->base_url = self::$instance_url;
		$endpoint       = '/services/data/' . self::$api_version . '/sobjects/' . $object . '/describe';
		$headers        = $this->get_auth_header();
		$response       = $this->get( $endpoint, array(), $headers );
		$fields         = array();
		if ( ! empty( $response ) ) {
			if ( isset( $response['code'] ) && 401 == $response['code'] && 'Unauthorized' == $response['message'] ) {
				if ( $this->renew_access_token() ) {
					$this->base_url = self::$instance_url;
					$headers        = $this->get_auth_header();
					$response       = $this->get( $endpoint, array(), $headers );
				}
			}
			if ( isset( $response['code'] ) && 200 == $response['code'] && 'OK' == $response['message'] ) {
				if ( ! empty( $response['data'] ) && ! empty( $response['data']['fields'] ) ) {
					$fields = $response['data']['fields'];
					set_transient( 'mwb_woo_salesforce_' . $object . '_fields', $fields );
				}
			}
		}

		return $fields;
	}


	/**
	 * Manage deleted record and create.
	 *
	 * @since 1.0.3
	 *
	 * @param string $object        Object name.
	 * @param array  $record_data   All records data.
	 * @param string $primary_field Primary field key.
	 * @param string $deleted_indexes Deleted indexes.
	 * @param string $record_response Response of previous sync.
	 * @param int    $feed_id         feed id.
	 * @return array Sync response.
	 */
	public function manage_deleted_records( $object, $record_data, $primary_field, $record_response, $feed_id, $deleted_indexes = array() ) {

		$deleted_record_data = array();

		foreach ( $deleted_indexes as $dkey ) {
			$deleted_record = $record_data[ $dkey ];
			if ( isset( $deleted_record['id'] ) ) {
				unset( $deleted_record['id'] );
				array_push( $deleted_record_data, $deleted_record );
			}
		}

		$deleted_record_request_data = $this->prepare_bulk_request_data( $object, $deleted_record_data );

		$deleted_record_response_data = $this->bulk_sync_record(
			$deleted_record_request_data['record_to_create'],
			$deleted_indexes,
			'post',
			$feed_id
		);

		if ( ! empty( $deleted_record_response_data ) ) {
			foreach ( $deleted_record_response_data as $line_key => $line_response ) {
				$record_response[ $line_key ] = $line_response;
			}
		}

		return $record_response;
	}

	// End of class.
}
