<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 *
 * @link   https://wpswings.com/
 * @since 1.0.0
 */

use Automattic\WooCommerce\Internal\DataStores\Orders\OrdersTableDataStore;
use Automattic\WooCommerce\Utilities\OrderUtil;
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Framework_Base {

	/**
	 *  The instance of this class.
	 *
	 * @since    1.0.0
	 * @var      string    $instance    The instance of this class.
	 */
	private static $instance;

	/**
	 * Main Integration_With_Salesforce_Framework_Base Instance.
	 *
	 * Ensures only one instance of Integration_With_Salesforce_Framework_Base is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @return Integration_With_Salesforce_Framework_Base - Main instance.
	 */
	public static function get_instance() {

		if ( is_null( self::$instance ) ) {

			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Constructor function.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
	}

	/**
	 * Main Integration_With_Salesforce_Framework_Base Instance.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return array Array of supported woocommerce objects.
	 */
	public static function supports() {
		return array(
			'product',
			'users',
			'shop_order',
			'wc_user_membership',
			'abandon_carts',
			'order_item',
		);
	}

	/**
	 * Shop Order Fields Mapping which are not in meta keys.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return array
	 */
	public static function shop_order_meta_extras() {
		$order_callbacks = array(
			'order_id'              => 'ID',
			'order_status'          => 'post_status',
			'order_items'           => 'get_items',
			'order_date'            => 'get_date_created',
			'order_paid_date'       => 'get_date_paid',
			'billing_state_name'    => 'get_billing_state_name',
			'billing_country_name'  => 'get_billing_country_name',
			'shipping_state_name'   => 'get_shipping_state_name',
			'shipping_country_name' => 'get_shipping_country_name',
			'coupons_used'          => 'get_coupon_used',
			'coupon_discount'       => 'get_coupon_discount',
			'order_link'            => 'get_edit_order_url',
			'order_subtotal'        => 'get_subtotal',
			'order_number'          => 'get_order_number',
			'order_notes'           => 'get_order_customer_notes',
		);
		/**
		 * Add option to sync in order.
		 *
		 * @since 1.0.3
		 *
		 * @param Array $order_callbacks Order option data.
		 */
		return apply_filters( 'woo_salesforce_order_callback', $order_callbacks );
	}

	/**
	 * User Fields Mapping which are not in meta keys.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return array
	 */
	public static function user_meta_extras() {
		return array(
			'user_email' => 'user_email',
			'username' => 'user_username',
		);
	}

	/**
	 * User Fields Mapping which are not in meta keys.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return array
	 */
	public static function membership_meta_extras() {
		return array(
			'membership_plan' => 'membership_plan',
			'membership_plan_id' => 'membership_plan_id',
			'user_membership_id' => 'user_membership_id',
			'membership_user_id' => 'membership_user_id',
			'membership_status' => 'membership_status',
			'membership_since' => 'membership_since',
			'membership_expiry' => 'membership_expiry',
			'membership_purchased_in' => 'membership_purchased_in',
			'membership_order_date' => 'membership_order_date',
			'membership_order_total' => 'membership_order_total',
			'membership_subscription' => 'membership_subscription',
			'membership_next_bill_on' => 'membership_next_bill_on',
		);
	}

	/**
	 * Products Fields Mapping which are not in meta keys.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return array
	 */
	public static function product_meta_extras() {
		return array(
			'product_id'                => 'ID',
			'product_name'              => 'get_name',
			'product_short_description' => 'get_short_description',
			'product_description'       => 'get_description',
			'product_create_date'       => 'get_date_created',
			'product_type'              => 'get_type',
			'product_display_url'       => 'get_permalink',
			'product_status'            => 'get_product_status',
			'product_publish_status'    => 'get_publish_status',
			'product_attributes'        => 'get_product_attributes',
			'product_categories'        => 'get_product_categories',
			'product_sub_categories'    => 'get_product_sub_categories',
			'product_tags'              => 'get_product_tags',
		);
	}

	/**
	 * Order_items Fields Mapping which are not in meta keys.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return array
	 */
	public function order_item_extras() {
		return array(
			'order_item_product_id' => 'get_order_item_product_id',
		);
	}

	/**
	 * Returns the mapping index from Woo we require.
	 *
	 * @param string $post_type        The object we need index for.
	 * @param string $custom_post_type If required a custom post type.
	 *
	 * @since  1.0.0
	 * @return array|bool          The current mapping step required.
	 */
	public function get_wp_meta( $post_type = false, $custom_post_type = false ) {

		$status = 'publish';
		global $wpdb;

		switch ( $post_type ) {
			case 'product':
			case 'shop_order':
				if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled()  && 'product' != $post_type ) {
					$order_meta = $wpdb->prefix . 'wc_orders_meta';
					$data_store = wc_get_container()->get( OrdersTableDataStore::class );
					$columns_mapping   = $data_store->get_internal_meta_keys();
					$result_order_data = array_unique($columns_mapping);
					$result = $result_order_data;
				
				} else {
					$result = $wpdb->get_col(
						$wpdb->prepare(
							"SELECT DISTINCT( $wpdb->postmeta.meta_key )
						FROM $wpdb->posts
						LEFT JOIN $wpdb->postmeta
						ON $wpdb->posts.ID = $wpdb->postmeta.post_id
						WHERE $wpdb->posts.post_type = %s
						AND $wpdb->postmeta.meta_key != ''",
							$post_type
						)
					);
				}
				if ( ! empty( self::product_meta_extras() ) && 'product' == $post_type ) {
					$result = array_merge( $result, array_keys( self::product_meta_extras() ) );
				} elseif ( ! empty( self::shop_order_meta_extras() ) && 'shop_order' == $post_type ) {
					$result = array_merge(
						$result,
						array_keys( self::shop_order_meta_extras() )
					);
				}

				break;

			case 'wc_user_membership':
				if ( ! empty( self::membership_meta_extras() ) && 'wc_user_membership' === $post_type ) {
					$result = self::membership_meta_extras();
				}
				break;

			case 'users':
				$query = "SELECT distinct $wpdb->usermeta.meta_key FROM 
                $wpdb->usermeta";

				$result = $this->format_query_object(
					mwb_woo_salesforce_get_query_results( $query )
				);

				if ( ! empty( self::user_meta_extras() ) && 'users' === $post_type ) {
					$result = array_merge( $result, array_keys( self::user_meta_extras() ) );
				}
				break;

			case 'abandon_carts':
				$query  = $wpdb->prepare(
					'SELECT `session_value`, `session_id` FROM
                %s',
					'wp_woocommerce_sessions'
				);
				$result = $this->format_query_object(
					mwb_woo_salesforce_get_query_results( $query )
				);
				break;

			case 'custom':
				$result = $wpdb->get_col(
					$wpdb->prepare(
						"SELECT DISTINCT( $wpdb->postmeta.meta_key )
	                FROM $wpdb->posts
	                LEFT JOIN $wpdb->postmeta
	                ON $wpdb->posts.ID = $wpdb->postmeta.post_id
	                WHERE $wpdb->posts.post_type = %s
	                AND $wpdb->postmeta.meta_key != ''",
						$custom_post_type
					)
				);
				break;
		}

		return ! empty( $result ) ? $result : false;
	}

	/**
	 * Returns the mapping index from Woo we require.
	 *
	 * @param array $array The query object result.
	 *
	 * @since 1.0.0
	 *
	 * @return array The formatted data as raw array
	 */
	public function format_query_object( $array = false ) {

		if ( empty( $array ) ) {
			return;
		}

		$formatted_array = array();

		foreach ( $array as $key => $value ) {
			$dataset = array_values( $value );
			if ( ! empty( $dataset[0] ) ) {
				array_push( $formatted_array, $dataset[0] );
			}
		}

		return $formatted_array;
	}

	/**
	 * Returns the requested data values for required index from Woo we require.
	 *
	 * @param string $obj_type The post type.
	 * @param string $key      The post meta key.
	 * @param string $id       The post object id.
	 * @param string $nonce    Nonce.
	 *
	 * @since 1.0.0
	 *
	 * @return string               The post meta values.
	 */
	public function get_prop_value( $obj_type = false, $key = false, $id = false, $nonce = false, $item_id = false ) {

		if ( false == $key || false == $id ) {
			return;
		}

		if ( empty( $obj_type ) || ! in_array( $obj_type, self::supports() ) ) {
			return;
		}

		if ( ! wp_verify_nonce( sanitize_text_field( wp_unslash( $nonce ) ), 'mwb-create-prop' ) ) {
			return;
		}

		$response = '';

		$admin_instance = new Integration_With_Salesforce_Admin( 'Salesforce Integration for WooCommerce', '1.2.1' );

		switch ( $obj_type ) {

			case 'users':
				if ( array_key_exists( $key, self::user_meta_extras() ) ) {

					$callback = self::user_meta_extras()[ $key ];
					if ( 'ID' == $callback ) {
						$response = $id;
					} elseif ( 'user_email' === $callback ) {
						$user = get_user_by( 'ID', $id );
						if ( $user ) {
							$response = $user->user_email;
						} else {
							$response = '';
						}
					} elseif ( 'user_username' === $callback ) {
						$user = get_userdata( $id );
						if ( $user ) {
							$response = $user->user_login;
						} else {
							$response = '';
						}

					} else {
						$user = get_user_by( 'id', $id );
					}
				} else {
					if ( ! empty( $_POST[ $key ] ) ) {

						if ( is_array( $_POST[ $key ] ) ) {
							$posted_value = array_map( 'sanitize_text_field', wp_unslash( $_POST[ $key ] ) );
						} else {
							$posted_value = sanitize_text_field( wp_unslash( $_POST[ $key ] ) );
						}
					}

					$response = ! empty( $posted_value ) ? $posted_value : get_user_meta( $id, $key, true );
				}
				break;

			case 'shop_order':
				if ( array_key_exists( $key, self::shop_order_meta_extras() ) ) {

					$callback = self::shop_order_meta_extras()[ $key ];
					if ( 'ID' == $callback ) {
						$response = $id;
					} elseif ( 'post_status' == $callback ) {
						$response = get_post( $id )->post_status;
					} elseif ( 'get_date_created' == $callback ) {
						$response = wc_get_order( $id )->get_date_created()->date_i18n( 'Y-m-d' );
					} elseif ( 'get_date_paid' == $callback ) {
						$paid_date = wc_get_order( $id )->get_date_paid();
						if ( $paid_date ) {
							$response = $paid_date->date_i18n( 'Y-m-d' );
						} else {
							$response = '';
						}
					} elseif ( 'get_billing_country_name' == $callback ) {
						$order = wc_get_order( $id );
						if ( ! empty( $order->billing_country ) ) {
							$response = WC()->countries->countries[ $order->billing_country ];
						} else {
							$response = '';
						}
					} elseif ( 'get_shipping_country_name' == $callback ) {
						$order = wc_get_order( $id );
						if ( ! empty( $order->shipping_country ) ) {
							$response = WC()->countries->countries[ $order->shipping_country ];
						} else {
							$response = '';
						}
					} elseif ( 'get_billing_state_name' == $callback ) {
						$billing_country = wps_salesforce_get_meta_data( $id, '_billing_country', true );
						if ( ! empty( $billing_country ) ) {
							$billing_state = wps_salesforce_get_meta_data( $id, '_billing_state', true );
							if ( ! empty( $billing_state ) ) {
								$country_states = WC()->countries->get_states( $billing_country );
								$response       = $country_states[ $billing_state ];
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_shipping_state_name' == $callback ) {
						$shipping_country = wps_salesforce_get_meta_data( $id, '_shipping_country', true );
						if ( ! empty( $shipping_country ) ) {
							$shipping_state = wps_salesforce_get_meta_data( $id, '_shipping_state', true );
							if ( ! empty( $shipping_state ) ) {
								$country_states = WC()->countries->get_states( $shipping_country );
								$response       = $country_states[ $shipping_state ];
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_coupon_used' == $callback ) {
						$order        = wc_get_order( $id );
						$coupons      = $order->get_items( 'coupon' );
						$coupons_used = array();

						if ( ! empty( $coupons ) ) {
							foreach ( $coupons as $key => $coupon_obj ) {
								$coupons_used[] = $coupon_obj->get_name();
							}
						}

						if ( ! empty( $coupons_used ) ) {

							$response = implode( ',', $coupons_used );
						} else {
							$response = '';
						}
					} elseif ( 'get_coupon_discount' == $callback ) {
						$order          = wc_get_order( $id );
						$coupons_amount = $order->get_total_discount();

						if ( ! empty( $coupons_amount ) ) {

							$response = $coupons_amount;
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_plan_id' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
								$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
								$user_membership           = wc_memberships_get_user_membership( $user_membership_post_id );
								$membership_plan_id        = $user_membership->id;
								$response                  = 'woo_plan_' . $membership_plan_id;
							} else {
								$response = '';

							}
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_plan' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = $membership->plan->name . ' #' . $membership->plan->id;
						}
					} elseif ( 'get_membership_create_time' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
								$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
							$membership_created_time   = date_i18n( wc_time_format(), strtotime( wps_salesforce_get_meta_data( $user_membership_post_id, '_start_date', true ) ) );
							$response                  = $membership_created_time;
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_create_date' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
								$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
							$membership_created_date   = date_i18n( wc_date_format(), strtotime( wps_salesforce_get_meta_data( $user_membership_post_id, '_start_date', true ) ) );
							$response                  = $membership_created_date;
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_end_date' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
								$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
							$membership_end_date       = date_i18n( wc_date_format(), strtotime( wps_salesforce_get_meta_data( $user_membership_post_id, '_start_date', true ) ) );
							$response                  = $membership_end_date;
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_end_time' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
							$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
							$membership_end_time       = date_i18n( wc_time_format(), strtotime( wps_salesforce_get_meta_data( $user_membership_post_id, '_start_date', true ) ) );
							$response                  = $membership_end_time;
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_user_membership_id' === $callback ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = $membership->id;
						}
					} elseif ( 'get_membership_user_id' === $callback ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = $membership->user_id;
						}
					} elseif ( 'get_membership_status' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = $membership->status;
						}
					} elseif ( 'get_membership_product_id' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
								$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
							$membership_product_id     = wps_salesforce_get_meta_data( $user_membership_post_id, '_product_id', true );
							$response                  = $membership_product_id;
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_product_name' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$prod_meta = ( get_post_meta( $id ) );
						if ( array_key_exists( '_wc_memberships_access_granted', $prod_meta ) ) {
							$membership_attached_array = ( get_post_meta( $id, '_wc_memberships_access_granted' ) );
							if (is_array($membership_attached_array)) {
								$user_membership_post_id   = ( array_key_first( $membership_attached_array[0] ) );
							$membership_product_id     = wps_salesforce_get_meta_data( $user_membership_post_id, '_product_id', true );
							$product                   = wc_get_product( $membership_product_id );
							$response                  = $product->get_name();
							} else {
								$response = '';
							}
						} else {
							$response = '';
						}
					} elseif ( 'get_membership_since' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = $membership->get_start_date();
						}
					} elseif ( 'get_membership_expiry' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = $membership->get_end_date();
							if ( empty( $response ) ) {
								$response = 'Never';
							}
						}
					} elseif ( 'get_membership_purchased_in' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = 'Order ' . $membership->get_order_id();
						}
					} elseif ( 'get_membership_order_date' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$response = gmdate( 'Y/m/d', get_post_time( 'U', true, $membership->get_order_id() ) );
						}
					} elseif ( 'get_membership_order_total' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$mem_ord = wc_get_order( $membership->get_order_id() );
							$response = $mem_ord->get_total();
						}
					} elseif ( 'get_membership_subscription' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$subscriptions_ids = wcs_get_subscriptions_for_order( $membership->get_order_id(), array( 'order_type' => 'any' ) );
							if ( !empty( $subscriptions_ids ) ) {
								foreach ( $subscriptions_ids as $subs_id => $subs_v ) {
									$response = $subs_id;
								}
							}
						}
					} elseif ( 'get_membership_next_bill_on' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$ord = wc_get_order($id);
						$user_id = $ord->get_customer_id();
						$membership_attached_array = wc_memberships_get_user_active_memberships($user_id);
						foreach ($membership_attached_array as $membership) {
							$subscriptions_ids = wcs_get_subscriptions_for_order( $membership->get_order_id(), array( 'order_type' => 'any' ) );
							if ( !empty( $subscriptions_ids ) ) {
								foreach ( $subscriptions_ids as $subs_id => $subs_v ) {
									$next_payment_date = ! empty( $subs_v->get_date( 'schedule_next_payment' ) ) ? $subs_v->get_date( 'schedule_next_payment' ) : '';

									if ( ! empty( $next_payment_date ) ) {

										$response = gmdate( 'Y-m-d', strtotime( $next_payment_date ) );
									}
								}
							}
						}
					} elseif ( 'get_order_customer_notes' === $callback ) {
						$order = wc_get_order( $id );
						$response = $order->get_customer_note();
					} elseif ( 'get_subscription_id' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$response = $subscription_id;
						}
					} elseif ( 'get_subscription_status' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$response = $subscription_obj->get_status();
						}
					} elseif ( 'get_subscription_customer' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$user = get_user_by( 'id', $subscription_obj->get_customer_id() );
							$response = $user->user_login;
						}
					} elseif ( 'get_subscription_payment_method' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$response = $subscription_obj->get_payment_method_title();
						}
					} elseif ( 'get_subscription_payment_schedule' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$response = $subscription_obj->get_billing_interval() . ' ' . $subscription_obj->get_billing_period();
						}
					} elseif ( 'get_subscription_start_date' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$subs_start_date = ! empty( $subscription_obj->get_date( 'schedule_start' ) ) ? $subscription_obj->get_date( 'schedule_start' ) : '';

							if ( ! empty( $subs_start_date ) ) {

								$response = gmdate( 'Y-m-d H:i:s', strtotime( $subs_start_date ) );
							}
						}
					} elseif ( 'get_subscription_end_date' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$subs_end_date = ! empty( $subscription_obj->get_date( 'schedule_end' ) ) ? $subscription_obj->get_date( 'schedule_end' ) : '';

							if ( ! empty( $subs_end_date ) ) {

								$response = gmdate( 'Y-m-d H:i:s', strtotime( $subs_end_date ) );
							} else {
								$response = 'Never';
							}
						}
					} elseif ( 'get_subscription_next_payment' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$next_payment_date = ! empty( $subscription_obj->get_date( 'schedule_next_payment' ) ) ? $subscription_obj->get_date( 'schedule_next_payment' ) : '';

							if ( ! empty( $next_payment_date ) ) {

								$response = gmdate( 'Y-m-d H:i:s', strtotime( $next_payment_date ) );
							}
						}
					} elseif ( 'get_subscription_information' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subs_info = 'Acepto las condiciones de uso del servicio : ' . get_post_meta($id, '_billing_acepto_las_condiciones_de_uso_del_servicio_', true);
						$subs_info .= ', Acepto suscribirme al boletín diario : ' . get_post_meta($id, '_billing_acepto_suscribirme_al_boletin_diario', true);
						$subs_info .= ', Acepto suscribirme al boletín premium (solo para miembros) : ' . get_post_meta($id, '_billing_acepto_suscribirme_al_boletin_premium_solo_para_miembros', true);
						if ( !empty( get_post_meta($id, '_billing_acepto_las_condiciones_de_uso_del_servicio_', true) ) || !empty( get_post_meta($id, '_billing_acepto_suscribirme_al_boletin_diario', true) ) || !empty( get_post_meta($id, '_billing_acepto_suscribirme_al_boletin_premium_solo_para_miembros', true) ) ) {
							$response = $subs_info;
						}
					} elseif ( 'get_subscription_variation_id' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$items = $subscription_obj->get_items();
							foreach ( $items as $i_key => $i_value ) {
								$i_data = $i_value->get_data();
								$var_id = $i_data['variation_id'];
								if ( 0 != $var_id && !empty( $var_id ) ) {
									$response = $var_id;
								}
							}
						}
					} elseif ( 'get_subscription_item_name' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$items = $subscription_obj->get_items();
							foreach ( $items as $i_key => $i_value ) {
								$i_data = $i_value->get_data();
								$item_name = $i_data['name'];
								$response = $item_name;
							}
						}
					} elseif ( 'get_subscription_item_price' == $callback && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$subscriptions_ids = wcs_get_subscriptions_for_order( $id, array( 'order_type' => 'any' ) );
						foreach ( $subscriptions_ids as $subscription_id => $subscription_obj ) {
							$items = $subscription_obj->get_items();
							foreach ( $items as $i_key => $i_value ) {
								$i_data = $i_value->get_data();
								$item_price = $i_data['total'];
								$response = $item_price;
							}
						}
					} else {
						$response = wc_get_order( $id )->$callback();
					}
				} else {

					if ( ! empty( $_POST[ $key ] ) ) {

						if ( is_array( $_POST[ $key ] ) ) {
							$posted_value = array_map( 'sanitize_text_field', wp_unslash( $_POST[ $key ] ) );
						} else {
							$posted_value = sanitize_text_field( wp_unslash( $_POST[ $key ] ) );
						}
					}

					if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
						$order = wc_get_order( $id );
						
						if ( $order ) {

							$response = ! empty( $posted_value ) ? $posted_value : $order->get_meta($key);
						} else {
							$response = ! empty( $posted_value ) ? $posted_value : get_post_meta( $id, $key, true );
						}

					

					} else {

						$response = ! empty( $posted_value ) ? $posted_value : get_post_meta( $id, $key, true );
					}

				}
				break;

			case 'wc_user_membership':
				if ( array_key_exists( $key, self::membership_meta_extras() ) ) {
					$callback = self::membership_meta_extras()[ $key ];
					if ( 'membership_plan' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							$response = $membership->plan->name . ' #' . $membership->plan->id;
						}
					} elseif ( 'membership_plan_id' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							$response = 'woo_plan_' . $membership->plan->id;
						}
					} elseif ( 'user_membership_id' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$response = $id;
					} elseif ( 'membership_user_id' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							$response = $membership->user_id;
						}
					} elseif ( 'membership_status' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							$response = $membership->status;
						}
					} elseif ( 'membership_since' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							$response = $membership->get_start_date();
						}
					} elseif ( 'membership_expiry' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							$response = $membership->get_end_date();
							if ( empty( $response ) ) {
								$response = 'Never';
							}
						}
					} elseif ( 'membership_purchased_in' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							if ( !empty( $membership->get_order_id() ) ) {
								$response = 'Order ' . $membership->get_order_id();
							}
						}
					} elseif ( 'membership_order_date' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							if ( !empty( $membership->get_order_id() ) ) {
								$response = gmdate( 'Y/m/d', get_post_time( 'U', true, $membership->get_order_id() ) );
							}
						}
					} elseif ( 'membership_order_total' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							if ( !empty( $membership->get_order_id() ) ) {
								$mem_ord = wc_get_order( $membership->get_order_id() );
								if ( $mem_ord instanceof WC_Order ) {
									$response = $mem_ord->get_total();
								}
							}
						}
					} elseif ( 'membership_subscription' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							if ( !empty( $membership->get_order_id() ) ) {
								$subscriptions_ids = wcs_get_subscriptions_for_order( $membership->get_order_id(), array( 'order_type' => 'any' ) );
								if ( !empty( $subscriptions_ids ) ) {
									foreach ( $subscriptions_ids as $subs_id => $subs_v ) {
										$response = $subs_id;
									}
								}
							}
						}
					} elseif ( 'membership_next_bill_on' === $callback && $admin_instance->wps_is_woocommerce_membership_plugin_active() && $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
						$membership = wc_memberships_get_user_membership( $id );
						if ( ! is_wp_error( $membership ) && $membership ) {
							if ( !empty( $membership->get_order_id() ) ) {
								$subscriptions_ids = wcs_get_subscriptions_for_order( $membership->get_order_id(), array( 'order_type' => 'any' ) );
								if ( !empty( $subscriptions_ids ) ) {
									foreach ( $subscriptions_ids as $subs_id => $subs_v ) {
										$next_payment_date = ! empty( $subs_v->get_date( 'schedule_next_payment' ) ) ? $subs_v->get_date( 'schedule_next_payment' ) : '';

										if ( ! empty( $next_payment_date ) ) {

											$response = gmdate( 'Y-m-d', strtotime( $next_payment_date ) );
										}
									}
								}
							}
						}
					}
				}
				break;

			case 'product':
				if ( array_key_exists( $key, self::product_meta_extras() ) ) {

					$callback = self::product_meta_extras()[ $key ];

					if ( 'ID' == $callback ) {
						$response = $id;

					} elseif ( 'get_product_status' == $callback ) {
						$product = get_post( $id );
						if ( ! empty( $product ) ) {
							$response = $this->mwb_salesforce_get_product_status( $product, $id );
							// $response = $product->post_status;
						} else {
							$response = '';
						}
					} elseif ( 'get_date_created' == $callback ) {
						if ( wc_get_product( $id ) ) {
							$product_created_date = wc_get_product( $id )->get_date_created();
							if ( $product_created_date ) {
								$response = $product_created_date->date_i18n( 'Y-m-d' );
							} else {
								$response = '';
							}
						}
					} elseif ( 'get_publish_status' == $callback ) {
						$product = get_post( $id );
						if ( ! empty( $product ) ) {
							$product_status = $this->mwb_salesforce_get_product_status( $product, $id );
							$response       = ( 'publish' == $product_status );
						}
					} elseif ( 'get_product_attributes' == $callback ) {
						$prod_attributes = array();
						$product         = wc_get_product( $id );
						if ( $product->is_type( 'variation' ) ) {
							$product_attributes = $product->get_attributes();

							if ( ! empty( $product_attributes ) ) {
								foreach ( $product_attributes as $key => $value ) {
									$key = str_replace( 'pa_', '', $key );
									array_push( $prod_attributes, ucwords( $key ) . '-' . ucwords( $value ) );
								}
							}
						} elseif ( $product->is_type( 'variable' ) ) {
							$product_attributes = $product->get_attributes();

							if ( ! empty( $product_attributes ) ) {
								foreach ( $product_attributes as $key => $value ) {
									$key = str_replace( 'pa_', '', $key );
									array_push( $prod_attributes, ucwords( $key ) . '-' . ucwords( $value ) );
								}
							}
						} else {
							$key                = '_product_attributes';
							$product_attributes = get_post_meta( $id, $key, true );
							if ( ! empty( $product_attributes ) ) {
								foreach ( $product_attributes as $key => $value ) {
									$attribute_value                     = $product->get_attribute( $key );
									$product_attributes[ $key ]['value'] = $attribute_value;
									$product_attributes[ $key ]['name']  = str_replace( 'pa_', '', $product_attributes[ $key ]['name'] );
									array_push( $prod_attributes, ucwords( $product_attributes[ $key ]['name'] ) . '-' . $product_attributes[ $key ]['value'] );
								}
							}
						}
						$response = implode( ', ', $prod_attributes );

					} elseif ( 'get_product_categories' == $callback ) {

						$terms            = get_the_terms( $id, 'product_cat' );
						$product_cat_name = array();
						foreach ( $terms as $term ) {
							$product_cat_name[] = $term->name;
						}
						$response = ( implode( ',', $product_cat_name ) );
					} elseif ( 'get_product_sub_categories' == $callback ) {
						$terms            = get_the_terms( $id, 'product_cat' );
						$product_cat_name = array();
						foreach ( $terms as $term ) {
							if ( ! empty( $term->parent ) ) {
								$product_cat_name[] = $term->name;
							}
						}
						$response = ( implode( ',', $product_cat_name ) );
					} elseif ( 'get_product_tags' == $callback ) {
						$parent_id = wp_get_post_parent_id( $id, 'product_type' );
						if ( ! empty( $parent_id ) ) {
							$terms = wp_get_object_terms( $parent_id, 'product_tag', array( 'fields' => 'names' ) );
						} else {
							$terms = wp_get_object_terms( $id, 'product_tag', array( 'fields' => 'names' ) );
						}
						$response = implode( ',', $terms );
					} else {
						$product = wc_get_product( $id );
						if ( ! empty( $product ) ) {
							$response = $product->$callback();
						}
					}
				} else {
					if ( ! empty( $_POST[ $key ] ) ) {

						if ( is_array( $_POST[ $key ] ) ) {
							$posted_value = array_map( 'sanitize_text_field', wp_unslash( $_POST[ $key ] ) );
						} else {
							$posted_value = sanitize_text_field( wp_unslash( $_POST[ $key ] ) );
						}
					}

					$response = ! empty( $posted_value ) ? $posted_value : get_post_meta( $id, $key, true );
				}
				break;
			case 'order_item':
				if ( array_key_exists( $key, $this->order_item_extras() ) ) {

					$callback = $this->order_item_extras()[ $key ];
					if ( 'get_order_item_product_id' == $callback ) {
						$item = new WC_Order_Item_Product($item_id);
						$product = $item->get_product();
						$response = $product->get_id();
					} else {
						$response = wc_get_order_item_meta( $item_id, $key, true );
					}
				} elseif ( strpos($key, 'mwb_salesforce_') !== false ) {
						$response = get_post_meta( $id, $key, true );
				} else {
				 $response = wc_get_order_item_meta( $item_id, $key, true );
				}
				break;
			default:
				// code...
				break;
		}

		return $response;
	}


	/**
	 * Returns the product status.
	 *
	 * @param  object $product Product.
	 * @param  int    $id      Product id.
	 * @return string
	 */
	public function mwb_salesforce_get_product_status( $product, $id ) {
		$product_status = '';
		if ( 'product_variation' == $product->post_type ) {
			$variation = wc_get_product( $id );
			if ( ! empty( $variation ) ) {
				$parent_id      = $variation->get_parent_id();
				$parent_product = get_post( $parent_id );
				if ( ! empty( $parent_product ) ) {
					$product_status = $parent_product->post_status;
				}
			}
		} else {
			$product_status = $product->post_status;
		}

		return $product_status;
	}

	/**
	 * Current Woo meta keys with Labels.
	 *
	 * @param array $dataset array for woo keys.
	 *
	 * @since 1.0.0
	 *
	 * @return array - Current Woo meta keys with Labels to Woo keys.
	 */
	public function parse_labels( $dataset ) {

		if ( ! empty( $dataset ) && is_array( $dataset ) ) {
			foreach ( $dataset as $key => $value ) {
				if ( ! empty( $value ) && is_array( $value ) ) {
					foreach ( $value as $label_key => $label_value ) {

						// Initialise the key via wp index.
						$dataset[ $key ][ $label_value ] = false;

						// Unset the numeric index.
						unset( $dataset[ $key ][ $label_key ] );

						// Create Wp index.
						$label_key = $label_value;

						// Create Wp Labels.
						$label_value                   = str_replace( '_', ' ', $label_value );
						$label_value                   = str_replace( '-', ' ', $label_value );
						$label_value                   = ucwords( $label_value );
						$dataset[ $key ][ $label_key ] = $label_value;
					}
				}
			}
		}
		return $dataset;
	}

	/**
	 * Replace the occurence within the string only once.
	 *
	 * @param string $from    The sub-string before replace.
	 * @param string $to      The sub-string after replace.
	 * @param string $content The string from which we operate.
	 *
	 * @since  1.0.0
	 * @return array - Current Woo meta keys with Labels to Woo keys.
	 */
	public function str_replace_first( $from, $to, $content ) {
		$from = '/' . preg_quote( $from, '/' ) . '/';

		return preg_replace( $from, $to, $content, 1 );
	}

	/**
	 * Get other associated salesforce object in required format.
	 *
	 * @param string $woo_obj_type   The WP post type.
	 * @param string $lookup_feed_id The Feed id to get lookup request.
	 * @param string $lookup_type    The CRM Object required id.
	 * @param string $woo_id         The WP post id.
	 * @param array  $extra_args      Extra args for order item.
	 * @since  1.0.0
	 * @return array - Current Woo meta keys with Labels to Woo keys.
	 */
	public function resolve_lookup( $woo_obj_type, $lookup_feed_id, $lookup_type, $woo_id, $extra_args = array() ) {

		$lookup_id = '';
		if ( 'publish' == get_post_status( $lookup_feed_id ) ) {
			
			$request_module = Integration_With_Salesforce::get_integration_module( 'request' );
			$lookup_feed_obj = $request_module->get_woo_object_based_on_event( $lookup_feed_id );
			$crm_object = $this->get_feed( $lookup_feed_id, 'crm_object' );

			if ( ( 'users' == $lookup_feed_obj ) && ( ( 'Account' == $crm_object ) || ( 'Contact' == $crm_object ) || ( 'Lead' == $crm_object ) ) ) {
				$order = wc_get_order($woo_id );
				$user_id = $order->get_customer_id();
				$lookup_id = get_user_meta( $user_id, 'mwb_salesforce_' . $crm_object . '_id', true );

			} elseif ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
				$order = wc_get_order($woo_id);
				if ( $order ) {

					$lookup_id = $order->get_meta('mwb_salesforce_' . $crm_object . '_id');
				} else {
					
					$lookup_id = wps_salesforce_get_meta_data(
						$woo_id,
						'mwb_salesforce_' . $crm_object . '_id',
						true
					);
					
				}
				
			} else {
			$lookup_id = wps_salesforce_get_meta_data(
				$woo_id,
				'mwb_salesforce_' . $crm_object . '_id',
				true
			);
			}
			// Check if lookup type is pricebook entry.
			// Get pricebook entry id from product.

			if (
			empty( $lookup_id )
			&& 'PricebookEntryId' === $lookup_type
			&& ! empty( $extra_args['product_id'] )
			) {
				$lookup_id = wps_salesforce_get_meta_data(
					$extra_args['product_id'],
					'mwb_salesforce_' . $crm_object . '_id',
					true
				);
			}
		}
		
		return ! empty( $lookup_id ) ? $lookup_id : '';
	}

	/**
	 * Returns the mapping step we require.
	 *
	 * @param string $obj_type The object post type.
	 * @param string $feed_id  The mapped feed for associated objects.
	 * @param string $obj_id   The object post id.
	 * @param array  $extra_args Extra args for order item.
	 *
	 * @since  1.0.0
	 * @return array The current mapping step required.
	 */
	public function get_request( $obj_type = false, $feed_id = false, $obj_id = false, $extra_args = array(), $item_id = false ) {
		if ( false == $obj_type || false == $feed_id || false == $obj_id ) {
			return;
		}

		$primary_field = $this->get_feed(
			$feed_id,
			'primary_field'
		);

		$feed = $this->get_feed( $feed_id );

		// Add reference id for update.
		$crm_object = $this->get_feed( $feed_id, 'crm_object' );

		// $crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		// $object_fields  = $crm_api_module->get_object_fields( $crm_object, true );
		// $api_name       = array_column( $object_fields, 'name' );

		// foreach ( $feed as $key => $mapping ) {
		// if ( ! in_array( $key, $api_name ) ) {
		// unset( $feed[ $key ] );
		// }
		// }.

		$feed['id'] = array(
			'field_type'  => 'standard_field',
			'field_value' => 'mwb_salesforce_' . $crm_object . '_id',
		);

		// Add reference id for update.

		if ( empty( $feed ) ) {
			return false;
		}

		// Process Feeds.
		$response = array();

		if ( 'users' == $obj_type ) {
			$response['user_id'] = $obj_id;
		} else {
			$response['post_id'] = $obj_id;
		}

		foreach ( $feed as $k => $mapping ) {

			$field_type = ! empty( $mapping['field_type'] ) ?
						$mapping['field_type'] : 'standard_field';

			switch ( $field_type ) {

				case 'standard_field':
					$is_status_mapping_required = ! empty( $mapping['use_status_mapping'] )
					&& 'yes' == $mapping['use_status_mapping']
					? true :
					false;

					$field_format = ! empty( $mapping['field_value'] ) ?
					$mapping['field_value'] : '';

					// If lookup field.
					if ( 0 === strpos( $field_format, 'feeds_' ) ) {

						$lookup_feed_id = str_replace( 'feeds_', '', $field_format );
						$field_value    = '';
						$field_value    = $this->resolve_lookup(
							$obj_type,
							$lookup_feed_id,
							$k,
							$obj_id,
							$extra_args
						);
						
					} elseif ( true == $is_status_mapping_required ) {

						if ( OrderUtil::custom_orders_table_usage_is_enabled() ) {
							$status_mapping = $this->get_feed( $feed_id, 'status_mapping' );
							$ord = wc_get_order( $obj_id );
							$order_status = 'wc-' . $ord->get_status();
							$field_value = ! empty( $status_mapping[ $order_status ] )
							? $status_mapping[ $order_status ]
							: $this->get_default_status( $crm_object );
						} else {
							$status_mapping = $this->get_feed( $feed_id, 'status_mapping' );
							$order_status   = $this->get_prop_value(
								$obj_type,
								'order_status',
								$obj_id,
								wp_create_nonce( 'mwb-create-prop' )
							);
	
							$field_value = ! empty( $status_mapping[ $order_status ] )
							? $status_mapping[ $order_status ]
							: $this->get_default_status( $crm_object );
						}

					} elseif ( 'Pricebook2Id' == $k ) { // key is pricebookid.
						$field_value = get_option( 'mwb_salesforce_selected_pricebook', '' );
					} else { // Just a standard meta key.
						

						$obj_required = strtok( $field_format, '_' );

						$meta_key = $this->str_replace_first(
							$obj_type . '_',
							'',
							$field_format
						);

						$field_value = '';

						if ( 'order_item' == $obj_type ) {
							$field_value = $this->get_prop_value(
								$obj_type,
								$meta_key,
								$obj_id,
								wp_create_nonce( 'mwb-create-prop' ),
								$item_id
							);
						} else {
							$field_value = $this->get_prop_value(
								$obj_type,
								$meta_key,
								$obj_id,
								wp_create_nonce( 'mwb-create-prop' )
							);
						}
						if ( 'id' === $k && empty( $field_value ) && ( 'Account' === $crm_object || 'Lead' === $crm_object || 'OrderItem' == $crm_object ) ) {
							$field_value = $this->may_be_get_field_value_from_existing_orders( $obj_id, $meta_key );
						}

						if ( 'id' === $k ) {
							if ( 'Membership__c' == $crm_object ) {
								$crm_integration_api = new Integration_With_Salesforce_Api();
								$duplicate_string = '';
								$duplicate_data = array( $response[$primary_field] );
								if ( ! empty( $duplicate_data ) ) {
									array_walk(
										$duplicate_data,
										function ( &$item ) {
											$item = "'" . $item . "'";
										}
									);
									$duplicate_string = '( ' . implode( ' , ', $duplicate_data ) . ' )';
								}
								$duplicate_query = sprintf(
									'SELECT Id, %s FROM %s WHERE %s IN %s',
									$primary_field,
									$crm_object,
									$primary_field,
									$duplicate_string
								);
								$query_response  = $crm_integration_api->query_records( $duplicate_query );
								if ( !empty( $query_response ) ) {
									$field_value = $query_response[0]['Id'];
								} else {
									$field_value = '';
								}
							}
						}

					}
					break;

				case 'custom_value':
					$field_key = ! empty( $mapping['custom_value'] ) ?
						$mapping['custom_value'] : '';

					preg_match_all( '/{(.*?)}/', $field_key, $dynamic_strings );

					if ( ! empty( $dynamic_strings[1] ) ) {
						$dynamic_values = $dynamic_strings[1];
						foreach ( $dynamic_values as $key => $value ) {
							$prefix       = $obj_type . '_';
							$field_format = $this->str_replace_first(
								$prefix,
								'',
								$value
							);
							if ( 'order_item' == $obj_type ) {
								$field_value = $this->get_prop_value(
									$obj_type,
									$field_format,
									$obj_id,
									wp_create_nonce( 'mwb-create-prop' ),
									$item_id
								);
							} else {
								$field_value = $this->get_prop_value(
									$obj_type,
									$field_format,
									$obj_id,
									wp_create_nonce( 'mwb-create-prop' )
								);
							}

							$substr = '{' . $prefix . $field_format . '}';

							$field_key = str_replace(
								$substr,
								$field_value,
								$field_key
							);

							$field_value = $field_key;
						}
					} else {
						
						
						$field_value = ! empty( $mapping['custom_value'] ) ?
						$mapping['custom_value'] : '';
					}
					
					
					break;
			}
				
				// $response[ $k ] = ! empty( $field_value ) ? $field_value : '';
			$response[ $k ] = $field_value;
		}
		
			
		if ( 'Account' === $crm_object || 'Contact' === $crm_object || 'Lead' === $crm_object ) {
			$response['related_order_ids'] = $this->may_be_get_field_value_from_existing_orders( $obj_id );
		}
			
		
		/**
		 * Filters the value of request data for specific object.
		 *
		 * @since 1.0.3
		 *
		 * @param Array $response Current request data.
		 * @param int   $obj_id  Woo object id.
		 * @param int   $feed_id Feed id.
		 */
		$response = apply_filters(
			'mwb_woo_salesforce_' . $crm_object . '_request_data',
			$response,
			$obj_id,
			$feed_id
		);

		/**
		 * Filters the value of request data for all objects.
		 *
		 * @since 1.0.3
		 *
		 * @param Array    $response Current request data.
		 * @param int      $obj_id  Woo object id.
		 * @param int      $feed_id Feed id.
		 * @param string   $feed_id CRM object.
		 */
		$response = apply_filters(
			'mwb_woo_salesforce_request_data',
			$response,
			$obj_id,
			$feed_id,
			$crm_object
		);
		return $response;
	}

	/**
	 * Returns the mapping step we require.
	 *
	 * @param string $feed_type The CRM Object post type.
	 *
	 * @since 1.0.0
	 *
	 * @return bool|array single or multiple feed ids.
	 */
	public function get_object_feed( $feed_type = false ) {
		if ( empty( $feed_type ) ) {
			return false;
		}

		$args = array(
			'post_type'   => 'mwb_salesforce_feed',
			'post_status' => 'publish',
			'fields'      => 'ids',
			'meta_query'  => array(
				'relation' => 'AND',
				array(
					'key'     => 'crm_object',
					'compare' => '=',
					'value'   => $feed_type,
				),
			),
		);

		$feeds = get_posts( $args );

		if ( ! empty( $feeds ) ) {
			if ( count( $feeds ) > 1 ) {
				$feed_id = $feeds;
			} else {
				$feed_id = reset( $feeds );
			}

			return $feed_id;
		}
		return false;
	}

	/**
	 * Get title of a particur feed.
	 *
	 * @param int $feed_id Id of feed.
	 *
	 * @return array.
	 */
	public function get_feed_title( $feed_id ) {
		$title = esc_html( 'Feed #' ) . $feed_id;
		$feed  = get_post( $feed_id );
		$title = ! empty( $feed->post_title ) ? $feed->post_title : $title;
		return $title;
	}

	/**
	 * Check and include admin view file
	 *
	 * @param string      $file_path Relative path of file.
	 * @param array       $params Array of extra params.
	 * @param bool|string $base   Base path of file.
	 */
	public static function load_template( $file_path, $params = array(), $base = false ) {

		try {

			$result = wc_get_template(
				$file_path,
				$params,
				'',
				$base ? $base : plugin_dir_path( __FILE__ )
			);

		} catch ( \Throwable $th ) {
			echo esc_html( $th->getMessage() );
			wp_die();
		}
	}

	/**
	 * Check if instant sync is enable.
	 *
	 * @return boolean
	 */
	public static function is_instant_sync_enable() {
		$enable = get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-sync', 'no' );
		$enable = ( 'yes' === $enable );
		return $enable;
	}

	/**
	 * Check if log is enable.
	 *
	 * @return boolean
	 */
	public static function is_log_enable() {
		$enable = get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-log', 'yes' );
		$enable = ( 'yes' === $enable );
		return $enable;
	}

	/**
	 * Get general settings.
	 *
	 * @return array Array of settings.
	 */
	public static function get_general_settings() {

		$admin_instance = new Integration_With_Salesforce_Admin( 'Salesforce Integration for WooCommerce', '1.2.1' );
		
		$settings[] = array(
			'type' => 'title',
		);

		$settings[] = array(
			'title'    => esc_html__( 'Enable instant sync', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-sync',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-sync', 'no' ),
			'desc_tip' => __( 'Enable instant sync to sync data instantly according to feed event.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'    => esc_html__( 'Enable background sync', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-background-sync',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-background-sync-allowed', 'no' ),
			'desc_tip' => __( 'Enable background sync for syncing data automatically in every 5 mins.', 'integration-with-salesforce' ) . '<br/>' . __( 'It is recommended to use either background sync or instant sync at a time.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'    => esc_html__( 'Enable logging', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-log',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-log', 'yes' ),
			'desc_tip' => __( 'Enable log creation to check status of synced data.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'             => esc_html__( 'Delete log entry after x days', 'integration-with-salesforce' ),
			'id'                => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-delete-log',
			'type'              => 'number',
			'value'             => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-delete-log', '30' ),
			'custom_attributes' => array( 'min' => 7 ),
			'desc'              => __( 'Enter number of days after which the log entry should be deleted from database.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'    => esc_html__( 'Wipe data on disconnect', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-wipe-data',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-wipe-data', 'no' ),
			'desc_tip' => __( 'Enable to delete the sync history on disconnecting the account.', 'integration-with-salesforce' ),
		);

		if ( $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
			$settings[] = array(
				'title'    => esc_html__( 'Synchronize Renewal Order to Salesforce', 'integration-with-salesforce' ),
				'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-sync-renewal',
				'type'     => 'checkbox',
				'class'    => 'mwb-crm-feed-switch',
				'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-sync-renewal', 'no' ),
				'desc_tip' => __( 'Enable to sync the renewal order.', 'integration-with-salesforce' ),
			);
		}

		$settings[] = array(
			'title'    => esc_html__( 'Synchronize Product Stock in WooCommerce', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-update-product-stock',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-update-product-stock', 'no' ),
			'desc_tip' => __( 'Enable to update product stock quantity in WooCommerce when updated in Salesforce.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'    => esc_html__( 'Product Stock Fields Name in WooCommerce', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-update-product-stock-api-name',
			'type'     => 'text',
			'class'    => 'mwb-custom-app-row',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-update-product-stock-api-name', '' ),
			'desc' => __( 'Enter the api name of the field you want to use to update product stock quantity in WooCommerce when updated in Salesforce.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'    => esc_html__( 'Product deletion from WooCommerce to Salesforce', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-product-deletion-woo-to-salesforce',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-product-deletion-woo-to-salesforce', 'no' ),
			'desc_tip' => __( 'Enable this option to automatically delete products from Salesforce when deleted from WooCommerce and also product will update as inactive on Salesforce when product will go trash and will be update as active on Salesforce when restored in WooCommerce. <br><b>Note: </b>Please do map the <b>IsActive</b> field mapping firstly in product feed and then sync all the existing products and pricebooks via bulk data sync before enabling this setting.', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'title'    => esc_html__( 'Product deletion from Salesforce to WooCommerce', 'integration-with-salesforce' ),
			'id'       => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-product-deletion-salesforce-to-woo',
			'type'     => 'checkbox',
			'class'    => 'mwb-crm-feed-switch',
			'value'    => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enable-product-deletion-salesforce-to-woo', 'no' ),
			'desc_tip' => __( 'Enable this option to automatically delete products from WooCommerce when deleted from Salesforce and also product will go in trash on WooCommerce when product will updated as inactive in Salesforce and will restore on WooCommerce when updated as active in Salesforce', 'integration-with-salesforce' ),
		);

		$settings[] = array(
			'type' => 'sectionend',
		);
		return $settings;
	}

	/**
	 * May be check if user has multiple orders.
	 * Map exsiting order account id with current order.
	 *
	 * @param  int $order_id   Current order id.
	 * @param  int $meta_key   Meta key to check.
	 * @return int $meta_value Meta value if found.
	 */
	public function may_be_get_field_value_from_existing_orders( $order_id, $meta_key = '' ) {

		$meta_value = '';

		$order = wc_get_order( $order_id );
		if ( ! $order ) {
			return $meta_value;
		}
		// Get billing email from order id.
		$billing_email = $order->get_billing_email();

		// Check if billing email exists in another order.

		$args = array(
			'billing_email' => $billing_email,
			'return'        => 'ids',
			'exclude'       => array( $order_id ),
		);

		$other_orders = wc_get_orders( $args );

		if ( '' === $meta_key ) {
			return $other_orders;
		}

		// Check if that order has meta key or not.
		if ( is_array( $other_orders ) && count( $other_orders ) ) {
			foreach ( $other_orders as $key => $other_order_id ) {
				if (Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
					$order = wc_get_order($other_order_id);
					
					if ( $order ) {

						$meta_value = $order->get_meta($meta_key);
					} else {
						$meta_value = get_post_meta( $other_order_id, $meta_key, true );
					}
				} else {
					$meta_value = get_post_meta( $other_order_id, $meta_key, true );
				}
				if ( ! empty( $meta_value ) ) {
					break;
				}
			}
		}
		// Return meta value.

		return $meta_value;
	}

	/**
	 * Returns the hooks/callbacks/priority for each active feed.
	 *
	 * @since  1.0.0
	 * @return string The current object event Hook.
	 */
	public function get_hook_requests_for_feed() {
		$feed_ids = $this->get_all_feed();

		if ( empty( $feed_ids ) ) {
			return;
		}

		$requested_hooks = array();

		foreach ( $feed_ids as $key => $feed_id ) {
			if ( ! empty( $feed_id ) ) {
				$event                       = $this->get_feed( $feed_id, 'feed_event' );
				$record_type                 = $this->get_feed(
					$feed_id,
					'crm_object'
				);
				$requested_hooks[ $feed_id ] = array(
					'hook'     => $this->get_hook( $event, $record_type ),
					'callback' => $this->get_callback( $event, $record_type ),
				);
			}
		}

		return $requested_hooks;
	}

	/**
	 * Returns the mapping step we require.
	 *
	 * @since 1.0.0
	 *
	 * @return bool|array single or multiple feed ids.
	 */
	public function get_all_feed() {
		$args = array(
			'post_type'      => 'mwb_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_feed',
			'post_status'    => 'publish',
			'fields'         => 'ids',
			'posts_per_page' => -1,
		);

		$feeds = get_posts( $args );

		if ( ! empty( $feeds ) ) {
			return $feeds;
		}
		return false;
	}

	/**
	 * Returns all the feeds.
	 *
	 * @since 1.0.0
	 *
	 * @return bool|array single or multiple feed ids.
	 */
	public function get_all_feeds() {
		$args = array(
			'post_type'      => 'mwb_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_feed',
			'post_status'    => array( 'publish', 'draft' ),
			'fields'         => 'ids',
			'posts_per_page' => -1,
		);

		$feeds = get_posts( $args );

		if ( ! empty( $feeds ) ) {
			return $feeds;
		}
		return false;
	}

	/**
	 * Returns the feed data we require.
	 *
	 * @param int|bool $feed_id  The object id for post type feed.
	 * @param string   $meta_key The object meta key for post type feed.
	 *
	 * @return array|bool The current data for required object.
	 *
	 * @since 1.0.0
	 */
	public function get_feed( $feed_id = false, $meta_key = 'mapping_data' ) {
		if ( false === $feed_id ) {
			return;
		}

		$mapping = get_post_meta( $feed_id, $meta_key, true );

		if ( empty( $mapping ) ) {
			$mapping = false;
		}

		return $mapping;
	}

	/**
	 * Returns the hooks for each active feed.
	 *
	 * @param string      $event  The event for feed.
	 * @param string|bool $object The CRM object for feed.
	 *
	 * @since  1.0.0
	 * @return string The current object event Hook.
	 */
	public function get_hook( $event = '', $object = false ) {
		$hook = '';
		switch ( $event ) {

			case 'new-order':
				$hook = 'woocommerce_new_order';
				break;

			case 'wc-pending':
				$hook = 'woocommerce_order_status_pending';
				break;

			case 'wc-processing':
				$hook = 'woocommerce_order_status_processing';
				break;

			case 'wc-on-hold':
				$hook = 'woocommerce_order_status_on-hold';
				break;

			case 'wc-completed':
				$hook = 'woocommerce_order_status_completed';
				break;

			case 'wc-cancelled':
				$hook = 'woocommerce_order_status_cancelled';
				break;

			case 'wc-refunded':
				$hook = array(
					'woocommerce_order_fully_refunded',
					'woocommerce_order_partially_refunded',
				);
				break;

			case 'wc-failed':
				$hook = 'woocommerce_order_status_failed';
				break;

			case 'status_change':
				$hook = array(
					'woocommerce_order_status_pending',
					'woocommerce_order_status_failed',
					'woocommerce_order_fully_refunded',
					'woocommerce_order_partially_refunded',
					'woocommerce_order_status_cancelled',
					'woocommerce_order_status_completed',
					'woocommerce_order_status_on-hold',
					'woocommerce_order_status_processing',
				);
				break;

			case 'product_update_create':
				// $hook = 'save_post_product';
				$hook = array(
					'woocommerce_new_product',
					'woocommerce_update_product',
				);
				break;

			case 'user_update_create':
				// $hook = 'woocommerce_new_order';
				$hook = array(
					'user_register',
					'wps_sf_edit_user_profile_update',
					'woocommerce_checkout_update_user_meta',
				);
				break;
			case 'send_manually':
				switch ( $object ) {

					case 'Order':
					case 'Contact':
					case 'Opportunity':
					case 'Account':
					case 'Case':
					case 'Quote':
						$hook = 'save_post_shop_order';
						break;

					case 'Product2':
					case 'PricebookEntry':
						$hook = array(
							'woocommerce_new_product',
							'woocommerce_update_product',
						);
						// $hook = 'save_post_product';
						break;
				}
				break;

			case 'create_subscription':
				$hook = 'woocommerce_subscription_payment_complete';
				break;

			case 'subscription_status_changed':
				$hook = 'woocommerce_subscription_status_changed';
				break;

			case 'subscription_updated':
				$hook = 'woocommerce_process_shop_subscription_meta';
				break;

			case 'membership_saved':
				$hook = 'wc_memberships_user_membership_saved';
				break;

			case 'membership_created_when_adding_new_user_as_member':
				$hook = 'wc_memberships_grant_free_membership_access_from_sign_up';
				break;
	
			default:
				break;
		}

		return $hook;
	}

	/**
	 * Returns the callbacks for each active feed.
	 *
	 * @param string      $event  The event for feed.
	 * @param string|bool $object The CRM object for feed.
	 *
	 * @since  1.0.0
	 * @return string The current object event Hook.
	 */
	public function get_callback( $event = '', $object = false ) {
		if ( empty( $event ) || empty( $object ) ) {
			return;
		}
		$callback = '';
		switch ( $event ) {
			case 'new-order':
				$callback = 'shop_order_updated';
				break;

			case 'wc-pending':
				$callback = 'shop_order_updated';
				break;

			case 'wc-processing':
				$callback = 'shop_order_updated';
				break;

			case 'wc-on-hold':
				$callback = 'shop_order_updated';
				break;

			case 'wc-completed':
				$callback = 'shop_order_updated';
				break;

			case 'wc-cancelled':
				$callback = 'shop_order_updated';
				break;

			case 'wc-refunded':
				$callback = 'shop_order_refunded';
				break;

			case 'wc-failed':
				$callback = 'shop_order_updated';
				break;

			case 'status_change':
				$callback = 'shop_order_status_changed';
				break;

			case 'product_update_create':
				$callback = 'create_and_update_product';
				break;

			case 'user_update_create':
				$callback = 'create_and_update_contact';
				break;

			case 'send_manually':
				switch ( $object ) {

					case 'Order':
					case 'Opportunity':
					case 'Contact':
					case 'Account':
					case 'Case':
					case 'Quote':
						$callback = 'shop_order_updated';
						break;

					case 'Product2':
					case 'PricebookEntry':
						$callback = 'create_and_update_product';
						break;

				}
				break;

			case 'create_subscription':
				$callback = 'subscription_created';
				break;

			case 'subscription_status_changed':
				$callback = 'subscription_status_change';
				break;

			case 'subscription_updated':
				$callback = 'subscription_update';
				break;

			case 'membership_saved':
				$callback = 'sync_membership_data';
				break;

			case 'membership_created_when_adding_new_user_as_member':
				$callback = 'sync_manually_created_membership_data';
				break;
				
			default:
				break;
		}
		return $callback;
	}


	/**
	 * Returns the hooks for each active feed.
	 *
	 * @param string $hook The hooks for feed.
	 *
	 * @since  1.0.0
	 * @return string The current hooks param count.
	 */
	public function get_hook_param_count( $hook = '' ) {
		if ( is_array( $hook ) ) {
			if ( in_array( 'woocommerce_order_fully_refunded', $hook ) ) { //phpcs:ignore
				$hook = 'woocommerce_order_fully_refunded';
			}
		}
		$count = '';
		switch ( $hook ) {

			case 'woocommerce_new_order':
				$count = '2';
				break;

			case 'woocommerce_order_status_pending':
				$count = '1';
				break;

			case 'woocommerce_order_status_processing':
				$count = '1';
				break;

			case 'woocommerce_order_status_on-hold':
				$count = '1';
				break;

			case 'woocommerce_order_status_completed':
				$count = '1';
				break;

			case 'woocommerce_order_status_cancelled':
				$count = '1';
				break;

			case 'woocommerce_order_status_failed':
				$count = '1';
				break;

			case 'woocommerce_order_fully_refunded':
			case 'woocommerce_order_partially_refunded':
				$count = '2';
				break;

			case 'save_post_shop_order':
				$count = '3';
				break;

			case 'save_post_product':
				$count = '3';
				break;

			case 'woocommerce_new_product':
			case 'woocommerce_update_product':
				$count = '2';
				break;
			case 'woocommerce_subscription_payment_complete':
				$count = '1';
				break;
			case 'user_register':
			case 'wps_sf_edit_user_profile_update':
			case 'woocommerce_checkout_update_user_meta':
				$count = '2';
				break;

			case 'woocommerce_subscription_status_changed':
				$count = '3';
				break;

			case 'woocommerce_process_shop_subscription_meta':
				$count = '2';
				break;

			case 'wc_memberships_user_membership_saved':
				$count = '2';
				break;

			case 'wc_memberships_grant_free_membership_access_from_sign_up':
				$count = '2';
				break;
		
			default:
				break;
		}

		return $count;
	}

	/**
	 * Returns the default status of a particular object.
	 *
	 * @param string $crm_object CRM object.
	 *
	 * @since  1.0.3
	 * @return string Default status of object record.
	 */
	public function get_default_status( $crm_object ) {

		$status = '';
		switch ( $crm_object ) {
			case 'Opportunity':
				$status = 'Prospecting';
				break;
			case 'Order':
			case 'Quote':
				$status = 'Draft';
				break;
		}
		/**
		 * Filters the value of Salesforce Default Status.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed  $status  Woo order status.
		 * @param mixed $crm_object Crm objects.
		 */
		return apply_filters( 'mwb_woo_salesforce_default_status', $status, $crm_object );
	}

	// End of class.
}
