<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link        https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
use Automattic\WooCommerce\Utilities\OrderUtil;
if ( ! class_exists( 'Integration_With_Salesforce_Framework_Base' ) ) {
	wp_die( 'Integration_With_Salesforce_Framework_Base does not exists.' );
}

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Framework extends Integration_With_Salesforce_Framework_Base {

	/**
	 *  The instance of this class.
	 *
	 * @since    1.0.0
	 * @var      string    $instance    The instance of this class.
	 */
	private static $instance;

	/**
	 * Main Integration_With_Salesforce_Framework Instance.
	 *
	 * Ensures only one instance of Integration_With_Salesforce_Framework is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @return Integration_With_Salesforce_Framework - Main instance.
	 */
	public static function get_instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Get current mapping scenerio for current CRM connection.
	 *
	 * @since 1.0.0
	 *
	 * @return array - Current CRM to Woo mapping.
	 */
	public function getMappingDataset() {

		$admin_instance = new Integration_With_Salesforce_Admin( 'Salesforce Integration for WooCommerce', '1.2.1' );
		$obj_type = array(
			'shop_order',
			'product',
			'users',
			( $admin_instance->wps_is_woocommerce_membership_plugin_active() ) ? 'wc_user_membership' : '',
		);

		$formatted_dataset = array();
		foreach ( $obj_type as $key => $obj ) {
			$formatted_dataset[ $obj ] = $this->getMappingOptions( $obj );
		}

		$formatted_dataset = $this->add_order_item_dataset( $formatted_dataset );
		$formatted_dataset = $this->parse_labels( $formatted_dataset );

		// No need for parse labels for feed dataset.
		$formatted_dataset = $this->add_available_feeds_dataset( $formatted_dataset );

		return $formatted_dataset;
	}

	/**
	 * Add available feeds dataset in current mapping dataset.
	 *
	 * @since 1.0.0
	 *
	 * @param array $formatted_dataset Array of mapping field options.
	 *
	 * @return array - Current CRM to Woo mapping.
	 */
	public function add_available_feeds_dataset( $formatted_dataset ) {
		$feed_module     = Integration_With_Salesforce_Feed_Module::get_instance();
		$available_feeds = $feed_module->get_available_feeds_for_mapping();
		if ( count( $available_feeds ) ) {
			$formatted_dataset['feeds'] = $available_feeds;
		}
		return $formatted_dataset;
	}

	/**
	 * Add order item dataset in current mapping dataset.
	 *
	 * @since 1.0.0
	 *
	 * @param  array $formatted_dataset Current dataset.
	 *
	 * @return array Array of requried data.
	 */
	public function add_order_item_dataset( $formatted_dataset ) {
		$format_dataset = array(
			'item_id',
			'item_price',
			'item_quantity',
		);
		global $wpdb;
		$post_type = 'line_item';
		$order_item_table_name = $wpdb->prefix . 'woocommerce_order_items';
		$order_itemmeta_table_name = $wpdb->prefix . 'woocommerce_order_itemmeta' ;
		$result = $wpdb->get_col(
			$wpdb->prepare(
			"SELECT DISTINCT(om.meta_key)
			FROM {$wpdb->prefix}woocommerce_order_items AS oi 
			JOIN {$wpdb->prefix}woocommerce_order_itemmeta AS om ON oi.order_item_id = om.order_item_id
			WHERE oi.order_item_type = %s
			ORDER BY oi.order_item_id;", $post_type
			)
		);
		if ( in_array( '_line_tax_data', $result ) ) {
			unset($result[array_search('_line_tax_data', $result)]);
		}
		 $formatted_dataset['order_item']= array_merge($format_dataset, array_keys( $this->order_item_extras()), $result);
		return $formatted_dataset;
	}

	/**
	 * Get current mapping scenerio for current CRM connection.
	 *
	 * @param string $obj_type - Woo Object type to get mapping for.
	 *
	 * @since 1.0.0
	 *
	 * @return array - Current CRM to Woo mapping.
	 */
	public function getMappingOptions( $obj_type = false ) {
		return $this->get_wp_meta( $obj_type );
	}

	/**
	 * Returns the mapped objects counts.
	 *
	 * @param string|boolean $obj_type The Object post type.
	 *
	 * @since 1.0.0
	 *
	 * @return bool|array single or multiple feed ids.
	 */
	public function getSyncedObjectsCount( $obj_type = false ) {
		if ( empty( $obj_type ) ) {
			return;
		}

		$feed_id     = $this->get_object_feed( $obj_type );
		$feed_query  = '';
		$feed_module = Integration_With_Salesforce_Feed_Module::get_instance();

		if ( ! empty( $feed_id ) ) {

			global $wpdb;
			
			if ( OrderUtil::custom_orders_table_usage_is_enabled() ) { 
				if ( 'Product2' == $obj_type ) {
					$post_meta = $wpdb->prefix . 'postmeta';
				} else {
					$post_meta = $wpdb->prefix . 'wc_orders_meta';
				}
			} else {
				$post_meta = $wpdb->prefix . 'postmeta';
			}
			

			$sql = "SELECT COUNT(DISTINCT `meta_value`) as count FROM {$post_meta} WHERE ";

			if ( is_array( $feed_id ) ) {

				foreach ( $feed_id as $key => $feed ) {

					$crm_object = $feed_module->get_feed_data( $feed, 'crm_object', '' );

					if ( empty( $feed_query ) ) {
						$feed_query = "`meta_key` =  'mwb_" . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_' . $crm_object . "_id'";
					} else {
						$feed_query .= " OR `meta_key` =  'mwb_" . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_' . $crm_object . "_id'";
					}
				}
			} else {
				$crm_object = $feed_module->get_feed_data( $feed_id, 'crm_object', '' );

				$feed_query = "`meta_key` =  'mwb_" . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_' . $crm_object . "_id'";
			}

			$sql .= $feed_query;
			if ( ! empty( $sql ) ) {
				$result = mwb_woo_salesforce_get_query_results( $sql );
			}
		}

		$count = ! empty( $result[0]['count'] ) ? $result[0]['count'] : 0;
		echo esc_html( $count );
		return esc_html( $count );
	}

	/**
	 * Get available Salesforce objects
	 */
	public static function get_available_crm_objects() {
		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$crm_objects    = $crm_api_module->get_crm_objects();
		return $crm_objects;
	}

	/**
	 * Get authorization settings.
	 *
	 * @return array $settings Settings array.
	 */
	public static function get_authorization_settings() {

		$settings[] = array(
			'title' => esc_html__( 'Connect your Salesforce account', 'integration-with-salesforce' ),
			'class' => 'mwb-wfw-sub-heading',
			'type'  => 'title',
		);
		$settings[] = array(
			'title'             => esc_html__( 'Use Custom APP', 'integration-with-salesforce' ),
			'id'                => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-use-custom-app',
			'type'              => 'checkbox',
			'class'             => 'mwb-crm-auth-toggle-switch',
			'value'             => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-use-custom-app', 'no' ),
			'custom_attributes' => array( 'required' => 'required' ),
			'desc_tip'          => esc_html__( 'Turn on this option to use your custom salesforce application for authentication.', 'integration-with-salesforce' ),
		);
		$settings[] = array(
			'title'             => esc_html__( 'Enviornment', 'integration-with-salesforce' ),
			'id'                => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enviornment',
			'type'              => 'select',
			'options'           => array(
				'production' => __( 'Production', 'integration-with-salesforce' ),
				'sandbox'    => __( 'Sandbox', 'integration-with-salesforce' ),
			),
			'value'             => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enviornment', 'production' ),
			'custom_attributes' => array(
				'required' => 'required',
			),
			'desc'              => esc_html__( 'Salesforce environment', 'integration-with-salesforce' ),

		);
		$settings[] = array(
			'title'             => esc_html__( 'Consumer key', 'integration-with-salesforce' ),
			'id'                => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-consumer-key',
			'class'             => 'mwb-custom-app-row',
			'type'              => 'text',
			'placeholder'       => 'Enter your Consumer key here',
			'value'             => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-consumer-key', '' ),
			'custom_attributes' => array( 'required' => 'required' ),
			'desc'              => esc_html__( 'Salesforce api consumer key', 'integration-with-salesforce' ),
		);
		$settings[] = array(
			'title'             => esc_html__( 'Consumer secret', 'integration-with-salesforce' ),
			'id'                => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-consumer-secret',
			'class'             => 'mwb-custom-app-row',
			'type'              => 'text',
			'placeholder'       => 'Enter your Consumer secret here',
			'value'             => get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-consumer-secret', '' ),
			'custom_attributes' => array( 'required' => 'required' ),
			'desc'              => esc_html__( 'Salesforce api consumer secret', 'integration-with-salesforce' ),
		);
		$settings[] = array(
			'title'             => esc_html__( 'Callback URL', 'integration-with-salesforce' ),
			'id'                => 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-redirect-uri',
			'class'             => 'mwb-custom-app-row',
			'type'              => 'text',
			'value'             => admin_url(),
			'custom_attributes' => array(
				'required' => 'required',
				'readonly' => true,
			),
			'desc'              => esc_html__( 'Redirect url or your salesforce app. Redirect url must have https://', 'integration-with-salesforce' ),
		);
		$settings[] = array(
			'type' => 'sectionend',
		);
		return $settings;
	}

	/**
	 * Get individual field mappin section.
	 *
	 * @param array  $field_options Field options.
	 * @param array  $fields_data   Field data.
	 * @param array  $default_data  Default data.
	 * @param bool   $require_status_mapping if field requires status mapping.
	 * @param bool   $require_pricebook_mapping if field requires pricebook mapping.
	 * @param string $object        selected object.
	 * @return void
	 */
	public static function get_field_section_html( $field_options, $fields_data, $default_data = array(), $require_status_mapping = false, $require_pricebook_mapping = false, $object = '' ) {
		if ( empty( $default_data ) ) {
			$default_data = array(
				'field_type'  => 'standard_field',
				'field_value' => '',
			);
		}

		$row_stndrd   = ( 'standard_field' === $default_data['field_type'] ) ? '' : 'row-hide';
		$row_custom   = ( 'custom_value' === $default_data['field_type'] ) ? '' : 'row-hide';
		$custom_value = ! empty( $default_data['custom_value'] ) ? $default_data['custom_value'] : '';
		$field_value  = ! empty( $default_data['field_value'] ) ? $default_data['field_value'] : '';

		$selected_pricebook = get_option( 'mwb_salesforce_selected_pricebook', '' );

		$pricebook_row = ( $require_pricebook_mapping && 'Quote' !== $object ) ? 'row-hide' : '';

		if ( $require_status_mapping && ( ! isset( $default_data['use_status_mapping'] ) ) ) {
			$default_data = array_merge(
				$default_data,
				array(
					'use_status_mapping' => '',
				)
			);
		}

		?>
		<div class="mwb-feeds__form-wrap mwb-fields-form-row <?php echo esc_attr( $pricebook_row ); ?>">
				<div class="mwb-form-wrapper">
					<div class="mwb-fields-form-section-head">
						<span class="field-label-txt"><?php echo esc_html( $fields_data['label'] ); ?></span>
						<input type="hidden" class="crm-field-name" name="crm_field[]" value="<?php echo esc_html( $fields_data['name'] ); ?>">
						<?php if ( $fields_data['nillable'] || $fields_data['defaultedOnCreate'] ) : ?>
						<span class="field-delete dashicons">
							<img src="<?php echo esc_url( INTEGRATION_WITH_SALESFORCE_URL . 'assets/src/images/trash.svg' ); ?>">
						</span>
						<?php endif; ?>
					</div>
					<div class="mwb-fields-form-section-meta">
						<span>
							<?php esc_html_e( 'API Name : ', 'integration-with-salesforce' ); ?><?php echo esc_html( $fields_data['name'] ); ?>
						</span>
						<span>
							<?php esc_html_e( 'Type : ', 'integration-with-salesforce' ); ?><?php echo esc_html( $fields_data['type'] ); ?> 
						</span>
						<span>
							<?php esc_html_e( 'Length : ', 'integration-with-salesforce' ); ?><?php echo esc_html( $fields_data['length'] ); ?> 
						</span>
						<?php if ( ! empty( $fields_data['picklistValues'] ) ) : ?>
						<span>
							<?php esc_html_e( 'Picklist Values: ', 'integration-with-salesforce' ); ?><?php echo esc_html( implode( ', ', array_column( $fields_data['picklistValues'], 'label' ) ) ); ?> 
						</span>
						<?php endif; ?>	
					</div>
					<div class="mwb-fields-form-section-form">
						<div class="form-field-row row-field-type">
							<label><?php esc_html_e( 'Field Type', 'integration-with-salesforce' ); ?></label>
							<select class="field-type-select" name="field_type[]">
								<option value=""><?php esc_html_e( 'Select an Option', 'integration-with-salesforce' ); ?></option>
								<option value="standard_field" <?php echo esc_attr( selected( 'standard_field', $default_data['field_type'] ) ); ?> >
								<?php esc_html_e( 'Standard Value', 'integration-with-salesforce' ); ?>
								</option>
								<option value="custom_value" <?php echo esc_attr( selected( 'custom_value', $default_data['field_type'] ) ); ?>>
									<?php esc_html_e( 'Custom Value', 'integration-with-salesforce' ); ?>		
								</option>
							</select>
						</div>
						<div class="form-field-row row-field-value row-standard_field <?php echo esc_attr( $row_stndrd ); ?>">
							<label><?php esc_html_e( 'Field Value', 'integration-with-salesforce' ); ?></label>
							<select class="field-value-select" name="field_value[]">
								<option value=""><?php esc_html_e( 'Select an Option', 'integration-with-salesforce' ); ?></option>
								<?php if ( ! $require_pricebook_mapping ) : ?>
									<?php foreach ( $field_options as $k1 => $options ) : ?>
										<optgroup label="<?php echo esc_attr( ucfirst( str_replace( '_', ' ', $k1 ) ) ); ?>">
										<?php foreach ( $options as $k2 => $name ) : ?>
											<option value="<?php echo esc_attr( $k1 . '_' . $k2 ); ?>" <?php echo esc_attr( selected( $k1 . '_' . $k2, $field_value ) ); ?>>
												<?php echo esc_html( $name ); ?>
											</option>
										<?php endforeach; ?>
										</optgroup>
									<?php endforeach; ?>
								<?php else : ?>
									<?php foreach ( $field_options as $k2 => $name ) : ?>
										<option value="<?php echo esc_attr( $name['Id'] ); ?>" <?php echo esc_attr( selected( $selected_pricebook, $name['Id'] ) ); ?>>
											<?php echo esc_html( $name['Name'] ); ?>
										</option>
									<?php endforeach; ?>
								<?php endif; ?>
							</select>
						</div>

						<div class="form-field-row row-custom_value row-field-value <?php echo esc_attr( $row_custom ); ?>">
							<label>
							<?php
							esc_html_e( 'Custom Value', 'integration-with-salesforce' );
							?>
							</label>
							<input type="text" class="custom-value-input" name="custom_value[]" value="<?php echo esc_attr( $custom_value ); ?>">
							<select class="custom-value-select" name="custom_field[]">
								<option value=""><?php esc_html_e( 'Select an Option', 'integration-with-salesforce' ); ?></option>
								<?php foreach ( $field_options as $k1 => $options ) : ?>
									<optgroup label="<?php echo esc_attr( ucfirst( str_replace( '_', ' ', $k1 ) ) ); ?>">
									<?php foreach ( $options as $k2 => $name ) : ?>
										<option value="<?php echo esc_attr( $k1 . '_' . $k2 ); ?>">
											<?php echo esc_html( $name ); ?>
										</option>
									<?php endforeach; ?>
									</optgroup>
								<?php endforeach; ?>
							</select>
						</div>

						<?php if ( isset( $default_data['use_status_mapping'] ) && $require_status_mapping ) : ?>
							<div class="form-field-row">
								<label><?php esc_html_e( 'Map with woo order statuses', 'integration-with-salesforce' ); ?></label>
								<input <?php echo esc_attr( checked( 'yes', $default_data['use_status_mapping'] ) ); ?> type="checkbox" class="use-status-mapping-cb" value="yes" name="use_status_mapping[<?php echo esc_attr( $fields_data['name'] ); ?>]">
							</div>	
						<?php endif; ?>
						<div class="form-field-description">
							<?php
							$description = '';
							if ( 'reference' === $fields_data['type'] ) {
								$description = __( 'Please map this field with an existing feed and make sure to trigger that feed before this feed', 'integration-with-salesforce' );
								if ( 'Pricebook2Id' === $fields_data['name'] ) {
									$description = __( 'Please select the same Pricebook as selected in product feed', 'integration-with-salesforce' );
								}
							}
							?>
							<p class="mwb-desc">
								<?php echo esc_html( $description ); ?>
							</p>
						</div>
					</div>
				</div>
			</div>
		<?php
	}

	/**
	 * Feeds conditional filter options.
	 *
	 * @since     1.0.0
	 * @return    array
	 */
	public function get_available_feed_filters() {

		$filter = array(
			'exact_match'       => esc_html__( 'Matches exactly', 'integration-with-salesforce' ),
			'no_exact_match'    => esc_html__( 'Does not match exactly', 'integration-with-salesforce' ),
			'contains'          => esc_html__( 'Contains (Text)', 'integration-with-salesforce' ),
			'not_contains'      => esc_html__( 'Does not contain (Text)', 'integration-with-salesforce' ),
			'exist'             => esc_html__( 'Exist in (Text)', 'integration-with-salesforce' ),
			'not_exist'         => esc_html__( 'Does not Exists in (Text)', 'integration-with-salesforce' ),
			'starts'            => esc_html__( 'Starts with (Text)', 'integration-with-salesforce' ),
			'not_starts'        => esc_html__( 'Does not start with (Text)', 'integration-with-salesforce' ),
			'ends'              => esc_html__( 'Ends with (Text)', 'integration-with-salesforce' ),
			'not_ends'          => esc_html__( 'Does not end with (Text)', 'integration-with-salesforce' ),
			'less_than'         => esc_html__( 'Less than (Text)', 'integration-with-salesforce' ),
			'greater_than'      => esc_html__( 'Greater than (Text)', 'integration-with-salesforce' ),
			'less_than_date'    => esc_html__( 'Less than (Date/Time)', 'integration-with-salesforce' ),
			'greater_than_date' => esc_html__( 'Greater than (Date/Time)', 'integration-with-salesforce' ),
			'equal_date'        => esc_html__( 'Equals (Date/Time)', 'integration-with-salesforce' ),
			'empty'             => esc_html__( 'Is empty', 'integration-with-salesforce' ),
			'not_empty'         => esc_html__( 'Is not empty', 'integration-with-salesforce' ),
		);

		return $filter;
	}

	/**
	 * Feeds Conditional html.
	 *
	 * @param     string $and_condition  The and condition of current html.
	 * @param     string $and_index      The and offset of current html.
	 * @param     string $or_index       The or offset of current html.
	 * @since     1.0.0
	 * @return    mixed
	 */
	public function render_and_condition( $and_condition = array(), $and_index = '1', $or_index = '' ) {

		if ( empty( $and_index ) || empty( $and_condition ) || empty( $or_index ) ) {
			return;
		}

		?>
		<div class="and-condition-filter" data-and-index=<?php echo esc_attr( $and_index ); ?> >
			<select name="condition[<?php echo esc_html( $or_index ); ?>][<?php echo esc_html( $and_index ); ?>][field]"  class="condition-form-field">
				<option value="-1" ><?php esc_html_e( 'Select Field', 'integration-with-salesforce' ); ?></option>
				<?php foreach ( $and_condition['object_fields'] as $key => $value ) : ?>
					<optgroup label="<?php echo esc_html( $key ); ?>" >
						<?php foreach ( $value as $index => $field ) : ?>
							<option value="<?php echo esc_html( $index ); ?>" <?php selected( $and_condition['field'], $index ); ?> ><?php echo esc_html( $field ); ?></option>
						<?php endforeach; ?>
					</optgroup>
				<?php endforeach; ?>
			</select>
			<select name="condition[<?php echo esc_html( $or_index ); ?>][<?php echo esc_html( $and_index ); ?>][option]" class="condition-option-field">
				<option value="-1"><?php esc_html_e( 'Select Condition', 'integration-with-salesforce' ); ?></option>
				<?php foreach ( $this->get_available_feed_filters() as $key => $value ) : ?>
					<option value="<?php echo esc_html( $key ); ?>" <?php selected( $and_condition['option'], $key ); ?>><?php echo esc_html( $value ); ?></option>
				<?php endforeach; ?>
			</select>
			<input type="text" name="condition[<?php echo esc_html( $or_index ); ?>][<?php echo esc_html( $and_index ); ?>][value]" class="condition-value-field" value="<?php echo esc_html( ! empty( $and_condition['value'] ) ? $and_condition['value'] : '' ); ?>" <?php ( ( 'empty' == $and_condition['option'] ) || ( 'not_empty' == $and_condition['option'] ) ) ? esc_html_e( 'disabled="disabled"' ) : ''; ?> placeholder="<?php esc_html_e( 'Enter value', 'integration-with-salesforce' ); ?>" >
			<?php if ( 1 != $and_index ) : // @codingStandardsIgnoreLine ?>
				<span class="dashicons dashicons-no"></span>
			<?php endif; ?>
		</div>
		<?php
	}

	/**
	 * Get available filter options.
	 *
	 * @since    1.0.0
	 * @return   array
	 */
	public function get_filter_mapping_dataset() {
		return $this->get_available_feed_filters();
	}

	// End of class.
}
