<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link        https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */

use Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController;
use Automattic\WooCommerce\Utilities\OrderUtil;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Request_Module {

	/**
	 *  The instance of this class.
	 *
	 * @since  1.0.0
	 * @var    string    $_instance    The instance of this class.
	 */
	private static $_instance;

	/**
	 *  The instance of framework class.
	 *
	 * @since  1.0.0
	 * @var    Integration_With_Salesforce_Framework    $framework_instance Instance of framework class.
	 */
	private $framework_instance;

	/**
	 *  The instance of framework class.
	 *
	 * @since  1.0.0
	 * @var    Integration_With_Salesforce_Feed_Module    $framework_instance Instance of framework class.
	 */
	private $feed_module;

	/**
	 *  The instance of framework class.
	 *
	 * @since  1.0.0
	 * @var    Integration_With_Salesforce_Api    $framework_instance Instance of framework class.
	 */
	private $api_module;

	/**
	 * Main Integration_With_Salesforce_Request_Module Instance.
	 *
	 * Ensures only one instance of Integration_With_Salesforce_Request_Module is loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @return Integration_With_Salesforce_Request_Module - Main instance.
	 */
	public static function get_instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Construct function.
	 *
	 * @since  1.0.0
	 */
	public function __construct() {
		$this->framework_instance = Integration_With_Salesforce::get_integration_module( 'framework' );
		$this->feed_module        = Integration_With_Salesforce::get_integration_module( 'feed' );
		$this->api_module         = Integration_With_Salesforce::get_integration_module( 'api' );
	}

	/**
	 * If instant sync is required.
	 *
	 * @since  1.0.0
	 * @return string|array  Feeds ids.
	 */
	public static function is_instant_sync_required() {
		// Get Salesforce Instance.
		$salesforce = Integration_With_Salesforce::get_integration_module( 'framework' );

		$current_function = function_exists( 'debug_backtrace' ) &&
		! empty( debug_backtrace()[1]['function'] ) ?
		debug_backtrace()[1]['function'] :
		'';

		if ( ! empty( $current_function ) ) {
			$args = array(
				'hook'     => current_action(),
				'callback' => $current_function,
			);

		} else {

			// Get from WP filter.
			global $wp_filter;
			if ( ! empty( ! empty( $wp_filter[ current_action() ]->callbacks ) ) ) {
				$callbacks = $wp_filter[ current_action() ]->callbacks;
				foreach ( $callbacks as $key => $_callback ) {
					$function = ! empty( $_callback ) ?
					$_callback :
					array();

					foreach ( $_callback  as $key => $function ) {
						if ( false !== strpos( wp_json_encode( $function ), 'crm_connect_manager' ) ) {
							foreach ( $function as $key => $func ) {
								$current_function = ! empty( $func[1] )
								? $func[1] :
								false;
								break;
							}
						}
					}
				}
			}

			$args = array(
				'hook'     => current_action(),
				'callback' => $current_function,
				'params'   => $salesforce->getHookParamCount( current_action() ),
			);
		}

		// Get all probable Feeds callbacks.
		$hookable_callbacks = $salesforce->get_hook_requests_for_feed();

		$feed_id = self::get_feed_id_by_request( $args );

		if ( ! empty( $feed_id ) && is_array( $feed_id ) ) {

			if ( 1 == count( $feed_id ) ) {

				$event = $salesforce->get_feed( $feed_id[0], 'feed_event' );
				// Check if sync is need to be instantly synced or not.
				if ( 'send_manually' == $event ) {
					if ( isset( $_POST['meta_box_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['meta_box_nonce'] ) ), 'meta_box_nonce' ) ) {
						$maybe_perform_manual_sync = is_admin() && ! empty( $_POST['mwb_manual_sync'] )
						&& true == $_POST['mwb_manual_sync'] ? true : false;
						if ( true == $maybe_perform_manual_sync ) {
							return $feed_id[0];
						} else {
							return false;
						}
					}
				} else {
					return $feed_id[0];
				}
			} else {

				foreach ( $feed_id as $key => $f_id ) {
					$event = $salesforce->get_feed( $f_id, 'feed_event' );
					// Check if sync is need to be instantly synced or not.
					if ( 'send_manually' == $event ) {
						if ( isset( $_POST['meta_box_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['meta_box_nonce'] ) ), 'meta_box_nonce' ) ) {
							$maybe_perform_manual_sync = is_admin() && ! empty( $_POST['mwb_manual_sync'] )
							&& true == $_POST['mwb_manual_sync'] ? true : false;
							if ( false == $maybe_perform_manual_sync ) {
								unset( $feed_id[ $key ] );
							}
						}
					}
				}
				return $feed_id;
			}
		} else {
			return false;
		}
	}

	/**
	 * If Feed id is not justified due to multiple hooks.
	 *
	 * @param array $args The args for hook and callback.
	 *
	 * @since  1.0.0
	 * @return bool  true|false.
	 */
	public static function get_feed_id_by_request( $args = array() ) {

		// Get Salesforce Instance.
		$salesforce         = Integration_With_Salesforce::get_integration_module( 'framework' );
		$hookable_callbacks = $salesforce->get_hook_requests_for_feed();

		if ( empty( $hookable_callbacks ) ) {
			return;
		}

		if ( empty( $args ) ) {
			$hooks = array(
				'woocommerce_order_status_pending',
				'woocommerce_order_status_failed',
				'woocommerce_order_fully_refunded',
				'woocommerce_order_partially_refunded',
				'woocommerce_order_status_cancelled',
				'woocommerce_order_status_completed',
				'woocommerce_order_status_on-hold',
				'woocommerce_order_status_processing',
			);

			$args = array(
				'hook'     => $hooks,
				'callback' => 'shop_order_status_changed',
			);
		}

		$feed_ids = array();
		foreach ( $hookable_callbacks as $feed_id => $feed_args ) {
			if ( $feed_args == $args ) {  //phpcs:ignore
				if ( ! empty( $feed_id ) ) {
					$feed_ids[] = $feed_id;
				}
			} elseif ( is_array( $feed_args['hook'] ) && in_array( $args['hook'], $feed_args['hook'] ) ) {
				$feed_ids[] = $feed_id;
			}
		}

		return $feed_ids;
	}

	/**
	 * Prepare/Fire a request whenever a product is created/updated.
	 *
	 * This hooks works only for admin side create/update.
	 *
	 * @param string $post_id Woo Object post ID.
	 * @param string $post    Woo Object post.
	 * @param string $feed_id Feed id.
	 *
	 * @since  1.0.0
	 * @return null
	 */
	public function create_and_update_product( $post_id, $post, $feed_id = '' ) {

		// Return if doing autosave.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		// Return if doing ajax :: Quick edits.
		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return;
		}

		if ( ! isset( $_POST['meta_box_nonce'] ) ) {
			return;
		}

		if ( ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['meta_box_nonce'] ) ), 'meta_box_nonce' ) ) {
			return;
		}

		// Return on post trash, quick-edit, new post.
		if ( empty( $_POST['action'] ) || 'editpost' != $_POST['action'] ) {
			return;
		}

		update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );

		if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
			return;
		}

		$cache_id = wp_cache_get( 'mwb_salesforce_product_synced' );

		if ( $cache_id == $post_id ) {
			return;
		}

		$feed_id = empty( $feed_id ) ? self::is_instant_sync_required() : false;
		if ( ! empty( $feed_id ) && is_numeric( $feed_id ) ) {

			$this->trigger_product_related_feed( $feed_id, $post_id, 'true' );
			$associated_feeds = $this->feed_module->get_associated_feeds( $feed_id );

			/**
			 * Trigger associated Feeds.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $post_id  Woo post id.
			 * @param mixed $associated_feeds associated feeds.
			 * @param mixed shop_order_updated callback.
			 */
			do_action(
				'mwb_salesforce_trigger_associated_feeds',
				$post_id,
				$associated_feeds,
				'create_and_update_product'
			);
		}
		wp_cache_set( 'mwb_salesforce_product_synced', $post_id );
	}

	
	/**
	 * Trigger feed related to product object.
	 *
	 * @param string $feed_id Feed id.
	 * @param string $post_id Woo Object post ID.
	 */
	public function trigger_product_related_feed( $feed_id, $post_id, $iActive ) {
		// All checks are done, so we can directly move to feed logic.

		$post = get_post( $post_id );

		if ( ! $post ) {
			return false;
		}

		$feed_title   = $this->framework_instance->get_feed_title( $feed_id );
		$filter_exist = $this->maybe_check_filter( $feed_id );

		// Get product.
		$product      = wc_get_product( $post_id );
		$bulk_request = array();
		$bulk_ids     = array();
		if ( 'variable' == $product->get_type() || 'variable-subscription' === $product->get_type() ) {
			$bulk_ids        = array();
			$feed_module     = Integration_With_Salesforce::get_integration_module( 'feed' );
			$product_feed_id = get_option( 'mwb_woo_salesforce_Product2_feed_id', '' );
			if ( ! empty( $product_feed_id ) ) {
				$is_default_feed = $feed_module->get_feed_data( $product_feed_id, 'default_feed', '' );
				if ( $is_default_feed ) {
					$sync_parent_product = $feed_module->get_feed_data( $product_feed_id, 'sync_parent_product', 'no' );
					if ( 'yes' === $sync_parent_product ) {
						
						$request    = $this->framework_instance->get_request(
							$post->post_type,
							$feed_id,
							$post_id
						);
						$bulk_ids[] = $post_id;
						/**
						 * Filters the value of Salesforce product request.
						 *
						 * @since 1.0.3
						 *
						 * @param mixed  $request  Woo product request.
						 * @param mixed $post_id product ids.
						 */
						$request = apply_filters(
							'woo_crm_salesforce_product_request',
							$request,
							$post_id
						);
						array_push( $bulk_request, $request );
					}
				}
			}

			$available_variations = $product->get_available_variations();
			if ( ! empty( $available_variations )
				&& is_array( $available_variations )
			) {

				foreach ( $available_variations as $key => $variation ) {
					
					$request = $this->framework_instance->get_request(
						$post->post_type,
						$feed_id,
						$variation['variation_id']
					);

					$bulk_ids[] = $variation['variation_id'];
					/**
					 * Filters the value of Salesforce product request.
					 *
					 * @since 1.0.3
					 *
					 * @param mixed  $request  Woo product request.
					 * @param mixed $post_id product ids.
					 */
					$request = apply_filters(
						'woo_crm_salesforce_product_request',
						$request,
						$variation['variation_id']
					);

					array_push( $bulk_request, $request );
				}
			}
		} elseif ( 'variation' == $product->get_type() || 'subscription_variation' === $product->get_type() ) {
			
			$request  = $this->framework_instance->get_request(
				'product',
				$feed_id,
				$post_id
			);
			$bulk_ids = array( $post_id );
			/**
			 * Filters the value of Salesforce product request.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $request  Woo product request.
			 * @param mixed $post_id product ids.
			 */
			$request = apply_filters(
				'woo_crm_salesforce_product_request',
				$request,
				$post_id
			);
		} else {
			
			$request  = $this->framework_instance->get_request(
				$post->post_type,
				$feed_id,
				$post_id
			);
			if ( isset( $request['IsActive'] ) && true == $request['IsActive'] ) {
				$request['IsActive'] = $iActive;
			}
			$bulk_ids = array( $post_id );
			/**
			 * Filters the value of Salesforce product request.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $request  Woo product request.
			 * @param mixed $post_id product ids.
			 */
			$request = apply_filters(
				'woo_crm_salesforce_product_request',
				$request,
				$post_id
			);
		}

		$request_arr = array();
		$request_arr = ! empty( $bulk_request ) ?
		$bulk_request :
		array( $request );

		$record_type = $this->framework_instance->get_feed(
			$feed_id,
			'crm_object'
		);

		$primary_field = $this->framework_instance->get_feed(
			$feed_id,
			'primary_field'
		);

		$dependencies = $this->feed_module->get_feed_dependencies( $record_type );
		$result_array = array();
		$request_data = array();
		$request_ids  = array();
		$result_data  = array();
		
		foreach ( $request_arr as $key => $request_item ) {
			$flag = true;
			// Check dependencies.
			if ( ! empty( $dependencies ) ) {
				foreach ( $dependencies as $dk => $dv ) {
					if ( empty( $request_item[ $dk ] ) ) {
						// Dependency not found add error in result array.
						$result_array[ $request_item['post_id'] ] = array(
							'id'      => '',
							'post_id' => $request_item['post_id'],
							'message' => 'Missing Dependency : ' . $dk,
						);

						$flag = false;
						break;
					}
				}
			}

			// Dependency check passed.
			if ( $flag ) {
				$request_ids[] = $request_item['post_id'];
				unset( $request_item['post_id'] );
				$request_data[] = $request_item;
			}
		}

		if ( ! empty( $request_data ) && is_array( $request_data ) ) {
			$result_data = $this->api_module->bulk_create_or_update_record( $record_type, $request_data, $primary_field, $feed_id );
			if ( ! empty( $filter_exist ) ) {
				$filtered_ids = array();
				if ( ! empty( $request_arr ) ) {
					foreach ( $request_arr as $key => $data ) {
						$filter_result = $this->mwb_salesforce_validate_filter( $filter_exist, $data );
						if ( true === $filter_result ) {
							array_push( $filtered_ids, $data['post_id'] );

						} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
							array_push( $filtered_ids, $data['post_id'] );
						}
					}
				}
				$request_ids = $filtered_ids;
			} else {
				$request_ids = $request_ids;
			}

			if ( ! empty( $result_data ) ) {
				if ( count( $request_ids ) == count( $result_data ) ) {
					$result_data = array_combine( $request_ids, $result_data );
				}
			}
			$result_array = array_replace( $result_array, $result_data );
		}

		if ( 'PricebookEntry' === $record_type ) {

			$standard_pricebook_error = false;
			foreach ( $result_array as $key => $line_result ) {
				if ( ! $line_result['success'] ) {
					if ( ! empty( $line_result['errors'] ) && ! empty( $line_result['errors'][0] ) ) {
						if ( 'STANDARD_PRICE_NOT_DEFINED' === $line_result['errors'][0]['statusCode'] ) {
							$standard_pricebook_error = true;
							break;
						}
					}
				}
			}

			if ( $standard_pricebook_error ) {
				$new_request_data = array();
				$result_data      = array();
				// $standard_pricebook_id = '01s5g00000D6GvqAAF'; // Save this while creating pricebook.
				$standard_pricebook_id = get_option( 'mwb_salesforce_standard_pricebook', '' );
				foreach ( $request_data as $key => $request_item ) {
					$request_item['Pricebook2Id'] = $standard_pricebook_id;
					array_push( $new_request_data, $request_item );
				}

				if ( ! empty( $new_request_data ) && is_array( $new_request_data ) ) {
					$new_result  = $this->api_module->bulk_create_or_update_record( $record_type, $new_request_data, $primary_field, $feed_id );
					$result_data = array();
					$result_data = $this->api_module->bulk_create_or_update_record( $record_type, $request_data, $primary_field, $feed_id );
					if ( ! empty( $filter_exist ) ) {
						$filtered_ids = array();
						if ( ! empty( $request_arr ) ) {
							foreach ( $request_arr as $key => $data ) {
								$filter_result = $this->mwb_salesforce_validate_filter( $filter_exist, $data );
								if ( true === $filter_result ) {
									array_push( $filtered_ids, $data['post_id'] );

								} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
									array_push( $filtered_ids, $data['post_id'] );
								}
							}
						}
						$request_ids = $filtered_ids;
					} else {
						$request_ids = $request_ids;
					}
					if ( ! empty( $result_data ) ) {
						if ( count( $request_ids ) == count( $result_data ) ) {
							$result_data = array_combine( $request_ids, $result_data );
						}
					}
				}
				$result_array = array_replace( $result_array, $result_data );
			}
		}
		
		foreach ( $result_array as $line_post_id => $line_response ) {
			$line_id = '';
			if ( ! empty( $line_response['id'] ) ) {
				$line_id = $line_response['id'];
				update_post_meta( $line_post_id, 'mwb_salesforce_' . $record_type . '_id', $line_id );
			}

			$line_request = array();
			$request_key  = array_search( $line_post_id, $bulk_ids );
			if ( false !== $request_key && isset( $request_arr[ $request_key ] ) ) {
				$line_request = $request_arr[ $request_key ];
			}

			$log_data = array(
				'woo_object'        => 'product',
				'woo_id'            => $line_post_id,
				'salesforce_object' => $record_type,
				'salesforce_id'     => $line_id,
				'event'             => $feed_title,
				'request'           => wp_json_encode( $line_request ),
				'response'          => wp_json_encode( $line_response ),
				'time'              => time(),
			);
			mwb_woo_salesforce_insert_log( $log_data );
		}
	}

	/**
	 * Trigger associated feed.
	 *
	 * @param string $post_id            Woo Object post ID.
	 * @param array  $associated_feeds   Array of associated Feeds.
	 * @param string $event              Event to determine type of object.
	 */
	public function may_be_trigger_associated_feeds( $post_id, $associated_feeds, $event ) {

		foreach ( $associated_feeds as $key => $feed_id ) {
			if ( 'create_and_update_product' === $event ) {
				$this->trigger_product_related_feed( $feed_id, $post_id, 'true' );
			} elseif ( 'shop_order_updated' === $event ) {
				$this->perform_shop_order_sync( $feed_id, $post_id );
			}
		}
	}

	/**
	 * Prepare/Fire a request whenever a shop order is created/updated.
	 *
	 * @param string $post_id - Woo Object post ID.
	 * @param string $post    - Woo Object post.
	 *
	 * @since  1.0.0
	 */
	public function shop_order_updated( $post_id, $post = array() ) {

		$wc_order = wc_get_order($post_id);
		// Get the ID of the selected checkout page from WooCommerce settings.
		$checkout_page_id = get_option( 'woocommerce_checkout_page_id' );

		// Get the content of the checkout page.
		$checkout_page_content = get_post_field( 'post_content', $checkout_page_id );

		// Check if the content contains a class associated with the block editor.
		if ( strpos( $checkout_page_content, 'wp-block-woocommerce-checkout' ) === false ) {
			$wps_traditional_checkout = true;
		} else {
			$wps_traditional_checkout = false;
		}

		if ( ! $wps_traditional_checkout ) {
			if ( 'checkout-draft' == $wc_order->get_status() ) {
				return;
			}
		}
		if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {

			$order = wc_get_order($post_id);
			if ( $order ) {

				$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'yes' );
	
				$order->save();
			} else {
				update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
			}
		} else {
			update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
		}

		if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
			return;
		}

		$feed_id = self::is_instant_sync_required();

		if ( empty( $feed_id ) ) {
			return;
		}

		$feeds_array = array();

		if ( is_numeric( $feed_id ) ) {
			$feeds_array[] = $feed_id;
		} elseif ( is_array( $feed_id ) ) {
			sort( $feed_id );
			$feeds_array = $feed_id;
		}

		foreach ( $feeds_array as $key => $f_id ) {

			// Sync current feed.
			$this->perform_shop_order_sync( $f_id, $post_id );

			// Get associated feeds.
			$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );

			/**
			 * Trigger associated Feeds.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $post_id  Woo post id.
			 * @param mixed $associated_feeds associated feeds.
			 * @param mixed shop_order_updated callback.
			 */
			do_action(
				'mwb_salesforce_trigger_associated_feeds',
				$post_id,
				$associated_feeds,
				'shop_order_updated'
			);
		}
	}

	public function subscription_created( $subscription ) {

		$post_id = $subscription->get_data()['parent_id'];
		if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {

			$order = wc_get_order($post_id);
			if ( $order ) {

				$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'yes' );
	
				$order->save();
			} else {
				update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
			}
		} else {
			update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
		}

		if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
			return;
		}

		$feed_id = self::is_instant_sync_required();

		if ( empty( $feed_id ) ) {
			return;
		}

		$feeds_array = array();

		if ( is_numeric( $feed_id ) ) {
			$feeds_array[] = $feed_id;
		} elseif ( is_array( $feed_id ) ) {
			sort( $feed_id );
			$feeds_array = $feed_id;
		}

		foreach ( $feeds_array as $key => $f_id ) {

			// Sync current feed.
			$this->perform_shop_order_sync( $f_id, $post_id );

			// Get associated feeds.
			$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );

			/**
			 * Trigger associated Feeds.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $post_id  Woo post id.
			 * @param mixed $associated_feeds associated feeds.
			 * @param mixed shop_order_updated callback.
			 */
			do_action(
				'mwb_salesforce_trigger_associated_feeds',
				$post_id,
				$associated_feeds,
				'shop_order_updated'
			);
		}
	}


	public function subscription_status_change( $subscription_id, $old_status, $new_status ) {
		$subscription = wcs_get_subscription($subscription_id);

		// Check if the subscription object exists
		if ($subscription) {
			// Get the parent ID
			$post_id = $subscription->get_parent_id();
	
			if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {

				$order = wc_get_order($post_id);
				if ( $order ) {
	
					$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'yes' );
		
					$order->save();
				} else {
					update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
				}
			} else {
				update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
			}
	
			if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
				return;
			}
	
			$feed_id = self::is_instant_sync_required();
	
			if ( empty( $feed_id ) ) {
				return;
			}
	
			$feeds_array = array();
	
			if ( is_numeric( $feed_id ) ) {
				$feeds_array[] = $feed_id;
			} elseif ( is_array( $feed_id ) ) {
				sort( $feed_id );
				$feeds_array = $feed_id;
			}
	
			foreach ( $feeds_array as $key => $f_id ) {
	
				// Sync current feed.
				$this->perform_shop_order_sync( $f_id, $post_id );
	
				// Get associated feeds.
				$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );
	
				/**
				 * Trigger associated Feeds.
				 *
				 * @since 1.0.3
				 *
				 * @param mixed  $post_id  Woo post id.
				 * @param mixed $associated_feeds associated feeds.
				 * @param mixed shop_order_updated callback.
				 */
				do_action(
					'mwb_salesforce_trigger_associated_feeds',
					$post_id,
					$associated_feeds,
					'shop_order_updated'
				);
			}
		}
	}

	public function subscription_update( $subscription_id, $subscription ) {
		$subscription = wcs_get_subscription($subscription_id);

		// Check if the subscription object exists
		if ($subscription) {
			// Get the parent ID
			$post_id = $subscription->get_parent_id();
	
			if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {

				$order = wc_get_order($post_id);
				if ( $order ) {
	
					$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'yes' );
		
					$order->save();
				} else {
					update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
				}
			} else {
				update_post_meta( $post_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
			}
	
			if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
				return;
			}
	
			$feed_id = self::is_instant_sync_required();
	
			if ( empty( $feed_id ) ) {
				return;
			}
	
			$feeds_array = array();
	
			if ( is_numeric( $feed_id ) ) {
				$feeds_array[] = $feed_id;
			} elseif ( is_array( $feed_id ) ) {
				sort( $feed_id );
				$feeds_array = $feed_id;
			}
	
			foreach ( $feeds_array as $key => $f_id ) {
	
				// Sync current feed.
				$this->perform_shop_order_sync( $f_id, $post_id );
	
				// Get associated feeds.
				$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );
	
				/**
				 * Trigger associated Feeds.
				 *
				 * @since 1.0.3
				 *
				 * @param mixed  $post_id  Woo post id.
				 * @param mixed $associated_feeds associated feeds.
				 * @param mixed shop_order_updated callback.
				 */
				do_action(
					'mwb_salesforce_trigger_associated_feeds',
					$post_id,
					$associated_feeds,
					'shop_order_updated'
				);
			}
		}
	}


	public function sync_membership_data( $object, $args ) {

		if ( isset( $args['is_update'] ) && true === $args['is_update'] ) {

			$post_id = $args['user_membership_id'];
		
			$feed_id = self::is_instant_sync_required();
	
			if ( empty( $feed_id ) ) {
				return;
			}
	
			$feeds_array = array();
	
			if ( is_numeric( $feed_id ) ) {
				$feeds_array[] = $feed_id;
			} elseif ( is_array( $feed_id ) ) {
				sort( $feed_id );
				$feeds_array = $feed_id;
			}
	
			foreach ( $feeds_array as $key => $f_id ) {
	
				$this->perform_shop_order_sync( $f_id, $post_id );
				// Get associated feeds.
				$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );
	
				/**
				 * Trigger associated Feeds.
				 *
				 * @since 1.0.3
				 *
				 * @param mixed  $post_id  Woo post id.
				 * @param mixed $associated_feeds associated feeds.
				 * @param mixed shop_order_updated callback.
				 */
				do_action(
					'mwb_salesforce_trigger_associated_feeds',
					$post_id,
					$associated_feeds,
					'shop_order_updated'
				);
			}
			
		}
	}

	public function sync_manually_created_membership_data( $plan_obj, $args ) {

		$post_id = $args['user_membership_id'];
		
		$feed_id = self::is_instant_sync_required();

		if ( empty( $feed_id ) ) {
			return;
		}

		$feeds_array = array();

		if ( is_numeric( $feed_id ) ) {
			$feeds_array[] = $feed_id;
		} elseif ( is_array( $feed_id ) ) {
			sort( $feed_id );
			$feeds_array = $feed_id;
		}

		foreach ( $feeds_array as $key => $f_id ) {

			// Sync current feed.

			$this->perform_shop_order_sync( $f_id, $post_id );
			// Get associated feeds.
			$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );

			/**
			 * Trigger associated Feeds.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  $post_id  Woo post id.
			 * @param mixed $associated_feeds associated feeds.
			 * @param mixed shop_order_updated callback.
			 */
			do_action(
				'mwb_salesforce_trigger_associated_feeds',
				$post_id,
				$associated_feeds,
				'shop_order_updated'
			);
		}
	}

	/**
	 * Trigger order items feed.
	 *
	 * @param string $feed_id Feed id.
	 * @param string $post_id Order id.
	 *
	 * @since  1.0.0
	 */
	public function trigger_order_items_feed( $feed_id, $post_id ) {
		// All checks are already done.
		// Move further with feed logic.

		$post_type   = 'shop_order';
		$record_type = $this->framework_instance->get_feed(
			$feed_id,
			'crm_object'
		);
		$order       = wc_get_order( $post_id );
		$request_ids = array();
		$request_arr = array();
		$prod_ids = array();
		$feed_title  = $this->framework_instance->get_feed_title( $feed_id );

		$dependencies  = $this->feed_module->get_feed_dependencies( $record_type );
		$relationship  = array();
		$result_array  = array();
		$request_data  = array();
		$primary_field = $this->framework_instance->get_feed(
			$feed_id,
			'primary_field'
		);
		$filter_exist  = $this->maybe_check_filter( $feed_id );

		if ( ! $order ) {
			return false;
		}
		$order_items = $order->get_items();
		// $order_items = $order->get_items( array( 'line_item', 'fee', 'shipping' ) );
		if ( 0 === count( $order_items ) ) {
			return false;
		}

		foreach ( $order_items as $items_key => $items_value ) {
			$line_item  = array();
			$items_id   = ! empty( $items_value->get_variation_id() ) ?
			$items_value->get_variation_id() :
			$items_value->get_product_id();
			$prod_ids[] = $items_id; 
			$extra_args = array(
				'order_id'   => $post_id,
				'product_id' => $items_id,
			);
			
			$line_item = $this->framework_instance->get_request(
				'order_item',
				$feed_id,
				$post_id,
				$extra_args,
				$items_key
			);

			$line_item['item_id'] = $items_key;
			if ( isset( $line_item['Quantity'] ) ) {
				$line_item['Quantity'] = $items_value->get_quantity();
			}
			if ( isset( $line_item['UnitPrice'] ) ) {
				// $product                = $items_value->get_product();
				// $line_item['UnitPrice'] = $product->get_price();
				// $line_item['UnitPrice'] = floatval( $order->get_item_total( $items_value, false, false ) );
				$line_item['UnitPrice'] = floatval( $items_value->get_total() ) / $items_value->get_quantity();
			}
			if ( isset( $line_item['TotalPrice'] ) ) {
				// $product                 = $items_value->get_product();
				// $line_item['TotalPrice'] = $product->get_price();
				// $line_item['TotalPrice'] = floatval( $order->get_item_total( $items_value, false, false ) );
				$line_item['TotalPrice'] = floatval( $items_value->get_total() );
			}

			$request_arr[ $items_key ] = $line_item;
		}

		if ( 0 === count( $request_arr ) ) {
			return false;
		}
		
		foreach ( $request_arr as $k => $data ) {
			$relationship[ $data['item_id'] ] = $data['post_id'];

			$flag = true;
			foreach ( $dependencies as $dk => $dv ) {
				if ( empty( $data[ $dk ] ) ) {
					$result_array[ $data['item_id'] ] = array(
						'id'      => '',
						'post_id' => $data['post_id'],
						'message' => 'Missing Dependency : ' . $dk,
					);
					$flag                             = false;
				}
			}
			if ( $flag ) {
				if ( ! empty( $data['id'] ) ) {
					$result_array[ $data['item_id'] ] = array(
						'id'      => '',
						'post_id' => $data['post_id'],
						'message' => 'Items aleady synced',
					);
				} else {
					$request_ids[] = $data['item_id'];
					unset( $data['item_id'] );
					unset( $data['post_id'] );
					$request_data[] = $data;
				}
			}
		}

		$result = array();
		if ( ! empty( $request_data ) && is_array( $request_data ) ) {
			$result = $this->api_module->bulk_create_or_update_record( $record_type, $request_data, $primary_field, $feed_id );
			foreach ( $result as $res_key => $res_value ) {
				if ( true == $res_value['success'] && !empty( $res_value['id'] ) ) {
					$framework_base = new Integration_With_Salesforce_Framework_Base();
					$feed_id = $framework_base->get_object_feed( 'Product2' );
					if ( ! empty( $feed_id ) && is_numeric( $feed_id ) ) {

						$this->trigger_product_related_feed( $feed_id, $prod_ids[$res_key], 'true' );
						$associated_feeds = $this->feed_module->get_associated_feeds( $feed_id );
			
						/**
						 * Trigger associated Feeds.
						 *
						 * @since 1.0.3
						 *
						 * @param mixed  $post_id  Woo post id.
						 * @param mixed $associated_feeds associated feeds.
						 * @param mixed shop_order_updated callback.
						 */
						do_action(
							'mwb_salesforce_trigger_associated_feeds',
							$prod_ids[$res_key],
							$associated_feeds,
							'create_and_update_product'
						);
					}
				}
			}
			if ( ! empty( $filter_exist ) ) {
				$filtered_ids = array();
				if ( ! empty( $request_arr ) ) {
					foreach ( $request_arr as $key => $data ) {
						$filter_result = $this->mwb_salesforce_validate_filter( $filter_exist, $data );
						if ( true === $filter_result ) {
							array_push( $filtered_ids, $data['item_id'] );

						} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
							array_push( $filtered_ids, $data['item_id'] );
						}
					}
				}
				$request_ids = $filtered_ids;
			} else {
				$request_ids = $request_ids;
			}
			if ( ! empty( $result ) ) {
				if ( count( $request_ids ) == count( $result ) ) {
					$result = array_combine( $request_ids, $result );
				}
			}
		}
		$result_array         = array_replace( $result_array, $result );
		$order_item_sync_meta = array();
		$log_data             = array();

		foreach ( $result_array as $line_post_id => $line_response ) {
			$line_id = '';
			if ( ! empty( $line_response['id'] ) ) {
				$order_item_sync_meta[] = $line_response;
				$line_id                = $line_response['id'];
			}

			$log_data = array(
				'woo_object'        => $post_type,
				'woo_id'            => $line_post_id,
				'salesforce_object' => $record_type,
				'salesforce_id'     => $line_id,
				'event'             => $feed_title,
				'request'           => wp_json_encode( $request_arr[ $line_post_id ] ),
				'response'          => wp_json_encode( $line_response ),
				'time'              => time(),
			);
			mwb_woo_salesforce_insert_log( $log_data );
		}

		if ( ! empty( $order_item_sync_meta ) ) {
			if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
				$order = wc_get_order($post_id);
				if ( $order ) {

					$order->update_meta_data( 'mwb_salesforce_' . $record_type . '_id', $order_item_sync_meta );
					$order->save();
				} else {
					update_post_meta( $post_id, 'mwb_salesforce_' . $record_type . '_id', $order_item_sync_meta );
				}
			} else {
				update_post_meta( $post_id, 'mwb_salesforce_' . $record_type . '_id', $order_item_sync_meta );
			}
		}
		return true;
	}

	/**
	 * Do the shop order update by feed id.
	 *
	 * @param string $feed_id Feed id.
	 * @param string $post_id Order id.
	 *
	 * @since  1.0.0
	 * @return null
	 */
	public function perform_shop_order_sync( $feed_id = false, $post_id = false ) {

		if ( ! empty( $feed_id ) && is_numeric( $feed_id ) ) {

			$post = get_post( $post_id );
			$order = wc_get_order( $post_id );
			$admin_instance = new Integration_With_Salesforce_Admin( 'Salesforce Integration for WooCommerce', '1.2.1' );

			if ( $admin_instance->wps_is_woocommerce_subscription_plugin_active() && 'no' == get_option( 'mwb-salesforce-sync-renewal', 'no' ) ) {
				$is_renewal_order     = wcs_order_contains_renewal( $order );
				if ( true == $is_renewal_order ) {
					return;
				}
			}
			$post_type  = Integration_With_Salesforce_Order_Handler::wps_get_object_type($post_id);
			if ( 'wc_user_membership' != $post_type ) {
				if ( !empty( $post->post_parent ) && 0 != $post->post_parent ) {
					return;
				}
			}
			
			if ( ! $post ) {
				return false;
			}

			$record_type = $this->framework_instance->get_feed(
				$feed_id,
				'crm_object'
			);

			// If order item feed trigger.
			if ( 'OrderItem' === $record_type || 'OpportunityLineItem' === $record_type || 'QuoteLineItem' === $record_type ) {
				$this->trigger_order_items_feed( $feed_id, $post_id );
				return;
			}

			if ( 'Account' === $record_type ) {
				if ( empty( $this->framework_instance->may_be_get_field_value_from_existing_orders( $post_id, 'mwb_salesforce_Account_id' ) ) ) {
					$this->check_if_account_already_exists_in_salesforce( $feed_id, $post_id, 'shop_order');
				}
			}

			$filter_exist = $this->maybe_check_filter( $feed_id );
			
			// Else continue with normal order feeds.
			$request = $this->framework_instance->get_request(
				$post_type,
				$feed_id,
				$post_id
			);
			/**
			 * Filters the value of Salesforce new order request.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed $request  Woo new order request.
			 * @param mixed $post_id post ids.
			 */
			$request = apply_filters(
				'woo_crm_salesforce_new_order_request',
				$request,
				$post_id
			);

			$record_type = $this->framework_instance->get_feed(
				$feed_id,
				'crm_object'
			);

			$feed_title = $this->framework_instance->get_feed_title( $feed_id );

			$primary_field = $this->framework_instance->get_feed(
				$feed_id,
				'primary_field'
			);

			$dependencies = $this->feed_module->get_feed_dependencies( $record_type );
			$result_array = array();
			$request_data = array();
			$request_arr  = array( $request );
			$request_ids  = array( $request['post_id'] );
			$result_data  = array();
			$flag         = true;
			
			if ( ! empty( $dependencies ) ) {
				foreach ( $dependencies as $dk => $dv ) {
					if ( empty( $request[ $dk ] ) ) {
						// Dependency not found add error in result array.
						$result_array[ $request['post_id'] ] = array(
							'id'      => '',
							'post_id' => $request['post_id'],
							'message' => 'Missing Dependency : ' . $dk,
						);

						$flag = false;
						break;
					}
				}
			}

			if ( $flag ) {
				unset( $request['post_id'] );
				if ( isset( $request['related_order_ids'] ) ) {
					unset( $request['related_order_ids'] );
				}
				$request_data[] = $request;
			}
			
			if ( ! empty( $request_data ) && is_array( $request_data ) ) {
				$result_data = $this->api_module->bulk_create_or_update_record( $record_type, $request_data, $primary_field, $feed_id );
				if ( ! empty( $filter_exist ) ) {
					$filtered_ids = array();
					if ( ! empty( $request_arr ) ) {
						foreach ( $request_arr as $key => $data ) {
							$filter_result = $this->mwb_salesforce_validate_filter( $filter_exist, $data );
							if ( true === $filter_result ) {
								array_push( $filtered_ids, $data['post_id'] );

							} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
								array_push( $filtered_ids, $data['post_id'] );
							}
						}
					}
					$request_ids = $filtered_ids;
				} else {
					$request_ids = $request_ids;
				}

				if ( ! empty( $result_data ) ) {
					if ( count( $request_ids ) == count( $result_data ) ) {
						$result_data = array_combine( $request_ids, $result_data );
					}
				}
				$result_array = array_replace( $result_array, $result_data );
			}

			foreach ( $result_array as $line_post_id => $line_response ) {
				$line_id = '';
				if ( ! empty( $line_response['id'] ) ) {
					$line_id = $line_response['id'];
					if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
						$order = wc_get_order( $line_post_id );
						if ( $order ) {

							$order->update_meta_data( 'mwb_salesforce_' . $record_type . '_id', $line_id );
							$order->save();
						} else {
							update_post_meta( $line_post_id, 'mwb_salesforce_' . $record_type . '_id', $line_id );
						}
					} else {
						update_post_meta( $line_post_id, 'mwb_salesforce_' . $record_type . '_id', $line_id );
					}
				}
				$log_data = array(
					'woo_object'        => $post_type,
					'woo_id'            => $post_id,
					'salesforce_object' => $record_type,
					'salesforce_id'     => $line_id,
					'event'             => $feed_title,
					'request'           => wp_json_encode( $request ),
					'response'          => wp_json_encode( $line_response ),
					'time'              => time(),
				);
				mwb_woo_salesforce_insert_log( $log_data );
			}
		}
	}

	/**
	 * Query object ids.
	 *
	 * @param string $post_type   Post type.
	 * @param string $return_type Return type.
	 * @param string $meta_key    Meta key to search.
	 *
	 * @since  1.0.0
	 * @return array              Array if object ids.
	 */
	public function get_woo_objects( $post_type, $return_type = 'ids', $meta_key = '' ) {

		$order_status = array_keys( wc_get_order_statuses() );

		$post_status = ( 'product' === $post_type ) ? 'all' : $order_status;

		if ( 'users' === $post_type ) {
			$args = array(
				'fields' => 'ID',
			);
			if ( '' !== $meta_key ) {
			
				$args['meta_query'] = array( //phpcs:ignore
					'relation' => 'OR',
					array(
						'key'     => $meta_key,
						'compare' => 'NOT EXISTS',
					),
					array(
						'key'     => $meta_key,
						'compare' => '=',
						'value'   => '',
					),
				);
			}
			$user_ids = get_users( $args );
			$posts = $user_ids;
		} elseif ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
			if ( 'product' === $post_type ) {
				$args = array(
					'post_type'      =>$post_type,
					'posts_per_page' => -1,
					'post_status'    => $post_status,
					'fields'         => $return_type,
				);
			
				if ( '' !== $meta_key ) {
			
					$args['meta_query'] = array( //phpcs:ignore
						'relation' => 'OR',
						array(
							'key'     => $meta_key,
							'compare' => 'NOT EXISTS',
						),
						array(
							'key'     => $meta_key,
							'compare' => '=',
							'value'   => '',
						),
					);
				}

				$posts = get_posts( $args );
			} else {

				if ( '' !== $meta_key ) {
					$query = new WC_Order_Query( array(
						'limit' => -1,
						'status' => $order_status,
						'orderby' => 'date',
						'order' => 'DESC',
						'return' => 'ids',
						'meta_query' =>array( //phpcs:ignore
							'relation' => 'OR',
							array(
								'key'     => $meta_key,
								'compare' => 'NOT EXISTS',
							),
							array(
								'key'     => $meta_key,
								'compare' => '=',
								'value'   => '',
							),
						),
					) );
				} else {
					$query = new WC_Order_Query( array(
						'limit' => -1,
						'status' => $post_status,
						'orderby' => 'date',
						'order' => 'DESC',
						'return' => 'ids',
						)
					);

				}
				$posts = $query->get_orders();
			}
		} else {
			$args = array(
				'post_type'      =>$post_type,
				'posts_per_page' => -1,
				'post_status'    => $post_status,
				'fields'         => $return_type,
			);
		
			if ( '' !== $meta_key ) {
		
				$args['meta_query'] = array( //phpcs:ignore
					'relation' => 'OR',
					array(
						'key'     => $meta_key,
						'compare' => 'NOT EXISTS',
					),
					array(
						'key'     => $meta_key,
						'compare' => '=',
						'value'   => '',
					),
				);
			}
			$posts = get_posts( $args );
		}
		return $posts;
	}

	/**
	 * Get object ids required for sync.
	 *
	 * @param string $feed_id   Feed id.
	 * @param string $sync_type bulk sync/one click sync.
	 *
	 * @since  1.0.0
	 * @return array $object_ids
	 */
	public function get_feed_required_object_ids( $feed_id, $sync_type ) {
		$object_ids  = array();
		$record_type = $this->framework_instance->get_feed( $feed_id, 'crm_object' );
		$object_type = $this->get_feed_woo_object( $record_type, $feed_id );
		if ( 'OrderItem' === $record_type || 'OpportunityLineItem' === $record_type || 'QuoteLineItem' === $record_type ) {
			$object_type = 'shop_order';
		}

		if ( 'order_item' === $object_type ) {
			$object_type = 'shop_order';
		}

		$meta_key = '';
		if ( 'ocs_sync' === $sync_type ) {
			$meta_key = 'mwb_salesforce_' . $record_type . '_id';
		}

		$object_ids = $this->get_woo_objects( $object_type, 'ids', $meta_key );
		$option_key = 'mwb_woo_salesforce_' . $record_type . '_' . $sync_type . '_sync_ids';

		update_option( $option_key, $object_ids );
		return $object_ids;
	}

	/**
	 * Prepare/Retrieve a request full of all the post entities specified.
	 *
	 * Retrieves the bulk data in crm request format.
	 *
	 * @param string $feed_id Feed id.
	 * @param array  $post_ids Array of post ids.
	 *
	 * @since  1.0.0
	 * @return array $bulk_request
	 */
	public function get_bulk_data( $feed_id = false, $post_ids = array() ) { 

		if ( false == $feed_id ) {
			return;
		}
		$record_type   = $this->framework_instance->get_feed( $feed_id, 'crm_object' );
		$object_type   = $this->get_feed_woo_object( $record_type, $feed_id );
		$primary_field = $this->framework_instance->get_feed( $feed_id, 'primary_field' );
		$feed_title    = $this->framework_instance->get_feed_title( $feed_id );
		$order_status  = array_keys( wc_get_order_statuses() );
		
		switch ( $object_type ) {
			case 'users':
				$args = array(
					'include' => $post_ids,
				);

				$users        = get_users( $args );
				$bulk_request = array();
				$bulk_ids     = array();
				$total        = count( $users );

				foreach ( $users as $user ) {

					$request = $this->framework_instance->get_request(
						$object_type,
						$feed_id,
						$user->ID
					);
					
					if ( 'Account' === $record_type ) {
						/**
						 * Filters the value of account array.
						 *
						 * @since 1.0.0
						 *
						 * @param mixed $request request.
						 * @param mixed $user_id user_id.
						 * @param mixed $record_type record_type.
						 */
						$request = apply_filters(
							'wps_salesforce_user_request',
							$request,
							$user->ID,
							$record_type
						);
									/**
						 * Filters the value of bulk request array.
						 *
						 * @since 1.0.0
						 *
						 * @param mixed $request request.
						 * @param mixed $user_id user_id.
						 * @param mixed $record_type record_type.
						 */
						$request = apply_filters(
							'filter_account_request_data_for_user',
							$request,
							$user->ID,
							$feed_id
						);
					} else {
						/**
						 * Filters the value of user account array.
						 *
						 * @since 1.0.0
						 *
						 * @param mixed $request request.
						 * @param mixed $user_id user_id.
						 * @param mixed $record_type record_type.
						 */
						$request = apply_filters(
							'wps_salesforce_user_request',
							$request,
							$user->ID,
							$record_type
						);
					}

					array_push( $bulk_request, $request );
					$bulk_ids[] = $user->ID;
				}
				/**
				 * Filters the value of bulk request array.
				 *
				 * @since 1.0.0
				 *
				 * @param mixed $bulk_request request.
				 * @param mixed $bulk_ids bulk_id.
				 */
				$bulk_request = apply_filters(
					'woo_crm_woo_zoho_user_bulk_request',
					$bulk_request,
					$bulk_ids
				);
				break;

			case 'shop_order':
			$args = array(
				'post_type'      => $object_type,
				'post_status'    => $order_status,
				'post__in'       => $post_ids,
				'posts_per_page' => count( $post_ids ),
			);

				try {

					$order_query = new WP_Query( $args );
					$bulk_request = array();
					$bulk_ids     = array();
					
					$total = $order_query->found_posts;
					
					if (Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled()) {
						if ( 'shop_order' == $object_type ) {
							$args = array(
								'post_type'      => $object_type,
								'post_status'    => $order_status,
								'post__in'       => $post_ids,
								'return'         => 'ids',
								'posts_per_page' => count( $post_ids ),
							);
							$order_query = new WC_Order_Query( $args );
							foreach ( $order_query->get_orders() as $key => $id ) :
								// $order_query->the_post();
								
								$wc_order = wc_get_order($id);
								// Get the ID of the selected checkout page from WooCommerce settings.
								$checkout_page_id = get_option( 'woocommerce_checkout_page_id' );

								// Get the content of the checkout page.
								$checkout_page_content = get_post_field( 'post_content', $checkout_page_id );

								// Check if the content contains a class associated with the block editor.
								if ( strpos( $checkout_page_content, 'wp-block-woocommerce-checkout' ) === false ) {
									$wps_traditional_checkout = true;
								} else {
									$wps_traditional_checkout = false;
								}

								if ( ! $wps_traditional_checkout ) {
									if ( 'checkout-draft' == $wc_order->get_status() ) {
										continue;
									}
								}

								$request = $this->framework_instance->get_request(
								$object_type,
								$feed_id,
								$id
								);
								array_push( $bulk_request, $request );
								$bulk_ids[] = $id;
								/**
								 * Filters the value of Salesforce update order bulk request.
								 *
								 * @since 1.0.3
								 *
								 * @param mixed  $bulk_request  Woo bulk request.
								 * @param mixed $bulk_ids Crm bulk ids.
								 */
								$bulk_request = apply_filters(
									'woo_crm_salesforce_update_order_bulk_request',
									$bulk_request,
									$bulk_ids
								);
							endforeach;
						} else {
							while ( $order_query->have_posts() ) :
								$order_query->the_post();
								
								$request = $this->framework_instance->get_request(
								$object_type,
								$feed_id,
								get_the_ID()
								);
								array_push( $bulk_request, $request );
								$bulk_ids[] = get_the_ID();
								/**
								 * Filters the value of Salesforce update order bulk request.
								 *
								 * @since 1.0.3
								 *
								 * @param mixed  $bulk_request  Woo bulk request.
								 * @param mixed $bulk_ids Crm bulk ids.
								 */
								$bulk_request = apply_filters(
									'woo_crm_salesforce_update_order_bulk_request',
									$bulk_request,
									$bulk_ids
								);
							endwhile;
						}
					} else {
						
						$posts = get_posts( $args );
						while ( $order_query->have_posts() ) :
							$order_query->the_post();
						
							$wc_order = wc_get_order(get_the_ID());
							// Get the ID of the selected checkout page from WooCommerce settings.
							$checkout_page_id = get_option( 'woocommerce_checkout_page_id' );

							// Get the content of the checkout page.
							$checkout_page_content = get_post_field( 'post_content', $checkout_page_id );

							// Check if the content contains a class associated with the block editor.
							if ( strpos( $checkout_page_content, 'wp-block-woocommerce-checkout' ) === false ) {
								$wps_traditional_checkout = true;
							} else {
								$wps_traditional_checkout = false;
							}

							if ( ! $wps_traditional_checkout ) {
								if ( 'checkout-draft' == $wc_order->get_status() ) {
									continue;
								}
							}
							$request = $this->framework_instance->get_request(
							$object_type,
							$feed_id,
							get_the_ID()
							);
							array_push( $bulk_request, $request );
							$bulk_ids[] = get_the_ID();
							/**
							 * Filters the value of Salesforce update order bulk request.
							 *
							 * @since 1.0.3
							 *
							 * @param mixed  $bulk_request  Woo bulk request.
							 * @param mixed $bulk_ids Crm bulk ids.
							 */
							$bulk_request = apply_filters(
								'woo_crm_salesforce_update_order_bulk_request',
								$bulk_request,
								$bulk_ids
							);
						endwhile;
					}

				} catch ( \Throwable $th ) {
					wp_die( esc_html( $th->getMessage() ) );
				}
				break;

			case 'order_item':
			$args = array(
				'post_type'      => 'shop_order',
				'post_status'    => $order_status,
				'post__in'       => $post_ids,
				'posts_per_page' => count( $post_ids ),
			);

				try {

					$order_query = new WP_Query( $args );

					$bulk_request = array();
					$bulk_ids     = array();

					$total = $order_query->found_posts;
					$count = 0;
					if (Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled()) {
						if ( 'shop_order' ) { 
							foreach ( $post_ids as $key => $id ) :
								// $order_query->the_post();
								$order = wc_get_order( $id );
								if ( $order ) {
									$order_items = $order->get_items();
									$request = array();
									if ( count( $order_items ) ) {
										foreach ( $order_items as $items_key => $items_value ) {
		
											$line_item  = array();
											$line_product = $items_value->get_product();
											if ( ! $line_product ) {
												continue;
											}
											$items_id   = ! empty( $items_value->get_variation_id() ) ?
											$items_value->get_variation_id() :
											$items_value->get_product_id();
											$extra_args = array(
											'order_id'   => $id,
											'product_id' => $items_id,
											);
											
											$line_item = $this->framework_instance->get_request(
											'order_item',
											$feed_id,
											$id,
											$extra_args
											);
		
											$line_item['item_id'] = $items_key;
											if ( isset( $line_item['Quantity'] ) ) {
												$line_item['Quantity'] = $items_value->get_quantity();
											}
											if ( isset( $line_item['UnitPrice'] ) ) {
												$line_item['UnitPrice'] = floatval( $items_value->get_subtotal() ) / $items_value->get_quantity();
												//$line_item['UnitPrice'] = floatval( $order->get_item_total( $items_value, false, false ) );
												//$line_item['UnitPrice'] = $line_product->get_price();
											}
											array_push( $bulk_request, $line_item );
											$request[]  = $line_item;
											$bulk_ids[] = $items_key;
										}
									}
								}
								/**
								 * Filters the value of Salesforce update order bulk request.
								 *
								 * @since 1.0.3
								 *
								 * @param mixed  $bulk_request  Woo bulk request.
								 * @param mixed $bulk_ids Crm bulk ids.
								 */
								$bulk_request = apply_filters(
								'woo_crm_salesforce_update_order_bulk_request',
								$bulk_request,
								$bulk_ids
								);
							endforeach;
						} else {
							while ( $order_query->have_posts() ) :
								$order_query->the_post();
								$order = wc_get_order( get_the_ID() );
								if ( $order ) {
									$order_items = $order->get_items();
									$request = array();
									if ( count( $order_items ) ) {
										foreach ( $order_items as $items_key => $items_value ) {
		
											$line_item  = array();
											$line_product = $items_value->get_product();
											if ( ! $line_product ) {
												continue;
											}
											$items_id   = ! empty( $items_value->get_variation_id() ) ?
											$items_value->get_variation_id() :
											$items_value->get_product_id();
											$extra_args = array(
											'order_id'   => get_the_ID(),
											'product_id' => $items_id,
											);
											
											$line_item = $this->framework_instance->get_request(
											'order_item',
											$feed_id,
											get_the_ID(),
											$extra_args
											);
		
											$line_item['item_id'] = $items_key;
											if ( isset( $line_item['Quantity'] ) ) {
												$line_item['Quantity'] = $items_value->get_quantity();
											}
											if ( isset( $line_item['UnitPrice'] ) ) {
												$line_item['UnitPrice'] = floatval( $items_value->get_subtotal() ) / $items_value->get_quantity();
												//$line_item['UnitPrice'] = floatval( $order->get_item_total( $items_value, false, false ) );
												//$line_item['UnitPrice'] = $line_product->get_price();
											}
											array_push( $bulk_request, $line_item );
											$request[]  = $line_item;
											$bulk_ids[] = $items_key;
										}
									}
								}
								/**
								 * Filters the value of Salesforce update order bulk request.
								 *
								 * @since 1.0.3
								 *
								 * @param mixed  $bulk_request  Woo bulk request.
								 * @param mixed $bulk_ids Crm bulk ids.
								 */
								$bulk_request = apply_filters(
									'woo_crm_salesforce_update_order_bulk_request',
									$bulk_request,
									$bulk_ids
								);
							endwhile;
						}
					} else {

						while ( $order_query->have_posts() ) :
							$order_query->the_post();
							$order = wc_get_order( get_the_ID() );
							if ( $order ) {
								$order_items = $order->get_items();
								$request = array();
								if ( count( $order_items ) ) {
									foreach ( $order_items as $items_key => $items_value ) {
	
										$line_item  = array();
										$line_product = $items_value->get_product();
										if ( ! $line_product ) {
											continue;
										}
										$items_id   = ! empty( $items_value->get_variation_id() ) ?
										$items_value->get_variation_id() :
										$items_value->get_product_id();
										$extra_args = array(
										'order_id'   => get_the_ID(),
										'product_id' => $items_id,
										);
	
										$line_item = $this->framework_instance->get_request(
										'order_item',
										$feed_id,
										get_the_ID(),
										$extra_args
										);
	
										$line_item['item_id'] = $items_key;
										if ( isset( $line_item['Quantity'] ) ) {
											$line_item['Quantity'] = $items_value->get_quantity();
										}
										if ( isset( $line_item['UnitPrice'] ) ) {
											$line_item['UnitPrice'] = floatval( $items_value->get_subtotal() ) / $items_value->get_quantity();
											//$line_item['UnitPrice'] = floatval( $order->get_item_total( $items_value, false, false ) );
											//$line_item['UnitPrice'] = $line_product->get_price();
										}
										array_push( $bulk_request, $line_item );
										$request[]  = $line_item;
										$bulk_ids[] = $items_key;
									}
								}
							}
							/**
							 * Filters the value of Salesforce update order bulk request.
							 *
							 * @since 1.0.3
							 *
							 * @param mixed  $bulk_request  Woo bulk request.
							 * @param mixed $bulk_ids Crm bulk ids.
							 */
							$bulk_request = apply_filters(
							'woo_crm_salesforce_update_order_bulk_request',
							$bulk_request,
							$bulk_ids
							);
						endwhile;
					}

				} catch ( \Throwable $th ) {
					wp_die( esc_html( $th->getMessage() ) );
				}
				break;

			case 'product':
				$args = array(
					'post_type'      => $object_type,
					'post_status'    => 'all',
				'post__in'       => $post_ids,
				'posts_per_page' => count( $post_ids ),
			);

				try {
					
					$product_query = new WP_Query( $args );
					$bulk_request  = array();
					$bulk_ids      = array();

					$total = $product_query->found_posts;
					
					while ( $product_query->have_posts() ) :
						
						$product_query->the_post();
						
						// Get product.
						$product = wc_get_product( get_the_ID() );
						
						if ( 'variable' == $product->get_type() || 'variable-subscription' === $product->get_type() ) {
							
							$feed_module     = Integration_With_Salesforce::get_integration_module( 'feed' );
							$product_feed_id = get_option( 'mwb_woo_salesforce_Product2_feed_id', '' );
							if ( ! empty( $product_feed_id ) ) {
								$is_default_feed = $feed_module->get_feed_data( $product_feed_id, 'default_feed', '' );
								if ( $is_default_feed ) {
									$sync_parent_product = $feed_module->get_feed_data( $product_feed_id, 'sync_parent_product', 'no' );
									if ( 'yes' === $sync_parent_product ) {
										
										$request = $this->framework_instance->get_request(
											$object_type,
											$feed_id,
											get_the_ID()
										);
										array_push( $bulk_request, $request );
										$bulk_ids[] = get_the_ID();
									}
								}
							}

							$available_variations = $product->get_available_variations();
							if ( ! empty( $available_variations )
							&& is_array( $available_variations )
							) {
								foreach ( $available_variations as $key => $variation ) {
									
									$request = $this->framework_instance->get_request(
									$object_type,
									$feed_id,
									$variation['variation_id']
									);
									array_push( $bulk_request, $request );
									$bulk_ids[] = $variation['variation_id'];
								}
							}
						} else {
								
							$request = $this->framework_instance->get_request(
								$object_type,
								$feed_id,
								get_the_ID()
							);
						
							
							array_push( $bulk_request, $request );
							$bulk_ids[] = get_the_ID();
						}

						/**
						 * Filters the value of Salesforce update order bulk request.
						 *
						 * @since 1.0.3
						 *
						 * @param mixed  $bulk_request  Woo bulk request.
						 * @param mixed $bulk_ids Crm bulk ids.
						 */
						$bulk_request = apply_filters(
							'woo_crm_salesforce_update_order_bulk_request',
							$bulk_request,
							$bulk_ids
						);

					endwhile;
						
				} catch ( \Throwable $th ) {
					wp_die( esc_html( $th->getMessage() ) );
				}
				
				break;
				
			default:
				$bulk_request = false;
				break;
		}

		$result = array(
			'request'       => $bulk_request,
			'bulk_ids'      => $bulk_ids,
			'object'        => $record_type,
			'woo_object'    => $object_type,
			'primary_field' => $primary_field,
			'feed_title'    => $feed_title,
		);

		return $result;
	}


	/**
	 * Prepare/Retrieve a request full of all the post entities specified.
	 *
	 * Retrieves the bulk data in crm request format.
	 *
	 * @param string $user_id Woo Object post ID.
	 * @param string $order    Woo Object post.
	 *
	 * @since 1.0.0
	 */
	public function create_and_update_contact( $user_id, $order ) {
		// var_dump($user_id);
		
		if ( ! $user_id ) {
			return;
		}

		// update_user_meta( $user_id, 'mwb_salesforce_allow_background_syncing', 'yes' );

		if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
			return;
		}

		$cache_id = wp_cache_get( 'wps_salesforce_user_synced' );

		if ( $cache_id == $user_id ) {
			return;
		}

		$feed_id = self::is_instant_sync_required();

		if ( is_numeric( $feed_id ) ) {
			$feeds_array[] = $feed_id;
		} elseif ( is_array( $feed_id ) ) {
			sort( $feed_id );
			$feeds_array = $feed_id;
		}

		foreach ( $feeds_array as $key => $f_id ) {
			$this->perform_wp_user_sync( $f_id, $user_id );
		}
	}

	/**
	 * Perfrom WP User sync according to feed.
	 * 
	 * @param  int $feed_id Feed id.
	 * @param  int $user_id User id.
	 *
	 * @since  1.0.2
	 */
	public function perform_wp_user_sync( $feed_id, $user_id ) {
		$request    = $this->framework_instance->get_request(
			'users',
			$feed_id,
			$user_id
		);
		$record_type = $this->framework_instance->get_feed(
			$feed_id,
			'crm_object'
		);
		$request = apply_filters(
			'wps_salesforce_user_request',
			$request,
			$user_id,
			$record_type
		);

		if ( 'Account' === $record_type ) {
			/**
			 * Filters the value of bulk request array.
			 *
			 * @since 1.0.0
			 *
			 * @param mixed $request request.
			 * @param mixed $user_id user_id.
			 * @param mixed $record_type record_type.
			 */
			$request = apply_filters(
				'filter_account_request_data_for_user',
				$request,
				$user_id,
				$feed_id
			);
			// if ( empty( $this->framework_instance->may_be_get_field_value_from_existing_orders( $user_id, 'mwb_salesforce_'.$record_type.'_id' ) ) ) {
				$this->check_if_account_already_exists_in_salesforce( $feed_id, $user_id, 'users' );
			// }
		}

		$filter_exist = $this->maybe_check_filter( $feed_id );

		$feed_title = $this->framework_instance->get_feed_title( $feed_id );

		$primary_field = $this->framework_instance->get_feed(
			$feed_id,
			'primary_field'
		);
		
		$dependencies = $this->feed_module->get_feed_dependencies( $record_type );
		$result_array = array();
		$request_data = array();
		$request_arr  = array( $request );
		$request_ids  = array( $request['user_id'] );
		$result_data  = array();
		$flag         = true;
		
		if ( ! empty( $dependencies ) ) {
			foreach ( $dependencies as $dk => $dv ) {
				if ( empty( $request[ $dk ] ) ) {
					// Dependency not found add error in result array.
					$result_array[ $request['user_id'] ] = array(
						'id'      => '',
						'user_id' => $request['user_id'],
						'message' => 'Missing Dependency : ' . $dk,
					);

					$flag = false;
					break;
				}
			}
		}

		if ( $flag ) {
			unset( $request['user_id'] );
			if ( isset( $request['related_order_ids'] ) ) {
				unset( $request['related_order_ids'] );
			}
			$request_data[] = $request;
		}
//doneeeeeeeeeeeeeeeeeeeee...............................
		if ( ! empty( $request_data ) && is_array( $request_data ) ) {
			$result_data = $this->api_module->bulk_create_or_update_record( $record_type, $request_data, $primary_field, $feed_id );
			if ( ! empty( $filter_exist ) ) {
				$filtered_ids = array();
				if ( ! empty( $request_arr ) ) {
					foreach ( $request_arr as $key => $data ) {
						$filter_result = $this->mwb_salesforce_validate_filter( $filter_exist, $data );
						if ( true === $filter_result ) {
							array_push( $filtered_ids, $data['post_id'] );

						} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
							array_push( $filtered_ids, $data['post_id'] );
						}
					}
				}
				$request_ids = $filtered_ids;
			} else {
				$request_ids = $request_ids;
			}

			if ( ! empty( $result_data ) ) {
				if ( count( $request_ids ) == count( $result_data ) ) {
					$result_data = array_combine( $request_ids, $result_data );
				}
			}
			$result_array = array_replace( $result_array, $result_data );
		}

		foreach ( $result_array as $line_post_id => $line_response ) {
			$line_id = '';
			if ( ! empty( $line_response['id'] ) ) {
				$line_id = $line_response['id'];

					update_user_meta( $line_post_id, 'mwb_salesforce_' . $record_type . '_id', $line_id );
				
			}
			$log_data = array(
				'woo_object'        => 'users',
				'woo_id'            => $user_id,
				'salesforce_object' => $record_type,
				'salesforce_id'     => $line_id,
				'event'             => $feed_title,
				'request'           => wp_json_encode( $request ),
				'response'          => wp_json_encode( $line_response ),
				'time'              => time(),
			);
			mwb_woo_salesforce_insert_log( $log_data );
		}
		// var_dump($record_type,$dependencies, $request, $filter_exist, $feed_title, $primary_field);die();
	}

	/**
	 * Prepare/Retrieve a request full of all the post entities specified.
	 *
	 * Retrieves the bulk data in crm request format.
	 *
	 * @param string $order_id  Woo Object post ID.
	 * @param string $refund_id Woo Refund post ID.
	 *
	 * @since 1.0.0
	 */
	public function shop_order_refunded( $order_id, $refund_id ) {
		$this->shop_order_updated( $order_id );
	}

	/**
	 * Prepare/Retrieve a request full of all the post entities specified.
	 *
	 * Retrieves the bulk data in crm request format.
	 *
	 * @param string $order_id                 Woo Object post ID.
	 * @param string $maybe_refund_id_or_order Woo Object for or refund ID.
	 *
	 * @since 1.0.0
	 */
	public function shop_order_status_changed( $order_id, $maybe_refund_id_or_order = false ) {

		$wc_order = wc_get_order($order_id);
		if ( 'pending' == $wc_order->get_status() ) {
			if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {

				$order = wc_get_order($order_id);
				if ( $order ) {
	
					$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'yes' );
		
					$order->save();
				} else {
					update_post_meta( $order_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
				}
			} else {
				update_post_meta( $order_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
			}
	
			if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
				return;
			}

			$feed_module  = Integration_With_Salesforce_Feed_Module::get_instance();
			$args = array(
				'post_type' => 'mwb_salesforce_feed',
				'post_status' => 'publish',
				'posts_per_page' => -1,
			);
			$all_feeds = get_posts($args);
			foreach ( $all_feeds as $k => $v ) {
				$feed_event    = $feed_module->get_feed_data( $v->ID, 'feed_event', '' );
				if ( 'new-order' == $feed_event ) {
					$feed_id[] = $v->ID;
				}
			}

			if ( empty( $feed_id ) ) {
				return;
			}
	
			$feeds_array = array();
	
			if ( is_numeric( $feed_id ) ) {
				$feeds_array[] = $feed_id;
			} elseif ( is_array( $feed_id ) ) {
				sort( $feed_id );
				$feeds_array = $feed_id;
			}
	
			foreach ( $feeds_array as $key => $f_id ) {
	
				// Sync current feed.
				$this->perform_shop_order_sync( $f_id, $order_id );
	
				// Get associated feeds.
				$associated_feeds = $this->feed_module->get_associated_feeds( $f_id );
	
				/**
				 * Trigger associated Feeds.
				 *
				 * @since 1.0.3
				 *
				 * @param mixed  $order_id  Woo post id.
				 * @param mixed $associated_feeds associated feeds.
				 * @param mixed shop_order_updated callback.
				 */
				do_action(
					'mwb_salesforce_trigger_associated_feeds',
					$order_id,
					$associated_feeds,
					'shop_order_updated'
				);
			}
		} else {
			if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {

				$order = wc_get_order($order_id);
				if ( $order ) {
	
					$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'yes' );
		
					$order->save();
				} else {
					update_post_meta( $order_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
				}
			} else {
				update_post_meta( $order_id, 'mwb_salesforce_allow_background_syncing', 'yes' );
			}
	
			if ( ! Integration_With_Salesforce_Framework_Base::is_instant_sync_enable() ) {
				return;
			}
	
			$feeds = self::get_feed_id_by_request();
	
			// Sort so that we might get deal before the sales orders.
			sort( $feeds );
			if ( ! empty( $feeds ) && is_array( $feeds ) ) {
				foreach ( $feeds as $key => $feed_id ) {
					$this->perform_shop_order_sync( $feed_id, $order_id );
					$associated_feeds = $this->feed_module->get_associated_feeds( $feed_id );
					/**
					 * Trigger associated Feeds.
					 *
					 * @since 1.0.3
					 *
					 * @param mixed  $post_id  Woo post id.
					 * @param mixed $associated_feeds associated feeds.
					 * @param mixed shop_order_updated callback.
					 */
					do_action(
						'mwb_salesforce_trigger_associated_feeds',
						$order_id,
						$associated_feeds,
						'shop_order_updated'
					);
				}
			}
		}
	}

	/**
	 * Get feed associated woocommerce object.
	 *
	 * @param  string $crm_object Name of object.
	 * @param  string $feed_id    Feed id.
	 * @return string            Woo Object key.
	 */
	public function get_feed_woo_object( $crm_object, $feed_id ) {
		$woo_object = '';
		switch ( $crm_object ) {
			case 'Account':
			case 'Contact':
			case 'Lead':
				$woo_object = $this->get_woo_object_based_on_event( $feed_id );
				break;
			case 'Order':
			case 'Opportunity':
			case 'Case':
			case 'Quote':
				$woo_object = 'shop_order';
				break;
			case 'Product2':
			case 'PricebookEntry':
				$woo_object = 'product';
				break;
			case 'OrderItem':
			case 'OpportunityLineItem':
			case 'QuoteLineItem':
				$woo_object = 'order_item';
				break;
			default:
				$woo_object = $this->get_woo_object_based_on_field_mapping( $feed_id );
				break;
		}
		return $woo_object;
	}

	/**
	 * Get woo source object based on field mapping.
	 *
	 * @param  int $feed_id Feed id.
	 *
	 * @since  1.0.2
	 * @return string
	 */
	public function get_woo_object_based_on_field_mapping( $feed_id ) {

		$mapping = $this->framework_instance->get_feed( $feed_id );

		$object_count = array(
			'users'      => 0,
			'shop_order' => 0,
			'product'    => 0,
			'order_item' => 0,
		);

		if ( ! empty( $mapping ) ) {
			foreach ( $mapping as $key => $field ) {
				$field_key = ( 'standard_field' == $field['field_type'] ) ? $field['field_value'] : $field['custom_value'];
				if ( false !== strpos( $field_key, 'users' ) ) {
					$object_count['users']++;
				} elseif ( false !== strpos( $field_key, 'product' ) ) {
					$object_count['product']++;
				} elseif ( false !== strpos( $field_key, 'order_item' ) ) {
					$object_count['order_item']++;
				} else {
					$object_count['shop_order']++;
				}
			}
		}
		$max_objects = array_keys( $object_count, max( $object_count ) );
		$woo_object  = isset( $max_objects[0] ) ? $max_objects[0] : 'shop_order';
		return $woo_object;
	}

	/**
	 * Check if filter exists in feed.
	 *
	 * @param     int $feed_id    Feed ID.
	 * @since     1.0.0
	 * @return    bool|array
	 */
	public function maybe_check_filter( $feed_id = '' ) {

		if ( empty( $feed_id ) ) {
			return;
		}

		if ( 'yes' == get_post_meta( $feed_id, 'mwb-salesforce-enable-filters', true ) ) { // phpcs:ignore
			$meta = get_post_meta( $feed_id, 'mwb-salesforce-condition-field', true );

			if ( ! empty( $meta ) && is_array( $meta ) && count( $meta ) > 0 ) {
				return $meta;
			}
		}

		return false;
	}

	/**
	 * Validate order, products with feeds filter conditions.
	 *
	 * @param     array $filters    An array of filter data.
	 * @param     array $data       Request data.
	 * @since     1.0.0
	 * @return    bool
	 */
	public function mwb_salesforce_validate_filter( $filters = array(), $data = array() ) {

		if ( ! empty( $filters ) && is_array( $filters ) ) {

			foreach ( $filters as $or_key => $or_filters ) {
				$result = true;

				if ( is_array( $or_filters ) ) {

					foreach ( $or_filters as $and_key => $and_filter ) {
						if ( '-1' == $and_filter['field'] || '-1' == $and_filter['option'] ) { // phpcs:ignore
							return array( 'result' => false );
						}

						$post_field = $and_filter['field'];
						$feed_value = ! empty( $and_filter['value'] ) ? $and_filter['value'] : '';
						$entry_val  = $this->mwb_salesforce_integration_get_entry_values( $post_field, $data );
						$result     = $this->is_value_allowed( $feed_value, $and_filter['option'], $entry_val );
						if ( false == $result ) { // phpcs:ignore
							break;
						}
					}
				}

				if ( true === $result ) {
					break;
				}
			}
		}

		return $result;
	}

	/**
	 * Verify and get entered field values.
	 *
	 * @param     string $field      Post field whose value to verify.
	 * @param     array  $entries    An array of data.
	 * @since     1.0.0
	 * @return    mixed              value of the field
	 */
	public function mwb_salesforce_integration_get_entry_values( $field, $entries ) {

		$value = false;

		if ( ! empty( $field ) || ! empty( $entries ) || is_array( $entries ) ) {
			if ( isset( $entries[ $field ] ) ) {
				$value = $entries[ $field ];

				if ( is_array( $value ) && ! empty( $value['value'] ) ) {
					$value = $value['value'];
				} elseif ( ! is_array( $value ) ) {
					$value = maybe_unserialize( $value );
				}
			}
		}

		if ( is_array( $value ) && 1 == count( $value ) ) { // phpcs:ignore
			$value = implode( ' ', $value );
		}

		return $value;
	}

	/**
	 * Validate form values with conditions.
	 *
	 * @param    string $feed_value     Value to compare with entry value.
	 * @param    string $option_type    Filter conditon type.
	 * @param    string $form_value     Entry value .
	 *
	 * @since    1.0.0
	 * @return   bool
	 */
	public function is_value_allowed( $feed_value, $option_type = false, $form_value = false ) {

		if ( false == $option_type ) { // phpcs:ignore
			return;
		}

		$time   = current_time( 'timestamp' ); // phpcs:ignore
		$result = false;
		if ( ! is_bool( $form_value ) || true == $form_value ) {

			switch ( $option_type ) {

				case 'exact_match':
					if ( $feed_value == $form_value ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'no_exact_match':
					if ( $feed_value != $form_value ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'contains':
					if ( false != strpos( $form_value, $feed_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'not_contains':
					if ( false == strpos( $form_value, $feed_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'exist':
					if ( false != strpos( $feed_value, $form_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'not_exist':
					if ( false == strpos( $feed_value, $form_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'starts':
					if ( 0 == strpos( $form_value, $feed_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'not_starts':
					if ( 0 != strpos( $form_value, $feed_value ) ) { // phpcs:ignore 
						$result = true;
					}
					break;

				case 'ends':
					if ( strlen( $form_value ) == strpos( $form_value, $feed_value ) + strlen( $feed_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'not_ends':
					if ( strlen( $form_value ) != strpos( $form_value, $feed_value ) + strlen( $feed_value ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'less_than':
					if ( (float) $form_value < (float) $feed_value ) {
						$result = true;
					}
					break;

				case 'greater_than':
					if ( (float) $form_value > (float) $feed_value ) {
						$result = true;
					}
					break;

				case 'less_than_date':
					if ( strtotime( $form_value, $time ) < strtotime( $feed_value, $time ) ) {
						$result = true;
					}
					break;

				case 'greater_than_date':
					if ( strtotime( $form_value, $time ) > strtotime( $feed_value, $time ) ) {
						$result = true;
					}
					break;

				case 'equal_date':
					if ( strtotime( $form_value, $time ) == strtotime( $feed_value, $time ) ) { // phpcs:ignore
						$result = true;
					}
					break;

				case 'empty':
					if ( empty( $form_value ) ) {
						$result = true;
					}
					break;

				case 'not_empty':
					if ( ! empty( $form_value ) ) {
						$result = true;
					}
					break;

				default:
					$result = false;
					break;
			}
		}
			return $result;
	}

	/**
	 * Function to check weather the account we are trying to sync already exists on Sf.
	 *
	 * @param mixed $feed_id feed id.
	 * @param mixed $post_id post id.
	 * @return void
	 */
	public function check_if_account_already_exists_in_salesforce( $feed_id, $post_id, $obj_type = 'shop_order' ) {
		$primary_key = get_post_meta( $feed_id, 'primary_field', true );

		$salesforce  = Integration_With_Salesforce::get_integration_module( 'framework' );
		$record_type = $salesforce->get_feed( $feed_id );

		$mapping = $record_type[ $primary_key ];

		$field_type = ! empty( $mapping['field_type'] ) ?
		$mapping['field_type'] : 'standard_field';

		$feed_module        = Integration_With_Salesforce::get_integration_module( 'feed' );
		$primary_field_name = $feed_module->get_feed_data( $feed_id, 'primary_field', '', 'Account' );

		$obj_type = 'shop_order';
		switch ( $field_type ) {
			case 'standard_field':
				$field_format = ! empty( $mapping['field_value'] ) ?
				$mapping['field_value'] : '';
				$meta_key     = $salesforce->str_replace_first(
					$obj_type . '_',
					'',
					$field_format
				);

				$field_value = '';

				$field_value = $salesforce->get_prop_value(
					$obj_type,
					$meta_key,
					$post_id,
					wp_create_nonce( 'mwb-create-prop' )
				);
				break;
			case 'custom_value':
				$field_key = ! empty( $mapping['custom_value'] ) ?
				$mapping['custom_value'] : '';

				preg_match_all( '/{(.*?)}/', $field_key, $dynamic_strings );

				if ( ! empty( $dynamic_strings[1] ) ) {
					$dynamic_values = $dynamic_strings[1];
					foreach ( $dynamic_values as $key => $value ) {
						$prefix       = $obj_type . '_';
						$field_format = $salesforce->str_replace_first(
							$prefix,
							'',
							$value
						);

						$field_value = $salesforce->get_prop_value(
							$obj_type,
							$field_format,
							$post_id,
							wp_create_nonce( 'mwb-create-prop' )
						);

						$substr = '{' . $prefix . $field_format . '}';

						$field_key = str_replace(
							$substr,
							$field_value,
							$field_key
						);

						$field_value = $field_key;

					}
				} else {

					$field_value = ! empty( $mapping['custom_value'] ) ?
					$mapping['custom_value'] : '';
				}
				break;
		}

		if ( ! empty( $field_value ) ) {

			$query                 = sprintf( "SELECT Id,Name FROM Account WHERE %2s='%3s'", $primary_field_name, urlencode( $field_value ) );
			$crm_class             = 'Integration_With_' . Integration_With_Salesforce::get_current_crm_name() . '_Api';
			$crm_api_module        = $crm_class::get_instance();
			$salesforce_account_id = $crm_api_module->query_records( $query );

			if ( empty( $salesforce_account_id ) ) {
				$this->api_module->renew_access_token();
				$salesforce_account_id = $crm_api_module->query_records( $query );
			}

			if ( ! empty( $salesforce_account_id ) ) {
				$salesforce_acc_id = ! empty( $salesforce_account_id[0]['Id'] ) ? $salesforce_account_id[0]['Id'] : 0;
				if (Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled()) {
					$order = wc_get_order($post_id);
					if ( $order ) {

						$order->update_meta_data( 'mwb_salesforce_Account_id', $salesforce_acc_id);
						$order->save();
					} else {
						update_post_meta( $post_id, 'mwb_salesforce_Account_id', $salesforce_acc_id );
					}
				} else {
					update_post_meta( $post_id, 'mwb_salesforce_Account_id', $salesforce_acc_id );
				}
			}
		}
	}
	
	/**
	 * Function to return object for account and user if the event is for WP user.
	 *
	 * @param int $feed_id
	 * @return void
	 */
	public function get_woo_object_based_on_event( $feed_id ) {

		$event = $this->framework_instance->get_feed( $feed_id, 'feed_event');
		if ( 'user_update_create' == $event ) {
			$object_type = 'users';
		} else {
			$object_type = 'shop_order';
		}
		return $object_type;
	}
	// End of class.
}
