<?php
/**
 * Global functions of plugin.
 *
 * @link        https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/woo-includes
 */

/**
 * Insert entry into log table.
 *
 * @param array $log_data Array of entry data.
 */
function mwb_woo_salesforce_insert_log( $log_data = array() ) {
	$log_enable = Integration_With_Salesforce_Framework_Base::is_log_enable();
	if ( ! $log_enable || empty( $log_data ) ) {
		return false;
	}

	global $wpdb;
	$log_table = $wpdb->prefix . 'mwb_woo_salesforce_log';
	$wpdb->insert( $log_table, $log_data ); //phpcs:ignore
}

/**
 * May be create database table to store feed logs.
 *
 * @param  string $slug crm slug.
 */
function mwb_woo_salesforce_create_log_table( $slug ) {
	global $wpdb;
	$crm_log_table = $wpdb->prefix . 'mwb_woo_' . $slug . '_log';

	if ( $wpdb->get_var( $wpdb->prepare( 'show tables like %s', $crm_log_table ) ) == $crm_log_table ) {
		return;
	}

	$crm_object = $slug . '_object';
	$crm_id     = $slug . '_id';

	$query = "CREATE TABLE IF NOT EXISTS $crm_log_table (
        `id` int(11) NOT NULL AUTO_INCREMENT,
        `woo_object` varchar(255),
        `woo_id` int(11),
        `$crm_object` varchar(255),
        `$crm_id` varchar(255),
        `event` varchar(255),
        `request` text,
        `response` text,
        `time` int(11),
        PRIMARY KEY (`id`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";
		$wpdb->query( $query ); //phpcs:ignore
}

/**
 * Execute wpdb query.
 *
 * @param  string $query Query to be executed.
 */
function mwb_woo_salesforce_execute_db_query( $query ) {
	global $wpdb;
	$wpdb->query( $query ); // @codingStandardsIgnoreLine.
}

/**
 * Execute wpdb query.
 *
 * @param  int $timestamp delete timestamp.
 */
function mwb_woo_salesforce_clear_log_data( $timestamp ) {
	global $wpdb;
	$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';
	$query      = $wpdb->prepare( "DELETE FROM {$table_name} WHERE `time` < %d", $timestamp ); //phpcs:ignore
	$wpdb->query( $query ); //phpcs:ignore
}

/**
 * Get all options from database.
 *
 * @param  string $crm crm slug.
 */
function mwb_woo_salesforce_get_all_options( $crm ) {
	global $wpdb;
	$crm = '%' . $crm . '%';
	return $wpdb->get_results( $wpdb->prepare( "SELECT `option_name`, `option_id` FROM {$wpdb->prefix}options WHERE `option_name` LIKE %s LIMIT 50 ", $crm ), ARRAY_A );
}
/**
 * Get total count from log table.
 *
 * @param  string $search_query Search query.
 * @return integer Total count.
 */
function mwb_woo_salesforce_get_total_log_count( $search_query ) {
	global $wpdb;
	$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';
	$count      = $wpdb->get_col( $wpdb->prepare( "SELECT COUNT(*) as `total_count` FROM {$table_name} WHERE 1 {$search_query} " )); // @codingStandardsIgnoreLine.
	return $count;
}

/**
 * Get salesforce log data from database.
 *
 * @param  string|boolean $search_value Search value.
 * @param  integer        $limit        Max limit of data.
 * @param  integer        $offset       Offest to start.
 * @param  boolean        $all          Return all results.
 * @return array                        Array of data.
 */
function mwb_woo_salesforce_get_log_data( $search_value = false, $limit = 25, $offset = 0, $all = false ) {

	global $wpdb;
	$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';

	$log_data = array();

	if ( $all ) {

		$log_data = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} ORDER BY `id` DESC" ), ARRAY_A ); // @codingStandardsIgnoreLine.
		return $log_data;

	}

	if ( ! $search_value ) {

		$log_data    = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} ORDER BY `id` DESC LIMIT %d OFFSET %d ", $limit, $offset ), ARRAY_A ); // @codingStandardsIgnoreLine.
		return $log_data;

	}

	$log_data    = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE `woo_id` = %d LIMIT %d OFFSET %d ", $search_value, $limit, $offset ), ARRAY_A ); // @codingStandardsIgnoreLine.

	return $log_data;
}

/**
 * Get salesforce error data from database.
 *
 * @param  string|boolean $filter_value filer value.
 * @param  integer        $limit        Max limit of data.
 * @param  integer        $offset       Offest to start.
 * @param  boolean        $all          Return all results.
 * @return array                        Array of data.
 */
function mwb_woo_salesforce_get_error_data( $filter_value = false, $limit = 25, $offset = 0, $all = false ) {

	global $wpdb;
	$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';

	$log_data = array();

	if ( $all ) {

		$log_data = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} ORDER BY `id` DESC" ), ARRAY_A ); // @codingStandardsIgnoreLine.
		return $log_data;

	}

	if ( 'error' === $filter_value ) {
		$log_data = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE `salesforce_id` = '-' OR  `salesforce_id` = '' ORDER BY `id` DESC LIMIT %d OFFSET %d ", $limit, $offset ), ARRAY_A ); // @codingStandardsIgnoreLine.
	} else {
		$log_data = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} ORDER BY `id` DESC LIMIT %d OFFSET %d ", $limit, $offset ), ARRAY_A ); // @codingStandardsIgnoreLine.
	}
	return $log_data;
}

/**
 * Get query results from database.
 *
 * @param  string $query Query to be executed.
 * @return array         Result data.
 */
function mwb_woo_salesforce_get_query_results( $query ) {
	global $wpdb;
	$result = $wpdb->get_results( $query, ARRAY_A ); // @codingStandardsIgnoreLine.
	return $result;
}

/**
 * Get default field mapping json.
 *
 * @return string Json string data.
 */
function mwb_woo_salesforce_get_mapping_json_data() {

	return '{"Account":{"Name":{"field_type":"custom_value","custom_value":"WooCommerce_Account_{shop_order__billing_email}"},"Phone":{"field_type":"standard_field","field_value":"shop_order__billing_phone"}},"Contact":{"AccountId":{"field_type":"standard_field","field_value":""},"LastName":{"field_type":"standard_field","field_value":"shop_order__billing_last_name"},"FirstName":{"field_type":"standard_field","field_value":"shop_order__billing_first_name"},"OtherStreet":{"field_type":"custom_value","custom_value":"{shop_order__shipping_address_1} {shop_order__shipping_address_2}"},"OtherCity":{"field_type":"standard_field","field_value":"shop_order__shipping_city"},"OtherState":{"field_type":"standard_field","field_value":"shop_order__shipping_state"},"OtherPostalCode":{"field_type":"standard_field","field_value":"shop_order__shipping_postcode"},"OtherCountry":{"field_type":"standard_field","field_value":"shop_order__shipping_country"},"MailingStreet":{"field_type":"custom_value","custom_value":"{shop_order__billing_address_1} {shop_order__billing_address_2}"},"MailingCity":{"field_type":"standard_field","field_value":"shop_order__billing_city"},"MailingState":{"field_type":"standard_field","field_value":"shop_order__billing_state"},"MailingPostalCode":{"field_type":"standard_field","field_value":"shop_order__billing_postcode"},"MailingCountry":{"field_type":"standard_field","field_value":"shop_order__billing_country"},"Phone":{"field_type":"standard_field","field_value":"shop_order__billing_phone"},"MobilePhone":{"field_type":"standard_field","field_value":"shop_order__billing_phone"},"HomePhone":{"field_type":"standard_field","field_value":"shop_order__billing_phone"},"Email":{"field_type":"standard_field","field_value":"shop_order__billing_email"},"Description":{"field_type":"custom_value","custom_value":"WooCommerce Contact created for order {shop_order_order_id}"}},"Product2":{"Name":{"field_type":"standard_field","field_value":"product_product_name"},"Description":{"field_type":"standard_field","field_value":"product_product_short_description"},"DisplayUrl":{"field_type":"standard_field","field_value":"product_product_display_url"},"StockKeepingUnit":{"field_type":"standard_field","field_value":"product__sku"},"IsActive":{"field_type":"custom_value","custom_value":"true"},"ProductCode":{"field_type":"standard_field","field_value":"product_product_id"}},"PricebookEntry":{"Pricebook2Id":{"field_type":"standard_field","field_value":""},"Product2Id":{"field_type":"standard_field","field_value":""},"UnitPrice":{"field_type":"standard_field","field_value":"product__price"},"IsActive":{"field_type":"custom_value","custom_value":"true"}},"Opportunity":{"Name":{"field_type":"custom_value","custom_value":"WooCommerce_Opportunity_{shop_order_order_id}"},"AccountId":{"field_type":"standard_field","field_value":""},"Description":{"field_type":"custom_value","custom_value":"WooCommerce Opportunity created for {shop_order_order_id}"},"StageName":{"field_type":"standard_field","field_value":"","use_status_mapping":"yes"},"Amount":{"field_type":"standard_field","field_value":"shop_order__order_total"},"CloseDate":{"field_type":"standard_field","field_value":"shop_order_order_date"}},"Order":{"Name":{"field_type":"custom_value","custom_value":"WooCommerce_Order_{shop_order_order_id}"},"Pricebook2Id":{"field_type":"standard_field","field_value":""},"AccountId":{"field_type":"standard_field","field_value":""},"EffectiveDate":{"field_type":"standard_field","field_value":"shop_order_order_date"},"Description":{"field_type":"custom_value","custom_value":"WooCommerce Order created for {shop_order_order_id}"},"BillingStreet":{"field_type":"custom_value","custom_value":"{shop_order__billing_address_1} {shop_order__billing_address_2}"},"BillingCity":{"field_type":"standard_field","field_value":"shop_order__billing_city"},"BillingState":{"field_type":"standard_field","field_value":"shop_order__billing_state"},"BillingPostalCode":{"field_type":"standard_field","field_value":"shop_order__billing_postcode"},"BillingCountry":{"field_type":"standard_field","field_value":"shop_order__billing_country"},"ShippingStreet":{"field_type":"custom_value","custom_value":"{shop_order__shipping_address_1} {shop_order__shipping_address_2}"},"ShippingCity":{"field_type":"standard_field","field_value":"shop_order__shipping_city"},"ShippingState":{"field_type":"standard_field","field_value":"shop_order__shipping_state"},"ShippingPostalCode":{"field_type":"standard_field","field_value":"shop_order__shipping_postcode"},"ShippingCountry":{"field_type":"standard_field","field_value":"shop_order__shipping_country"},"BillToContactId":{"field_type":"standard_field","field_value":""},"ShipToContactId":{"field_type":"standard_field","field_value":""},"Status":{"field_type":"standard_field","field_value":"","use_status_mapping":"yes"}},"OrderItem":{"OrderId":{"field_type":"standard_field","field_value":""},"PricebookEntryId":{"field_type":"standard_field","field_value":""},"Quantity":{"field_type":"standard_field","field_value":"order_item_item_quantity"},"UnitPrice":{"field_type":"standard_field","field_value":"order_item_item_price"}}}';
}

/**
 * Check for all the options saved via current crm plugin.
 *
 * @return array
 */
function mwb_woo_crm_saved_feed_meta() {

	global $wpdb;
	$framework = Integration_With_Salesforce::get_integration_module( 'framework-base' );
	$all_feeds = $framework->get_all_feeds();

	$table_name = $wpdb->prefix . 'postmeta';

	$sql = "SELECT * FROM {$table_name} WHERE `meta_key` LIKE 'mwb_salesforce_allow_background_syncing' or ";

	foreach ( $all_feeds as $key => $feed_id ) {
		$record_type = $framework->get_feed( $feed_id, 'crm_object' );
		if ( $key === ( count( $all_feeds ) - 1 ) ) {
			$operator = '';
		} else {
			$operator = 'or';
		}
		$feed_key = '%mwb_salesforce_' . $record_type . '_id%';
		$sql     .= "`meta_key` LIKE '" . $feed_key . "' " . $operator;
	}

	return $wpdb->get_results( $sql, ARRAY_A );
}

/**
 * Get total count from log table.
 *
 * @param  string $search_query Search query.
 * @return integer Total count.
 */
function mwb_woo_salesforce_get_total_log_count_feed_filters( $search_by_error ) {
	global $wpdb;
	$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';
	$count      = $wpdb->get_col( $wpdb->prepare( "SELECT COUNT(*) as `total_count` FROM {$table_name} WHERE `event` = %s ",$search_by_error )); // @codingStandardsIgnoreLine.
	return $count;
}

/**
 * Get salesforce log data from database.
 *
 * @param  string|boolean $search_value Search value.
 * @param  integer        $limit        Max limit of data.
 * @param  integer        $offset       Offest to start.
 * @param  boolean        $all          Return all results.
 * @return array                        Array of data.
 */
function mwb_woo_salesforce_get_feed_log_data( $search_value = false, $limit = 25, $offset = 0 ) {

	global $wpdb;
	$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';

	$log_data = array();

	$log_data    = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE `event` = %s LIMIT %d OFFSET %d ", $search_value, $limit, $offset ), ARRAY_A ); // @codingStandardsIgnoreLine.

	return $log_data;
}
